#!/bin/bash
set -e
set -o pipefail

SCRIPTS_PATH="$1"
UTIL_FILE_PATH="$SCRIPTS_PATH/build/ci/pipelines_docker_buildx_util.sh"
source ${UTIL_FILE_PATH}

readonly INSTALLER_HOME="$( cd "$( dirname "${BASH_SOURCE[0]}" )" && pwd )"
readonly INSTALLER_BIN="$INSTALLER_HOME/bin"
readonly BP_VERSION_FILE="$INSTALLER_BIN/buildplaneVersion.txt"
readonly BUILD_PLANE_IMAGES="$INSTALLER_BIN/buildplaneImages.json"
readonly CP_VERSION_FILE="$INSTALLER_BIN/controlplaneVersion.txt"

__verify_inputs() {
  if [ ! -f "$BP_VERSION_FILE" ]; then
    __error_exit "Missing build plane version file: $BP_VERSION_FILE"
  fi

  if [ ! -f "$CP_VERSION_FILE" ]; then
    __error_exit "Missing control plane version file: $CP_VERSION_FILE"
  fi

  if [ -z $RT_REGISTRY ]; then
    __error_exit "Missing RT_REGISTRY env"
  fi

  if [ -z $MICRO_BUILD_IMAGE ]; then
    __error_exit "Missing MICRO_BUILD_IMAGE"
  fi

  if [ -z $BASE_RUN_IMAGE ]; then
    __error_exit "Missing BASE_RUN_IMAGE"
  fi

  export BP_VERSION=$(cat $BP_VERSION_FILE)
  export CP_VERSION=$(cat $CP_VERSION_FILE)

  if [ -z "$CP_VERSION" ]; then
    __error_exit "Missing control plane version"
  fi

  export BUILD_DIR="$INSTALLER_BIN/.build"
}

__prep_build_dir() {
  __log_message "Creating build directory: $BUILD_DIR"
  rm -rf $BUILD_DIR || true
  mkdir -p $BUILD_DIR
  rsync -az \
    --human-readable \
    --exclude ".git" \
    --exclude "pipelines.yml" \
    --exclude "node_modules" \
    --exclude "bin/*.tar.gz" \
    --exclude "bin/*.deb" \
    --exclude "bin/*.rpm" \
    $INSTALLER_HOME/ \
    $INSTALLER_BUILD_PATH
}

__generate_artifact_versions() {
  __log_message "Generating artifact versions"
  export INSTALLER_ARCHIVE="pipelines-$CP_VERSION.tar.gz"
  export DOCKER_TAG="$CP_VERSION"
  export RPM_FILENAME="pipelines-$CP_VERSION.rpm"
  export DEB_FILENAME="pipelines-$CP_VERSION.deb"
  export INSTALLER_BUILD_NAME="pipelines-$CP_VERSION"
  export INSTALLER_BUILD_PATH="$BUILD_DIR/$INSTALLER_BUILD_NAME"
}

__build_tar() {
  __log_message "Building installer tarfile"
  pushd $BUILD_DIR
  tar -czf $INSTALLER_ARCHIVE $INSTALLER_BUILD_NAME

  __log_message "Copying installer tarfile to project bin directory"
  cp -vr $INSTALLER_ARCHIVE $INSTALLER_BIN
  popd
}

__build_and_publish_image() {
  __log_message "Building Docker image for $IMG_NAME"

  __common_steps

  __log_message "Building docker image"
  local installer_image="$IMG_NAME:$IMG_TAG"

  docker buildx build --platform $JFROG_IMAGE_BUILD_ARCH \
          --build-arg RT_SERVER=entplus \
          --build-arg RT_URL=$RT_URL \
          --build-arg RT_API_KEY=$RT_API_KEY \
          --build-arg RT_USER=$RT_USER \
          --build-arg RT_DOCKER_REPO=$RT_DOCKER_REPO \
          --build-arg MICRO_BUILD_IMAGE=$MICRO_BUILD_IMAGE \
          --build-arg BASE_RUN_IMAGE=$BASE_RUN_IMAGE \
          -t $installer_image \
          --metadata-file=build-metadata --push . \
          || __error_exit "Failed building Docker image $installer_image"
  
  __publish_multiarch_image_buildinfo
}

__build_debian() {
  __log_message "Building debian helper image"
  pushd $INSTALLER_HOME/build/debian
  docker build \
    -t pipelines-debpkg-local:master \
    -f Dockerfile .
  popd

  __log_message "Bulding installer debian archive"
  docker run \
    --name debian-builder \
    --rm \
    -e PIPELINES_VERSION=$PIPELINES_VERSION \
    -e INSTALLER_BUILD_NAME=$INSTALLER_BUILD_NAME \
    -v $INSTALLER_BUILD_PATH:/opt/jfrog/pipelines \
    pipelines-debpkg-local:master

  __log_message "Copying installer debian to project bin directory"
  cp -vr $INSTALLER_BUILD_PATH/$DEB_FILENAME $INSTALLER_BIN
}

__build_rpm() {
  __log_message "Building rpm helper image"
  pushd $INSTALLER_HOME/build/rpm
  docker build \
    -t pipelines-rpmpkg-local:master \
    -f Dockerfile .

  __log_message "Copying tarfile to build rpm"
  cp -vr $INSTALLER_BIN/$INSTALLER_ARCHIVE $INSTALLER_BUILD_PATH/build/rpm

  __log_message "Building installer rpm archive"
  docker run \
    --name rpm-builder \
    --rm \
    -e PIPELINES_VERSION=$PIPELINES_VERSION \
    -e RPM_FILENAME=$RPM_FILENAME \
    -e INSTALLER_ARCHIVE=$INSTALLER_ARCHIVE \
    -v $INSTALLER_BUILD_PATH/build/rpm:/opt/jfrog/pipelines/build/rpm \
    pipelines-rpmpkg-local:master

  __log_message "Copying installer rpm to project bin directory"
  cp -vr $INSTALLER_BUILD_PATH/build/rpm/$RPM_FILENAME $INSTALLER_BIN

  popd
}

__update_bp_images() {
  # TODO
  # use build plane json to create sql script for injecting build plane images
  true
}

__cleanup() {
  __log_message "Cleaning up build directory"
  rm -rf $BUILD_DIR
}

main() {
  __log_message "Building installer"
  __verify_inputs
  __generate_artifact_versions
  __prep_build_dir
  __build_tar
  __build_and_publish_image
  __build_debian
  __build_rpm
  __update_bp_images
  __cleanup
}

main
