package main

import (
	"bytes"
	"crypto/tls"
	"encoding/json"
	"fmt"
	"io/ioutil"
	"net/http"
	"os"
	"os/signal"
	"syscall"
	"time"
	"strings"

	_ "go.uber.org/automaxprocs"
	"github.com/kelseyhightower/envconfig"
	"github.com/rs/zerolog"
	"github.com/rs/zerolog/log"
	api_core_v1 "k8s.io/api/core/v1"
	meta_v1 "k8s.io/apimachinery/pkg/apis/meta/v1"
	"k8s.io/client-go/kubernetes"
	"k8s.io/client-go/rest"
	"k8s.io/client-go/tools/clientcmd"
)

var (
	config     Config
	httpClient http.Client
	clientset kubernetes.Interface
	currentRetryDelayForCreateK8sSecret uint16 = 1
	currentRetryDelayForUpsertK8sSecret uint16 = 1
	currentRetryDelayForUpdateK8sSecret uint16 = 1
)

const (
	// VaultAPIHealthEndpoint is the endpoint is used to check the health status of Vault.
	VaultAPIHealthEndpoint = "/v1/sys/health"
	// VaultAPIInitEndpoint is the endpoint is used to initialize a new Vault.
	VaultAPIInitEndpoint = "/v1/sys/init"
	//VaultAPIUnsealEndpoint is the endpoint is used to unseal the Vault
	VaultAPIUnsealEndpoint = "/v1/sys/unseal"
	// VaultRootSecretName is the name of the Kubernetes secret used for the root token
	VaultRootSecretName = "root-vault-secret"
	// VaultUnlockSecretName is the name of the Kubernetes secret used for the unseal keys
	VaultUnlockSecretName = "unlock-vault-secret"
	// MaxRetryDelay is the maximum delay for backoff retry
	MaxRetryDelay = 512
)

// InitRequest holds a Vault init request.
type InitRequest struct {
	SecretShares      int `json:"secret_shares"`
	SecretThreshold   int `json:"secret_threshold"`
	StoredShares      int `json:"stored_shares"`
	RecoveryShares    int `json:"recovery_shares"`
	RecoveryThreshold int `json:"recovery_threshold"`
}

// InitResponse holds a Vault init response.
type InitResponse struct {
	Keys       []string `json:"keys"`
	KeysBase64 []string `json:"keys_base64"`
	RootToken  string   `json:"root_token"`
}

// UnsealRequest holds a Vault unseal request.
type UnsealRequest struct {
	Key   string `json:"key"`
	Reset bool   `json:"reset"`
}

// UnsealResponse holds a Vault unseal response.
type UnsealResponse struct {
	Sealed   bool `json:"sealed"`
	T        int  `json:"t"`
	N        int  `json:"n"`
	Progress int  `json:"progress"`
}

// Config holds the configuration of vailt-init, including default values (which you really want to override).
// The properties starting with "Vault" are documented on the Project Vault page https://www.vaultproject.io/api/system/init.html
type Config struct {
	VaultAddress           string        `split_words:"true" default:"http://localhost:8200"`
	VaultSecretShares      int           `split_words:"true" default:"5"`
	VaultSecretThreshold   int           `split_words:"true" default:"3"`
	VaultAutoUnseal        bool          `split_words:"true" default:"true"`
	VaultStoredShares      int           `split_words:"true" default:"1"`
	VaultRecoveryShares    int           `split_words:"true" default:"1"`
	VaultRecoveryThreshold int           `split_words:"true" default:"1"`
	VaultType              string        `split_words:"true" default:"oss"`
	VaultNamespace         string        `split_words:"true" default:"default"`
	CheckInterval          time.Duration `split_words:"true" default:"10s"`
	LogLevel               string        `default:"info"`
	ConsoleLog             bool          `default:"false"`
}

func main() {
	// Get configuration from environment variables
	err := envconfig.Process("", &config)
	if err != nil {
		panic(err.Error())
	}

	if os.Getenv("VAULT_NAMESPACE") != "" {
		config.VaultNamespace = os.Getenv("VAULT_NAMESPACE")
	}
	if os.Getenv("CHECK_INTERVAL") != "" {
		config.CheckInterval, err = time.ParseDuration(os.Getenv("CHECK_INTERVAL"))
	}

	// Initialize logger
	loglevel, err := zerolog.ParseLevel(config.LogLevel)
	if err != nil {
		panic(err.Error())
	}
	zerolog.SetGlobalLevel(loglevel)

	if config.ConsoleLog {
		log.Logger = log.Output(zerolog.ConsoleWriter{Out: os.Stderr})
	}

	log.Info().Msg("Starting the vault-init service")

	log.Debug().Msgf("Configuration set to: %+v", config)

	clientset = getKubernetesClient()

	httpClient = http.Client{
		Transport: &http.Transport{
			TLSClientConfig: &tls.Config{
				InsecureSkipVerify: true,
			},
		},
	}

	signalCh := make(chan os.Signal)
	signal.Notify(signalCh,
		syscall.SIGINT,
		syscall.SIGTERM,
		syscall.SIGKILL,
	)

	for {
		select {
		case <-signalCh:
			stop()
		default:
		}

		response, err := httpClient.Head(config.VaultAddress + VaultAPIHealthEndpoint)

		if response != nil && response.Body != nil {
			response.Body.Close()
		}

		if err != nil {
			log.Info().Msg(err.Error())
			time.Sleep(config.CheckInterval)
			continue
		}

		switch response.StatusCode {
		case 200:
			log.Info().Msg("Vault is initialized and unsealed")
		case 429:
			log.Info().Msg("Vault is unsealed and in standby mode")
		case 501:
			log.Info().Msg("Vault is not initialized")
			err := initialize()
			if err != nil {
				log.Error().Msgf("Error while initializing vault: %s", err.Error())
			}
			if config.VaultAutoUnseal {
				unseal()
			}
		case 503:
			log.Info().Msg("Vault is sealed")
			if config.VaultAutoUnseal {
				unseal()
			}
		default:
			log.Warn().Msgf("Vault is in an unknown state. Status code: %d", response.StatusCode)
		}

		log.Info().Msgf("Next check in %s", config.CheckInterval)

		select {
		case <-signalCh:
			stop()
		case <-time.After(config.CheckInterval):
		}
	}
}

func stop() {
	log.Info().Msg("Received shutdown signal")
	os.Exit(0)
}

// getKubernetesClient retrieves the cluster client from within or outside the cluster
// depending on where the app is ran
func getKubernetesClient() kubernetes.Interface {

	var client kubernetes.Interface

	clusterConfig, err := rest.InClusterConfig()
	if err == nil {
		client, err := kubernetes.NewForConfig(clusterConfig)
		if err == nil {
			return client
		}
	}

	if err != nil {
		log.Warn().Msgf("Error getting K8s config from in-cluster, will retry with external config: %v", err)
	}

	// construct the path to resolve to `~/.kube/config`
	kubeConfigPath := os.Getenv("HOME") + "/.kube/config"

	// create the config from the path
	config, err := clientcmd.BuildConfigFromFlags("", kubeConfigPath)
	if err != nil {
		// generate the client based off of the config
		log.Error().Msgf("Error getting K8s config from external config: %v", err)
		stop()
	}

	// generate the client based off of the config
	client, err = kubernetes.NewForConfig(config)
	if err != nil {
		log.Error().Msgf("Error creating K8s client: %v", err)
		stop()
	}

	log.Info().Msg("Successfully created K8s client")
	return client
}

func constructSecretObject(name string, data map[string][]byte) (secret *api_core_v1.Secret) {
	secret = &api_core_v1.Secret{
					Type: api_core_v1.SecretTypeOpaque,
					ObjectMeta: meta_v1.ObjectMeta{
						Name: name,
					},
					Data: data,
				}
	return
}

func initialize() error {
	log.Info().Msg("Initializing Vault")

	var initRequest InitRequest

	if config.VaultType == "oss" {
		initRequest = InitRequest{
			SecretShares:    config.VaultSecretShares,
			SecretThreshold: config.VaultSecretThreshold,
		}
	} else {
		initRequest = InitRequest{
			SecretShares:      config.VaultSecretShares,
			SecretThreshold:   config.VaultSecretThreshold,
			StoredShares:      config.VaultStoredShares,
			RecoveryShares:    config.VaultRecoveryShares,
			RecoveryThreshold: config.VaultRecoveryThreshold,
		}
	}

	log.Info().Msgf("Initializating request: %+v", initRequest)

	initRequestData, err := json.Marshal(&initRequest)
	if err != nil {
		log.Error().Msgf("Error while creating init request: %s", err.Error())
		return err
	}

	response, err := vaultRequest(VaultAPIInitEndpoint, http.MethodPut, initRequestData)
	if err != nil {
		log.Error().Msg(err.Error())
		return err
	}

	var initResponse InitResponse

	if err := json.Unmarshal(response, &initResponse); err != nil {
		log.Error().Msgf("Error unmarshaling Vault response: %s ", err.Error())
		return err
	}

	log.Info().Msg("Storing unseal keys and the root token as Kubernetes secrets")

	rootToken := make(map[string][]byte)
	rootToken["token"] = []byte(initResponse.RootToken)

	rootSecret := constructSecretObject(VaultRootSecretName,rootToken)

	upsertKubernetesSecret(config.VaultNamespace, rootSecret)

	log.Info().Msg("Successfully stored root token")

	unsealKeys := make(map[string][]byte)

	for idx, key := range initResponse.Keys {
		unsealKeys[fmt.Sprintf("key%d", idx)] = []byte(key)
	}

	unsealKeysSecret := constructSecretObject(VaultUnlockSecretName,unsealKeys)

	upsertKubernetesSecret(config.VaultNamespace, unsealKeysSecret)

	log.Info().Msg("Successfully stored unseal keys")

	log.Info().Msg("Initialization complete")

	return nil
}

func unseal() {
	log.Info().Msg("Unsealing Vault")

	secret, err := clientset.CoreV1().Secrets(config.VaultNamespace).Get(VaultUnlockSecretName, meta_v1.GetOptions{})
	if err != nil {
		log.Error().Msgf("Error reading unseal keys from Kubernetes: %s",err.Error())
		return
	}

	for i := 0; i < config.VaultSecretShares; i++ {
		done, err := unsealOne(string(secret.Data[fmt.Sprintf("key%d", i)]))
		if done {
			return
		}

		if err != nil {
			log.Error().Msg(err.Error())
			return
		}		
	}

	log.Error().Msgf("Failed to unseal Vault server using unlock-vault-secret keys, current threshold is : %d", config.VaultSecretThreshold)
}

func unsealOne(key string) (bool, error) {
	unsealRequest := UnsealRequest{
		Key: key,
	}

	unsealRequestData, err := json.Marshal(&unsealRequest)
	if err != nil {
		return false, err
	}

	response, err := vaultRequest(VaultAPIUnsealEndpoint, http.MethodPut, unsealRequestData)
	if err != nil {
		return false, err
	}

	var unsealResponse UnsealResponse
	if err := json.Unmarshal(response, &unsealResponse); err != nil {
		return false, err
	}

	if !unsealResponse.Sealed {
		return true, nil
	}

	return false, nil
}

// vaultRequest sends an HTTP request to Vault and replies back with the body payload and an error if exists
func vaultRequest(endpoint string, httpMethod string, payload []byte) ([]byte, error) {
	request, err := http.NewRequest(httpMethod, config.VaultAddress+endpoint, bytes.NewReader(payload))
	if err != nil {
		log.Error().Msgf("Error while creating HTTP request for [%s]: %s", endpoint, err.Error())
		return nil, err
	}

	response, err := httpClient.Do(request)
	if err != nil {
		log.Error().Msgf("Error sending request to Vault for [%s]: %s", endpoint, err.Error())
		return nil, err
	}
	defer response.Body.Close()

	body, err := ioutil.ReadAll(response.Body)
	if err != nil {
		log.Error().Msgf("Error reading response for [%s]: %s", endpoint, err.Error())
		return nil, err
	}

	if response.StatusCode != 200 && response.StatusCode != 204 {
		if response.StatusCode == 500 {
			log.Info().Msg("If tables are truncated from psql db, need to re-truncate it using the below command. Vault is in un-initialized state")
			log.Info().Msg("Run the following command in postgresdb : truncate vault_kv_store; truncate vault_ha_locks;")
			log.Info().Msg("Make sure to delete the vault pod and pipeline-service pod once re-truncate is done")
		}
		log.Error().Msgf("Received non-OK response from Vault: %d (%s)", response.StatusCode, string(body))
		return nil, err
	}

	return body, nil
}

func upsertKubernetesSecret(namespace string, secret *api_core_v1.Secret) {
	shouldRetry := true
	secretName := secret.ObjectMeta.Name
	namespaceSecretClient := clientset.CoreV1().Secrets(namespace)
	for shouldRetry == true {
		log.Info().Msgf("Checking if the secret %s already exists.", secretName)
		_, err := namespaceSecretClient.Get(secretName, meta_v1.GetOptions{})
		if err != nil {
			if strings.Contains(err.Error(), "not found") {
				log.Info().Msgf("Could not find secret %s, so will be creating one.", secretName)
				createKubernetesSecret(namespace, secret)
				log.Info().Msgf("Successfully created secret %s.", secretName)
				shouldRetry = false
			} else {
				currentRetryDelayForUpsertK8sSecret = computeRetryDelay(currentRetryDelayForUpsertK8sSecret)
				log.Info().Msgf("Failed to get secret %s with error: %s. Will retry after %d seconds.", secretName, err.Error(), currentRetryDelayForUpsertK8sSecret)
				time.Sleep(time.Duration(currentRetryDelayForUpsertK8sSecret) * time.Second)
			}
		} else {
			log.Info().Msgf("Secret %s already exists, so will be updating the secret with the new values.", secretName)
			updateKubernetesSecret(namespace, secret)
			log.Info().Msgf("Successfully updated secret %s.", secretName)
			shouldRetry = false
		}	
	}
}

func createKubernetesSecret(namespace string, secret *api_core_v1.Secret) {
	shouldRetry := true
	secretName := secret.ObjectMeta.Name
	namespaceSecretClient := clientset.CoreV1().Secrets(namespace)
	for shouldRetry == true {
		log.Info().Msgf("Attempting to create secret %s.", secretName)
		_, err := namespaceSecretClient.Create(secret)
		if err != nil {
			currentRetryDelayForCreateK8sSecret = computeRetryDelay(currentRetryDelayForCreateK8sSecret)
			log.Error().Msgf("Error creating Kubernetes secret %s with error %s. Will retry after %d seconds.", secretName, err.Error(), currentRetryDelayForCreateK8sSecret)
			time.Sleep(time.Duration(currentRetryDelayForCreateK8sSecret) * time.Second)
		} else {
			log.Info().Msgf("Successfully created secret %s.", secretName)
			shouldRetry = false
		}
	}
}

func updateKubernetesSecret(namespace string, secret *api_core_v1.Secret) {
	shouldRetry := true
	secretName := secret.ObjectMeta.Name
	namespaceSecretClient := clientset.CoreV1().Secrets(namespace)
	for shouldRetry == true {
		log.Info().Msgf("Attempting to update secret %s.", secretName)
		_, err := namespaceSecretClient.Update(secret)
		if err != nil {
			currentRetryDelayForUpdateK8sSecret = computeRetryDelay(currentRetryDelayForUpdateK8sSecret)
			log.Error().Msgf("Error updating Kubernetes secret %s : %s. Will retry after %d seconds.", secretName, err.Error(), currentRetryDelayForUpdateK8sSecret)
			time.Sleep(time.Duration(currentRetryDelayForUpdateK8sSecret) * time.Second)
		} else {
			log.Info().Msgf("Successfully updated secret %s.", secretName)
			shouldRetry = false
		}
	}
}

func computeRetryDelay(currentDelay uint16) uint16 {
	if currentDelay > MaxRetryDelay {
		return 1
	}

	return currentDelay * 2
}
