#!/bin/bash
set -e
set -o pipefail

__connect_to_artifactory() {
  log_proc_marker "Connecting with Artifactory"
  $INSTALLER_HOME/execUtil.sh "PingArtifactory"
  log_proc_msg "Successfully pinged Artifactory server"
}

__validate_licenses() {
  log_proc_marker "Validating licenses with artifactory"

  $INSTALLER_HOME/execUtil.sh "ValidateLicenses"
  local license_hash=$(echo $response \
    | jq -r '.licenseHash')
  if [ ! -z "$license_hash" ]; then
    log_proc_msg "Updating license hash"
    set_system_config "shared.artifactory.licenseId" "$license_hash"
  else
    log_proc_error "Missing license hash, exiting"
    exit 1
  fi
}

__get_artifactory_service_id() {
  log_proc_marker "Getting Artifactory service ID"
  exec_util "GetServiceID"

  local artifactory_service_id=$(echo $response \
    | jq -r '.serviceId')
  if [ ! -z "$artifactory_service_id" ]; then
    log_proc_msg "Updating Artifactory service ID"
    set_system_config "shared.artifactory.artifactoryServiceId" "$artifactory_service_id"
  fi
}

__get_proxy_config() {
  log_proc_marker "Getting proxy settings from artifactory"

  if [ ! -z "$PIPELINES_CORE_PROXY_HTTPPROXY" ] || [ ! -z "$PIPELINES_CORE_PROXY_HTTPSPROXY" ]; then
    log_proc_msg "Proxy already set from CLI, skipping artifactory proxy check"
    return
  fi

  $INSTALLER_HOME/execUtil.sh "GetProxies"
  local default_proxy=$(echo $response \
    | jq '.')

  if [ -z "$default_proxy" ] || [ "$default_proxy" == '{}' ]; then
    log_proc_msg "No proxy configured as global default, skipping proxy configuration"
  else
    local proxy_host=$(echo $default_proxy \
      | jq -r '.host')
    local proxy_port=$(echo $default_proxy \
      | jq -r '.port')
    local proxy_url="http://$proxy_host:$proxy_port"

    local proxy_username=$(echo $default_proxy \
      | jq -r '.username')
    if [ ! -z "$proxy_username" ]; then
      set_system_config "core.proxy.username" "$proxy_username"
    fi

    local proxy_password=$(echo $default_proxy \
      | jq -r '.password')
    if [ ! -z "$proxy_password" ]; then
      set_system_config "core.proxy.password" "$proxy_password"
    fi

    set_system_config "core.proxy.httpProxy" "$proxy_url"
    set_system_config "core.proxy.httpsProxy" "$proxy_url"
    if [ -z "$PIPELINES_CORE_PROXY_NOPROXY" ]; then
      set_system_config "core.proxy.noProxy" "localhost,127.0.0.1"
    fi
  fi
}

__get_mailserver_config() {
  log_proc_marker "Configuring mailserver"
  log_proc_msg "Getting mailserver settings from artifactory"

  $INSTALLER_HOME/execUtil.sh "GetMailserverConfig"
  local mailserver_config=$(echo $response \
    | jq '.')

  if [ ! -z "$mailserver_config" ];  then
    log_proc_msg "Found mailserver configuration"
    local mailserver_username=$(echo $mailserver_config \
      | jq -r '.username')
    local mailserver_password=$(echo $mailserver_config \
      | jq -r '.password')
    local mailserver_host=$(echo $mailserver_config \
      | jq -r '.host')
    local mailserver_port=$(echo $mailserver_config \
      | jq -r '.port')
    local mailserver_tls=$(echo $mailserver_config \
      | jq -r '.tls')
    local mailserver_ssl=$(echo $mailserver_config \
      | jq -r '.ssl')

    set_system_config "core.mailServer.username" "$mailserver_username"
    set_system_config "core.mailServer.password" "$mailserver_password"
    set_system_config "core.mailServer.host" "$mailserver_host"
    set_system_config "core.mailServer.port" "$mailserver_port"
    set_system_config "core.mailServer.tls" "$mailserver_tls"
    set_system_config "core.mailServer.ssl" "$mailserver_ssl"
  else
    log_proc_warn "No mailserver configuration found "
  fi
}

main() {
  log_proc_marker "Bootstrapping Artifactory"
  __connect_to_artifactory
  __validate_licenses
  __get_artifactory_service_id
  __get_proxy_config
  __get_mailserver_config
}

main
