#!/bin/bash
set -e
set -o pipefail

# Logger methods ##########################################
###########################################################
log_proc_marker() {
  local prompt="$@"
  echo ""
  echo "# $(date +"%T") #######################################"
  echo "# $prompt"
  echo "##################################################"
}

log_proc_msg() {
  local message="$@"
  echo "|___ $@"
}

log_proc_error() {
  local message="$1"
  local error="$2"
  local bold_red_text='\e[91m'
  local reset_text='\033[0m'

  echo -e "$bold_red_text|___ $message$reset_text"
  echo -e "     $error"
}

log_proc_success() {
  local message="$@"
  local bold_green_text='\e[32m'
  local reset_text='\033[0m'

  echo -e "$bold_green_text|___ $message$reset_text"
}

# End Logger methods ######################################
###########################################################

export LOGS_FILE="$LOGS_DIR/$SERVICE_NAME.log"
export KUBECONFIG="$CONFIG_DIR/kubernetes.conf"
export SERVICE_DEPLOY_CONFIG="$CONFIG_DIR/master/$SERVICE_NAME"_deployment.yml

## Write logs of this script to component specific file
exec &> >(tee -a "$LOGS_FILE")

__validate_service_configs() {
  log_proc_msg "Service $SERVICE_NAME configuration"
  log_proc_msg "SERVICE: $SERVICE_NAME"
  log_proc_msg "LOGS_FILE: $LOGS_FILE"
  log_proc_msg "SCRIPTS_DIR: $SCRIPTS_DIR"
  log_proc_msg "PIPELINES_CORE_RUNMODE: $PIPELINES_CORE_RUNMODE"
}

__cleanup_containers() {
  log_proc_msg "Stopping stale container for the service"
  if [[ "$SERVICE_NAME" == "api" ]]; then
    docker stop $SERVICE_NAME || true
  fi
  docker rm -f $SERVICE_NAME || true
}

__run_service() {
  log_proc_msg "Running service: $SERVICE_NAME"
  local run_output=$($RUN_COMMAND)
  if [ "$?" -ne 0 ] || [[ "$run_output" == *"Error"* ]]; then
    log_proc_error "Failed to start service"
    log_proc_error "Docker run returned: $run_output"
    exit 1
  else
    log_proc_msg "Successfully started service: $run_output"
  fi
}

main() {
  if [ -z "$SERVICE_NAME" ] || [ "$SERVICE_NAME" == "" ]; then
    log_proc_error "'SERVICE_NAME' env not present, exiting"
    exit 1
  else
    log_proc_marker "Booting service: $SERVICE_NAME"
    __validate_service_configs
    __cleanup_containers
    __run_service
  fi
}


main
