#!/bin/bash
set -e
set -o pipefail

export OBSERVABILITY_COMPOSE="$OBSERVABILITY_CONFIG_DIR/observability_compose.yml"

__generate_compose_config() {
  log_proc_msg "Generating observability compose config"

  local compose_template="$INSTALLER_SCRIPTS_DIR/configs/observability_compose.yml.template"
  cp -vr $compose_template $OBSERVABILITY_COMPOSE

  log_proc_msg "Updating component name"
  sed -i 's#{{COMPONENT}}#'$COMPONENT'#g' $OBSERVABILITY_COMPOSE

  log_proc_msg "Updating observability image"
  sed -i 's#{{PIPELINES_SHARED_OBSERVABILITY_IMAGE}}#'$PIPELINES_SHARED_OBSERVABILITY_IMAGE'#g' $OBSERVABILITY_COMPOSE  

  local url_parse_path="$INSTALLER_SCRIPTS_DIR/urlParse.sh"
  local protocol=$($url_parse_path --protocol $PIPELINES_SHARED_ARTIFACTORY_BASEURL)
  local hostname=$($url_parse_path --hostname $PIPELINES_SHARED_ARTIFACTORY_BASEURL)
  local port=$($url_parse_path --port $PIPELINES_SHARED_ARTIFACTORY_BASEURL)

  local jpu_host="$hostname"
  if [ ! -z "$port" ]; then
    jpu_host="$hostname:$port"
  fi

  log_proc_msg "Updating observability JPU Protocol"
  sed -i 's#{{JPUPROTOCOL}}#'$protocol'#g' $OBSERVABILITY_COMPOSE

  log_proc_msg "Updating observability JPU host"
  sed -i 's#{{JPUHOST}}#'$jpu_host'#g' $OBSERVABILITY_COMPOSE

  log_proc_msg "Updating volume directory"
  sed -i 's#{{PIPELINES_CONFIG_DIR}}#'$CONFIG_DIR'#g' $OBSERVABILITY_COMPOSE
  sed -i 's#{{PIPELINES_LOG_DIR}}#'$LOGS_DIR'#g' $OBSERVABILITY_COMPOSE

  log_proc_msg "Updating observability port"
  sed -i 's#{{PIPELINES_SHARED_OBSERVABILITY_ACCESSPORT}}#'$PIPELINES_SHARED_OBSERVABILITY_ACCESSPORT'#g' $OBSERVABILITY_COMPOSE 
  log_proc_msg "PIPELINES_SHARED_OBSERVABILITY_IMAGE:" $PIPELINES_SHARED_OBSERVABILITY_IMAGE

  log_proc_msg "Updating join key"
  sed -i 's#{{PIPELINES_SHARED_ROUTER_JOINKEY}}#'$PIPELINES_SHARED_ROUTER_JOINKEY'#g' $OBSERVABILITY_COMPOSE

}

__run_observability() {
  log_proc_msg "Running observability container"
  docker-compose \
    --file $OBSERVABILITY_COMPOSE down || true

  docker-compose \
    --file $OBSERVABILITY_COMPOSE \
    up \
    --detach $COMPONENT

  log_proc_msg "Observability container successfully running"
}

__check_observability() {

  local wait_time=5
  local max_retries=30
  local retry=0
  local observability_healthy=false

  while [ $observability_healthy == false ] && [[ $retry -lt $max_retries ]]; do
    _observability_get_health

    local retry_message=""
    if [[ $retry -gt 0 ]]; then
      retry_message="Retry $retry: "
    fi

    if [ $is_request_success != true ]; then
      log_proc_msg "${retry_message}Error checking observability health: $response"
      log_proc_msg "${retry_message}Status code: $response_status_code"
      retry=$(( retry + 1 ))
      if [[ $retry -lt $max_retries ]]; then
        log_proc_msg "${retry_message}Retrying in $wait_time seconds"
        sleep $wait_time
      fi
    elif [ ! -z "$response" ]; then
      local observability_state=$(echo $response | jq '.observability.state')
      if [[ "$observability_state" =~ "HEALTHY" ]]; then
        log_proc_msg "${retry_message}Observability status reported to be healthy"
        observability_healthy=true
      else
        log_proc_msg "${retry_message}Observability not healthy, reported $observability_state"
        retry=$(( retry + 1 ))
        if [[ $retry -lt $max_retries ]]; then
          log_proc_msg "${retry_message}Retrying in $wait_time seconds"
          sleep $wait_time
        fi
      fi
    else
      log_proc_msg "${retry_message}No response"
      retry=$(( retry + 1 ))
      if [[ $retry -lt $max_retries ]]; then
        log_proc_msg "${retry_message}Retrying in $wait_time seconds"
        sleep $wait_time
      fi
    fi
  done
  if [[ $retry -eq $max_retries ]]; then
    log_proc_error "Observability unhealthy. Maximum retries exceeded."
    exit 1
  fi
}

main() {
  log_proc_msg "Installing observability"
  __generate_compose_config
  __run_observability
}

main
