#!/bin/bash
set -e
set -o pipefail

export ROUTER_COMPOSE="$ROUTER_CONFIG_DIR/router_compose.yml"

__generate_compose_config() {
  log_proc_msg "Generating router compose config"

  local compose_template="$INSTALLER_SCRIPTS_DIR/configs/router_compose.yml.template"
  cp -vr $compose_template $ROUTER_COMPOSE

  log_proc_msg "Updating component name"
  sed -i 's#{{COMPONENT}}#'$COMPONENT'#g' $ROUTER_COMPOSE

  log_proc_msg "Updating router node id"
  sed -i 's#{{PIPELINES_CORE_ID}}#'$PIPELINES_CORE_ID'#g' $ROUTER_COMPOSE

  log_proc_msg "Updating router image"
  sed -i 's#{{PIPELINES_SHARED_ROUTER_IMAGE}}#'$PIPELINES_SHARED_ROUTER_IMAGE'#g' $ROUTER_COMPOSE

  log_proc_msg "Updating router port"
  sed -i 's#{{PIPELINES_SHARED_ROUTER_ACCESSPORT}}#'$PIPELINES_SHARED_ROUTER_ACCESSPORT'#g' $ROUTER_COMPOSE

  log_proc_msg "Updating router data port"
  sed -i 's#{{PIPELINES_SHARED_ROUTER_DATAPORT}}#'$PIPELINES_SHARED_ROUTER_DATAPORT'#g' $ROUTER_COMPOSE

  local url_parse_path="$INSTALLER_SCRIPTS_DIR/urlParse.sh"
  local protocol=$($url_parse_path --protocol $PIPELINES_SHARED_ARTIFACTORY_BASEURL)
  local hostname=$($url_parse_path --hostname $PIPELINES_SHARED_ARTIFACTORY_BASEURL)
  local port=$($url_parse_path --port $PIPELINES_SHARED_ARTIFACTORY_BASEURL)

  local jpu_host="$hostname"
  if [ ! -z "$port" ]; then
    jpu_host="$hostname:$port"
  fi

  log_proc_msg "Updating router JPU Protocol"
  sed -i 's#{{JPUPROTOCOL}}#'$protocol'#g' $ROUTER_COMPOSE

  log_proc_msg "Updating router JPU host"
  sed -i 's#{{JPUHOST}}#'$jpu_host'#g' $ROUTER_COMPOSE

  log_proc_msg "Updating node id"
  sed -i 's#{{PIPELINES_CORE_INSTALLERIP}}#'$PIPELINES_CORE_INSTALLERIP'#g' $ROUTER_COMPOSE

  log_proc_msg "Join key has length ${#PIPELINES_SHARED_ROUTER_JOINKEY}"
  log_proc_msg "Updating router join key"
  sed -i 's#{{PIPELINES_SHARED_ROUTER_JOINKEY}}#'$PIPELINES_SHARED_ROUTER_JOINKEY'#g' $ROUTER_COMPOSE

  log_proc_msg "Updating volume directory"
  sed -i 's#{{PIPELINES_CONFIG_DIR}}#'$CONFIG_DIR'#g' $ROUTER_COMPOSE
  sed -i 's#{{PIPELINES_LOG_DIR}}#'$LOGS_DIR'#g' $ROUTER_COMPOSE
}

__run_router() {
  log_proc_msg "Running router container"
  docker-compose \
    --file $ROUTER_COMPOSE down || true

  docker-compose \
    --file $ROUTER_COMPOSE \
    up \
    --detach $COMPONENT

  log_proc_msg "Router container successfully running"
}

__check_router() {
  sleep 5
  log_proc_msg "Checking router container status on: $PIPELINES_CORE_INSTALLERIP:$PIPELINES_SHARED_ROUTER_DATAPORT"
  check_service_connection "$PIPELINES_CORE_INSTALLERIP" "$PIPELINES_SHARED_ROUTER_DATAPORT" "$COMPONENT"

  local wait_time=5
  local max_retries=30
  local retry=0
  local router_healthy=false

  while [ $router_healthy == false ] && [[ $retry -lt $max_retries ]]; do
    _router_get_health

    local retry_message=""
    if [[ $retry -gt 0 ]]; then
      retry_message="Retry $retry: "
    fi

    if [ $is_request_success != true ]; then
      log_proc_msg "${retry_message}Error checking router health: $response"
      log_proc_msg "${retry_message}Status code: $response_status_code"
      retry=$(( retry + 1 ))
      if [[ $retry -lt $max_retries ]]; then
        log_proc_msg "${retry_message}Retrying in $wait_time seconds"
        sleep $wait_time
      fi
    elif [ ! -z "$response" ]; then
      local router_state=$(echo $response | jq '.router.state')
      if [[ "$router_state" =~ "HEALTHY" ]]; then
        log_proc_msg "${retry_message}Router status reported to be healthy"
        router_healthy=true
      else
        log_proc_msg "${retry_message}Router not healthy, reported $router_state"
        retry=$(( retry + 1 ))
        if [[ $retry -lt $max_retries ]]; then
          log_proc_msg "${retry_message}Retrying in $wait_time seconds"
          sleep $wait_time
        fi
      fi
    else
      log_proc_msg "${retry_message}No response"
      retry=$(( retry + 1 ))
      if [[ $retry -lt $max_retries ]]; then
        log_proc_msg "${retry_message}Retrying in $wait_time seconds"
        sleep $wait_time
      fi
    fi
  done
  if [[ $retry -eq $max_retries ]]; then
    log_proc_error "Router unhealthy. Maximum retries exceeded."
    exit 1
  fi
}

main() {
  log_proc_msg "Installing router"
  __generate_compose_config
  __run_router
  __check_router
}

main
