#!/bin/bash
set -e
set -o pipefail

__whitelist_proxy_envs() {
  log_proc_msg "Adding proxy env variables to whitelist"

  local proxy_whitelist_file="/etc/sudoers.d/pipelines-proxy-whitelist"

  if [ ! -w "$proxy_whitelist_file" ]; then
    log_proc_warn "User $PIPELINES_CORE_USER does not have permissions to edit proxy whitelist, skipping"
  else
    echo "Defaults        env_keep += \"http_proxy https_proxy no_proxy\"" \
      | tee "$proxy_whitelist_file"
  fi
}

__configure_http_proxy() {
  log_proc_msg "Configuring http proxy"

  if [ ! -w "$SYSTEM_ENV" ]; then
    log_proc_warn "Skipping http proxy configuration"
    log_proc_warn "User $PIPELINES_CORE_USER does not have permissions to edit $SYSTEM_ENV"
    return
  fi

  if [ ! -z "$PIPELINES_CORE_PROXY_HTTPPROXY" ]; then
    local http_proxy_env="export http_proxy=\"$PIPELINES_CORE_PROXY_HTTPPROXY\""
    local http_proxy_exists=$(grep "^$http_proxy_env$" $SYSTEM_ENV || echo "")
    if [ -z "$http_proxy_exists" ]; then
      log_proc_msg "Adding http_proxy to /etc/environment"
      sed -i '/^http_proxy=/d' $SYSTEM_ENV
      echo "$http_proxy_env" | tee -a $SYSTEM_ENV
    fi
  else
    log_proc_msg "No http proxy setting configured, unsetting the envs"
    local http_proxy_exists=$(grep "^export http_proxy=" $SYSTEM_ENV || echo "")
    if [ ! -z "$http_proxy_exists" ]; then
      log_proc_msg "Removing http_proxy from /etc/environment"
      unset http_proxy
      sed -i '/^export http_proxy=/d' $SYSTEM_ENV
    fi
  fi
}

__configure_https_proxy() {
  log_proc_msg "Configuring https proxy"

  if [ ! -w "$SYSTEM_ENV" ]; then
    log_proc_warn "Skipping https proxy configuration"
    log_proc_warn "User $PIPELINES_CORE_USER does not have permissions to edit $SYSTEM_ENV"
    return
  fi

  if [ ! -z "$PIPELINES_CORE_PROXY_HTTPSPROXY" ]; then
    local https_env="export https_proxy=\"$PIPELINES_CORE_PROXY_HTTPSPROXY\""
    local https_exists=$(grep "^$https_env$" $SYSTEM_ENV || echo "")
    if [ -z "$https_exists" ]; then
      log_proc_msg "Adding https_proxy to /etc/environment"
      sed -i '/^export https_proxy=/d' $SYSTEM_ENV
      echo "$https_env" | tee -a $SYSTEM_ENV
    fi
  else
    log_proc_msg "No https proxy setting configured, unsetting the envs"
    local https_exists=$(grep "^export https_proxy=" $SYSTEM_ENV || echo "")
    if [ ! -z "$https_exists" ]; then
      log_proc_msg "Removing https_proxy from /etc/environment"
      unset https_proxy
      sed -i '/^export https_proxy=/d' $SYSTEM_ENV
    fi
  fi
}

__configure_no_proxy() {
  log_proc_msg "Configuring no proxy"

  if [ ! -w "$SYSTEM_ENV" ]; then
    log_proc_warn "Skipping noProxy configuration"
    log_proc_warn "User $PIPELINES_CORE_USER does not have permissions to edit $SYSTEM_ENV"
    return
  fi

  if [ ! -z "$PIPELINES_CORE_PROXY_NOPROXY" ]; then
    local no_proxy_env="export no_proxy=\"$PIPELINES_CORE_PROXY_NOPROXY\""
    local no_proxy_exists=$(grep "^$no_proxy_env$" $SYSTEM_ENV || echo "")
    if [ -z "$no_proxy_exists" ]; then
      log_proc_msg "Adding no_proxy to /etc/environment"
      sed -i '/^export no_proxy=/d' $SYSTEM_ENV
      echo "$no_proxy_env" | tee -a $SYSTEM_ENV
    fi
  else
    log_proc_msg "No setting configured for no proxy, unsetting the envs"
    local no_proxy_exists=$(grep "^export no_proxy=" $SYSTEM_ENV || echo "")
    if [ ! -z "$no_proxy_exists" ]; then
      log_proc_msg "Removing no_proxy from /etc/environment"
      unset no_proxy
      sed -i '/^export no_proxy=/d' $SYSTEM_ENV
    fi
  fi
}

main() {
  log_proc_marker "Configuring proxy env variables"
  __whitelist_proxy_envs
  __configure_http_proxy
  __configure_https_proxy
  __configure_no_proxy
}

main
