'use strict';

var self = createBuildIntegration;
module.exports = self;

var async = require('async');
var _ = require('underscore');
var path = require('path');
var fs = require('fs');
var VaultAdapter = require('./lib/VaultAdapter.js');
var systemYamlHandler = require('./lib/systemYamlHandler');

function createBuildIntegration(args, cb) {
  var bag = {
    resBody: {},
    integrationCredsDir: 'buildplane-creds',
    integrationConfigDir: 'buildplane-config',
    skipConfiguration: false
  };
  bag.configPath = path.join(global.config.containerConfigDir,
    bag.integrationConfigDir);
  bag.credsPath = path.join(global.config.containerConfigDir,
    bag.integrationCredsDir);

  bag.who = util.format('createBuildIntegration|%s', self.name);
  logger.info(bag.who, 'Starting');

  async.series([
      _validateProvider.bind(null, bag),
      _validateCredentials.bind(null, bag),
      _getMasterIntegration.bind(null, bag),
      _createIntegration.bind(null, bag)
    ],
    function (err) {
      logger.info(bag.who, 'Completed');
      cb(err, bag.resBody);
    }
  );
}

function _validateProvider(bag, next) {
  var who = bag.who + '|' + _validateProvider.name;
  logger.verbose(who, 'Inside');

  var providerPath = path.join(bag.configPath, 'provider');
  if (!fs.existsSync(providerPath)) {
    bag.skipConfiguration = true;
    return next();
  }

  var provider = fs.readFileSync(providerPath, 'utf-8').trim();

  if (provider === 'aws') {
    bag.provider = 'aws';
    bag.name = 'aws_dynamic_nodes';
  } else if (provider === 'gcp') {
    bag.provider = 'gcp';
    bag.name = 'gcp_dynamic_nodes';
  } else if (provider === 'k8s') {
    bag.provider = 'k8s';
    bag.name = 'k8s_dynamic_nodes';
  } else {
    return next(
      new ActErr(who, ActErr.InvalidParam,
        'Please provide a valid provider for dynamic node integration.')
    );
  }

  systemYamlHandler.get(function(err, oldYaml) {
    if (err) return next(err);
    const newYaml = oldYaml;
    newYaml.runtime.internalBuildIntegrationName = bag.name;
    systemYamlHandler.put(newYaml, function(err, _data) {
      return next(err);
    });
  });
}

function _validateCredentials(bag, next) {
  if (bag.skipConfiguration) return next();

  var who = bag.who + '|' + _validateCredentials.name;
  logger.verbose(who, 'Inside');

  if (bag.provider === 'aws') {
    var accessKeyPath = path.join(bag.credsPath, 'accessKey');
    bag.accessKey = Buffer.from(
      fs.readFileSync(accessKeyPath, 'utf-8')).toString('utf-8').trim();

    var secretKeyPath = path.join(bag.credsPath, 'secretKey');
    bag.secretKey = Buffer.from(
      fs.readFileSync(secretKeyPath, 'utf-8')).toString('utf-8').trim();

    if (_.isEmpty(bag.accessKey))
      return next(
        new ActErr(who, ActErr.InvalidParam, 'Missing accessKey')
      );

    if (_.isEmpty(bag.secretKey))
      return next(
        new ActErr(who, ActErr.InvalidParam, 'Missing secretKey')
      );

    bag.masterIntegrationName = 'amazonKeys';
    bag.masterIntegrationType = 'generic';
  } else if (bag.provider === 'k8s') {
    var kubeconfigPath = path.join(bag.credsPath, 'kubeconfig');
    bag.kubeconfig = Buffer.from(
      fs.readFileSync(kubeconfigPath, 'utf-8')).toString('utf-8').trim();

    if (_.isEmpty(bag.kubeconfig))
      return next(
        new ActErr(who, ActErr.InvalidParam, 'Missing kubeconfig')
      );

    bag.masterIntegrationName = 'kubernetesConfig';
    bag.masterIntegrationType = 'generic';
  }

  return next();
}

function _getMasterIntegration(bag, next) {
  if (bag.skipConfiguration) return next();

  var who = bag.who + '|' + _getMasterIntegration.name;
  logger.verbose(who, 'Inside');

  var query = util.format(
    'SELECT * FROM "masterIntegrations" WHERE "name"=\'%s\';',
    bag.masterIntegrationName);

  global.config.client.query(query,
    function (err, integrations) {
      if (err)
        return next(
          new ActErr(who, ActErr.DBOperationFailed,
            'Failed to find masterIntegration with error: ' + util.inspect(err)
          )
        );

      if (!_.isEmpty(integrations.rows) && !_.isEmpty(integrations.rows[0])) {
        bag.masterIntegrationId = integrations.rows[0].id;
        return next();
      }

      return next(
        new ActErr(who, ActErr.DataNotFound, 'No masterIntegration found')
      );
    }
  );
}

function _createIntegration(bag, next) {
  if (bag.skipConfiguration) return next();

  var who = bag.who + '|' + _createIntegration.name;
  logger.verbose(who, 'Inside');

  var projectId = 1;
  var propertyBag = {};
  var permissions = {
    "allowAllPipelineSources": true
  };
  var createdByUsername = 'Service User';
  var updatedByUsername = 'Service User';
  var createdBy = 1;
  var updatedBy = 1;
  var createdAt = '2019-12-08';
  var updatedAt = '2019-12-08';
  var accessResourceNamePrefix = '';

  var insertStatement = util.format('INSERT INTO "projectIntegrations" ' +
    '("name", "projectId", "masterIntegrationId", "masterIntegrationType", "masterIntegrationName"' +
    ',"isInternal", "propertyBag", "permissions"' +
    ',"createdByUserName", "updatedByUserName", "createdBy", "updatedBy", "createdAt", "updatedAt", "accessResourceNamePrefix", "provider") ' +
    'values ' +
    '(\'%s\', \'%s\', \'%s\', \'%s\', \'%s\'' +
    ',\'%s\', \'%s\', \'%s\'' +
    ',\'%s\', \'%s\', \'%s\', \'%s\', \'%s\', \'%s\', \'%s\', \'vault\') on CONFLICT DO NOTHING;',
    bag.name, projectId, bag.masterIntegrationId, bag.masterIntegrationType, bag.masterIntegrationName,
    'true', JSON.stringify(propertyBag), JSON.stringify(permissions),
    createdByUsername, updatedByUsername, createdBy, updatedBy, createdAt, updatedAt, accessResourceNamePrefix);

  global.config.client.query(insertStatement,
    function (err) {
      if (err)
        return next(
          new ActErr(who, ActErr.DBOperationFailed,
            'Failed to create project integration with error: ' +
            util.inspect(err))
        );
      return next();
    }
  );
}

if (require.main === module)
  createBuildIntegration();
