'use strict';

const {series: asyncSeries} = require('async');
const _ = require('underscore');
const path = require('path');
const fs = require('fs');

const {
  DEFAULT_NODEPOOL_ARCH,
  DEFAULT_NODEPOOL_OS,
  INTEGRATION_CONFIG_DIR,
  MAC_OS_13_5,
  MAC_NODEPOOL_INSTANCE_TYPE,
  MAC_NODEPOOL_NAME_SUFFIX,
  MAC_NODEPOOL_INTERVAL_MINS,
  DEFAULT_NODEPOOL_PERMISSIONS,
  DEFAULT_NODEPOOL_TIMEOUT,
  DEFAULT_NODEPOOL_PROJECT_ID,
  DEFAULT_NODEPOOL_CREATED_AT,
  DEFAULT_NODEPOOL_UPDATED_AT,
  DEFAULT_NODEPOOL_MAX_DISK,
  DEFAULT_NODEPOOL_CACHE_ENABLED,
  DEFAULT_NODEPOOL_ON_DEMAND,
  DEFAULT_NODEPOOL_AWS_INTERVAL_MINS,
  DEFAULT_NODEPOOL_K8S_INTERVAL_MINS,
  DEFAULT_NODEPOOL_ACCESS_PREFIX
} = require('./constants.js');

// This is getting overloaded to create
// multiple nodePools. It should be renamed
// to 'createDynamicNodePools'
function createDynamicNodePool(args, cb) {
  const bag = {
    resBody: {},
    integrationConfigDir: INTEGRATION_CONFIG_DIR,
    skipConfiguration: false,
    defaultOS: DEFAULT_NODEPOOL_OS,
    defaultArchitecture: DEFAULT_NODEPOOL_ARCH,
  };

  bag.configPath = path.join(global.config.containerConfigDir,
    bag.integrationConfigDir);

  bag.who = util.format('createDynamicNodePool|%s', createDynamicNodePool.name);
  logger.info(bag.who, 'Starting');

  const macBag = {
    ...bag,
    defaultOS: MAC_OS_13_5,
  };

  // Don't do anything If any nodepools are already in the db
  _checkIfAnyNodePoolExists(bag, (err) => {
    if(bag.skipConfiguration) {
      return cb(err, bag.resBody);
    }
    _createDynamicNodePool(bag, (err) => {
      if (!global.config.isMacOSEnabled) {
        return cb(err, bag.resBody);
      }
      _createDynamicNodePool(macBag, cb);
    });
  });
}

function _createDynamicNodePool(bag, cb) {
  asyncSeries([
      _validateProvider.bind(null, bag),
      _validateConfigs.bind(null, bag),
      _checkIfNodePoolExists.bind(null, bag),
      _getMasterIntegration.bind(null, bag),
      _getBuildPlaneImage.bind(null, bag),
      _getIntegration.bind(null, bag),
      _createNodePool.bind(null, bag),
      _updateDefaultNodePool.bind(null, bag)
    ],
    function (err) {
      logger.info(bag.who, 'Completed');
      cb(err, bag.resBody);
    }
  );
}

function _checkIfAnyNodePoolExists(bag, next) {
  const who = bag.who + '|' + _checkIfNodePoolExists.name;
  logger.verbose(who, 'Inside');

  const query = 'SELECT id FROM "nodePools" LIMIT 1;';

  global.config.client.query(query,
    function (err, nodepools) {
      if (err)
        return next(
          new ActErr(who, ActErr.DBOperationFailed,
            'Failed to find nodePools with error: ' + util.inspect(err)
          )
        );

      if (!_.isEmpty(nodepools.rows) && !_.isEmpty(nodepools.rows[0])) {
        logger.debug(who, 'Skipping configuration. Node Pool exists');
        bag.skipConfiguration = true;
        return next();
      }

      return next();
    }
  );
}

function _checkIfNodePoolExists(bag, next) {
  const who = bag.who + '|' + _checkIfNodePoolExists.name;
  logger.verbose(who, 'Inside');

  const query =
    `SELECT id FROM "nodePools" WHERE name = '${bag.nodePoolName}' LIMIT 1;`;

  global.config.client.query(query,
    function (err, nodepools) {
      if (err)
        return next(
          new ActErr(who, ActErr.DBOperationFailed,
            'Failed to find nodePools with error: ' + util.inspect(err)
          )
        );

      if (!_.isEmpty(nodepools.rows) && !_.isEmpty(nodepools.rows[0])) {
        logger.debug(who, 'Skipping configuration. Node Pool exists');
        bag.skipConfiguration = true;
        return next();
      }

      return next();
    }
  );
}

function _validateProvider(bag, next) {
  if (bag.skipConfiguration) return next();

  const who = bag.who + '|' + _validateProvider.name;
  logger.verbose(who, 'Inside');

  const providerPath = path.join(bag.configPath, 'provider');
  if (!fs.existsSync(providerPath)) {
    logger.debug(`No provider found at ${providerPath}.
     Skipping configuration`);
    bag.skipConfiguration = true;
    return next();
  }

  const provider = fs.readFileSync(providerPath, 'utf-8').trim();

  if (provider === 'aws') {
    bag.provider = 'aws';
    bag.nodePoolName = 'aws_dynamic_node_pool';
    bag.integrationName = 'aws_dynamic_nodes';
    bag.masterIntegrationName = 'amazonKeys';
  } else if (provider === 'gcp') {
    bag.provider = 'gcp';
    bag.nodePoolName = 'gcp_dynamic_node_pool';
    bag.integrationName = 'gcp_dynamic_nodes';
    bag.masterIntegrationName = 'gcloudKey';
  } else if (provider === 'k8s') {
    bag.provider = 'k8s';
    bag.nodePoolName = 'k8s_dynamic_node_pool';
    bag.integrationName = 'k8s_dynamic_nodes';
    bag.masterIntegrationName = 'kubernetesConfig';
  } else {
    return next(
      new ActErr(who, ActErr.InvalidParam,
        'Please provide a valid provider for dynamic node integration.')
    );
  }

  return next();
}

function _validateConfigs(bag, next) {
  if (bag.skipConfiguration) return next();

  const who = bag.who + '|' + _validateConfigs.name;
  logger.verbose(who, 'Inside');

  const accountIdPath = path.join(bag.configPath, 'accountId');
  bag.accountId = Buffer.from(
    fs.readFileSync(accountIdPath, 'utf-8')).toString('utf-8').trim();

  const nodePoolPath = path.join(bag.configPath, 'nodePoolName');
  bag.nodePoolName = Buffer.from(
    fs.readFileSync(nodePoolPath, 'utf-8')).toString('utf-8').trim();

  const nodelimitPath = path.join(bag.configPath, 'nodelimit');
  bag.nodelimit = Buffer.from(
    fs.readFileSync(nodelimitPath, 'utf-8')).toString('utf-8').trim();

  if (bag.provider === 'aws') {
    const instanceTypePath = path.join(bag.configPath, 'instanceType');
    bag.instanceType = Buffer.from(
      fs.readFileSync(instanceTypePath, 'utf-8')).toString('utf-8').trim();

    const vpcIdPath = path.join(bag.configPath, 'vpcId');
    bag.vpcId = Buffer.from(
      fs.readFileSync(vpcIdPath, 'utf-8')).toString('utf-8').trim();

    const securityGroupIdPath = path.join(bag.configPath, 'securityGroupId');
    bag.securityGroupId = Buffer.from(
      fs.readFileSync(securityGroupIdPath, 'utf-8')).toString('utf-8').trim();

    const keyPairNamePath = path.join(bag.configPath, 'keyPairName');
    bag.keyPairName = Buffer.from(
      fs.readFileSync(keyPairNamePath, 'utf-8')).toString('utf-8').trim();

    const subnetId = path.join(bag.configPath, 'subnetId');
    bag.subnetId = Buffer.from(
      fs.readFileSync(subnetId, 'utf-8')).toString('utf-8').trim();

    const regionPath = path.join(bag.configPath, 'region');
    bag.region = Buffer.from(
      fs.readFileSync(regionPath, 'utf-8')).toString('utf-8').trim();

  } else if (bag.provider === 'gcp') {
    //remove following when GCP dynamic nodes are implemented
    return next(
      new ActErr(who, ActErr.InvalidParam,
        'GCP dynamic nodes not implemented yet.')
    );
  } else if (bag.provider === 'k8s') {
    const cpuPath = path.join(bag.configPath, 'cpu');
    bag.cpu = Buffer.from(
      fs.readFileSync(cpuPath, 'utf-8')).toString('utf-8').trim();

    const memoryPath = path.join(bag.configPath, 'memory');
    bag.memory = Buffer.from(
      fs.readFileSync(memoryPath, 'utf-8')).toString('utf-8').trim();

    const storageClassPath = path.join(bag.configPath, 'storageClass');
    bag.storageClass = Buffer.from(
      fs.readFileSync(storageClassPath, 'utf-8')).toString('utf-8').trim();

    const namespacePath = path.join(bag.configPath, 'namespace');
    bag.namespace = Buffer.from(
      fs.readFileSync(namespacePath, 'utf-8')).toString('utf-8').trim();

    const labelsPath = path.join(bag.configPath, 'labels');
    bag.labels = Buffer.from(
      fs.readFileSync(labelsPath, 'utf-8')).toString('utf-8').trim();

  }

  if (bag.defaultOS === MAC_OS_13_5) {
    bag.nodePoolName += MAC_NODEPOOL_NAME_SUFFIX;
    bag.instanceType = MAC_NODEPOOL_INSTANCE_TYPE;
    bag.nodeIdleIntervalInMins = MAC_NODEPOOL_INTERVAL_MINS;
    bag.nodelimit = global.config.macOSNodeLimit;
  }

  return next();
}

function _getMasterIntegration(bag, next) {
  if (bag.skipConfiguration) return next();
  if (bag.masterIntegrationName === 'kubernetesConfig') return next();

  const who = bag.who + '|' + _getMasterIntegration.name;
  logger.verbose(who, 'Inside');

  const query = util.format(
    'SELECT * FROM "masterIntegrations" WHERE "name"=\'%s\';',
    bag.masterIntegrationName);

  global.config.client.query(query,
    function (err, integrations) {
      if (err)
        return next(
          new ActErr(who, ActErr.DBOperationFailed,
            'Failed to find masterIntegration with error: ' + util.inspect(err)
          )
        );

      if (!_.isEmpty(integrations.rows) && !_.isEmpty(integrations.rows[0])) {
        bag.masterIntegrationId = integrations.rows[0].id;
        return next();
      }

      return next(
        new ActErr(who, ActErr.DataNotFound, 'No masterIntegration found')
      );
    }
  );
}

function _getBuildPlaneImage(bag, next) {
  if (bag.skipConfiguration) return next();
  if (bag.masterIntegrationName === 'kubernetesConfig') return next();

  const who = bag.who + '|' + _getBuildPlaneImage.name;
  logger.debug(who, 'Inside');

  const query = util.format('SELECT * FROM "buildPlaneImages" WHERE ' +
    '"isAvailable"=true AND "masterIntegrationId"=%s', bag.masterIntegrationId);

  global.config.client.query(query,
    function (err, buildPlaneImages) {
      if (err)
        return next(
          new ActErr(who, ActErr.DBOperationFailed,
            'Failed to find buildPlaneImages roles with error: ' +
            util.inspect(err))
        );

      if (!_.isEmpty(buildPlaneImages.rows) &&
        !_.isEmpty(buildPlaneImages.rows[0])) {
        _.each(buildPlaneImages.rows,
          function (buildPlaneImage) {
            if (bag.region === buildPlaneImage.region &&
              bag.defaultOS === buildPlaneImage.operatingSystem &&
              bag.defaultArchitecture === buildPlaneImage.architecture) {
              if (global.config.isSaas && bag.defaultOS.startsWith('MacOS')) {
                if (buildPlaneImage.name.endsWith(
                  global.config.saasEnvironment))
                  bag.buildPlaneImage = buildPlaneImage;
              } else {
                bag.buildPlaneImage = buildPlaneImage;
              }
            }
          }
        );


        if (_.isEmpty(bag.buildPlaneImage)) {
          bag.skipConfiguration = true;
          logger.debug(who,
            `Skipping configuration. No buildPlaneImages exist for the region:
            ${bag.region}`);
        }
      } else {
        bag.skipConfiguration = true;
        logger.debug(who, 'Skipping configuration. No buildPlaneImages exist');
      }
      return next();
    }
  );
}

function _getIntegration(bag, next) {
  if (bag.skipConfiguration) return next();

  const who = bag.who + '|' + _getIntegration.name;
  logger.verbose(who, 'Inside');

  let query = '';
  if (bag.provider === 'aws' || bag.provider === 'k8s')
    query = util.format('SELECT * FROM "projectIntegrations" WHERE ' +
      '"name"=\'%s\';', bag.integrationName);

  global.config.client.query(query,
    function (err, integrations) {
      if (err)
        return next(
          new ActErr(who, ActErr.DBOperationFailed,
            'Failed to find new integration with error: ' + util.inspect(err))
        );

      if (!_.isEmpty(integrations.rows) &&
        !_.isEmpty(integrations.rows[0])) {
        bag.projectIntegration = integrations.rows[0];

        return next();
      }

      return next(
        new ActErr(who, ActErr.DataNotFound,
          'No project integration found')
      );
    }
  );
}

function _createNodePool(bag, next) {
  if (bag.skipConfiguration) return next();

  /* jshint maxcomplexity: 20 */
  const who = bag.who + '|' + _createNodePool.name;
  logger.debug(who, 'Inside');

  let providerMetadataPropertyBag;
  let architecture;
  let operatingSystem;
  let buildPlaneImageId;
  let nodeIdleIntervalInMins;
  if (bag.provider === 'aws') {
    architecture = bag.buildPlaneImage.architecture;
    operatingSystem = bag.buildPlaneImage.operatingSystem;
    buildPlaneImageId = bag.buildPlaneImage.id;
    nodeIdleIntervalInMins = bag.nodeIdleIntervalInMins || DEFAULT_NODEPOOL_AWS_INTERVAL_MINS;
    providerMetadataPropertyBag = {
      instanceSize: bag.instanceType,
      securityGroupId: bag.securityGroupId,
      keyPairName: bag.keyPairName,
      vpcId: bag.vpcId,
      subnetId: bag.subnetId,
      region: bag.region
    };
  } else if (bag.provider === 'k8s') {
    architecture = bag.defaultArchitecture;
    operatingSystem = bag.defaultOS;
    buildPlaneImageId = null;
    nodeIdleIntervalInMins = DEFAULT_NODEPOOL_K8S_INTERVAL_MINS;
    providerMetadataPropertyBag = {
      storageClass: bag.storageClass
    };

    if (!_.isEmpty(bag.labels)) {
      const labels = {};
      bag.labels = bag.labels.replace(/\s/g, '').split(',');
      _.each(bag.labels,
        function (label) {
          const splitLabel = label.split(':');
          labels[splitLabel[0]] = splitLabel[1];
        }
      );
      providerMetadataPropertyBag.labels = labels;
    }

    if (!_.isEmpty(bag.cpu))
      providerMetadataPropertyBag.cpu = bag.cpu;

    if (!_.isEmpty(bag.memory))
      providerMetadataPropertyBag.memory = bag.memory;

    if (!_.isEmpty(bag.namespace))
      providerMetadataPropertyBag.namespace = bag.namespace;
  }

  const np = {
    name: bag.nodePoolName,
    projectId: DEFAULT_NODEPOOL_PROJECT_ID,
    isOnDemand: DEFAULT_NODEPOOL_ON_DEMAND,
    architecture: architecture,
    operatingSystem: operatingSystem,
    buildPlaneImageId: buildPlaneImageId,
    projectIntegrationId: bag.projectIntegration.id,
    permissions: DEFAULT_NODEPOOL_PERMISSIONS,
    timeoutMS: DEFAULT_NODEPOOL_TIMEOUT,
    isCacheEnabled: DEFAULT_NODEPOOL_CACHE_ENABLED,
    nodeIdleIntervalInMins: bag.nodeIdleIntervalInMins ||
      nodeIdleIntervalInMins,
    maxDiskUsagePercentage: DEFAULT_NODEPOOL_MAX_DISK,
    numberOfNodes: bag.nodelimit,
    providerMetadataPropertyBag: providerMetadataPropertyBag,
    createdAt: DEFAULT_NODEPOOL_CREATED_AT,
    updatedAt: DEFAULT_NODEPOOL_UPDATED_AT,
    accessResourceNamePrefix: DEFAULT_NODEPOOL_ACCESS_PREFIX
  };

  const insertStatement = formatCreatNodePoolQuery(
    np.name, np.projectId, np.isOnDemand, np.architecture, np.operatingSystem,
    np.buildPlaneImageId, np.projectIntegrationId,
    np.permissions, np.timeoutMS, np.isCacheEnabled,
    np.nodeIdleIntervalInMins, np.maxDiskUsagePercentage, np.numberOfNodes,
    np.providerMetadataPropertyBag, np.createdAt, np.updatedAt, np.accessResourceNamePrefix);

  global.config.client.query(insertStatement,
    function (err) {
      if (err)
        return next(
          new ActErr(who, ActErr.DBOperationFailed,
            'Failed to create project integration with error: ' +
            util.inspect(err))
        );
      return next();
    }
  );
}

function _updateDefaultNodePool(bag, next) {
  if (bag.skipConfiguration) return next();

  /* jshint maxcomplexity: 20 */
  const who = bag.who + '|' + _updateDefaultNodePool.name;
  logger.debug(who, 'Inside');

  const configPropertyBag = {
    defaultNodePoolId: 1
  };

  const updateStatement = util.format('UPDATE projects SET ' +
    '"configPropertyBag" = \'%s\' WHERE id = 1;',
    JSON.stringify(configPropertyBag));

  global.config.client.query(updateStatement,
    function (err) {
      if (err)
        return next(
          new ActErr(who, ActErr.DBOperationFailed,
            'Failed to update default node pool with error: ' +
            util.inspect(err))
        );
      return next();
    }
  );
}

function formatCreatNodePoolQuery (
  /* jshint -W072 */
  name,
  projectId,
  isOnDemand,
  arch,
  os,
  buildPlaneImageId,
  projectIntegrationId,
  permissions,
  timeoutMS,
  isCacheEnabled,
  nodeIdleIntervalInMins,
  maxDiskUsagePercentage,
  numberOfNodes,
  providerMetadataPropertyBag,
  createdAt,
  updatedAt,
  accessResourceNamePrefix
) {
  return util.format('INSERT INTO "nodePools" ' +
    '("name", "projectId", "isOnDemand", "architecture", "operatingSystem", "buildPlaneImageId"' +
    ', "projectIntegrationId", "permissions", "timeoutMS", "isCacheEnabled"' +
    ',"nodeIdleIntervalInMins", "maxDiskUsagePercentage", "numberOfNodes"' +
    ', "providerMetadataPropertyBag", "createdAt", "updatedAt", "accessResourceNamePrefix") ' +
    'values ' +
    '(\'%s\', %s, \'%s\', \'%s\', \'%s\', %s' +
    ', %s, \'%s\', %s, \'%s\'' +
    ', %s, %s, %s' +
    ',\'%s\', \'%s\', \'%s\', \'%s\') on CONFLICT DO NOTHING;',
    name, projectId, isOnDemand, arch, os, buildPlaneImageId,
    projectIntegrationId, JSON.stringify(permissions), timeoutMS, isCacheEnabled,
    nodeIdleIntervalInMins, maxDiskUsagePercentage, numberOfNodes,
    JSON.stringify(providerMetadataPropertyBag), createdAt, updatedAt, accessResourceNamePrefix);
}

// set default export
const moduleExports = module.exports = createDynamicNodePool;

// export others for unit testing
moduleExports.formatCreatNodePoolQuery = formatCreatNodePoolQuery;
