'use strict';

var self = createProject;
module.exports = self;

var async = require('async');
var _ = require('underscore');
var passwordGenerator = require('password-generator');
var sshKeygen = require('@jfrog/ssh-keygen');
var tmp = require('tmp');
var systemYamlHandler = require('./lib/systemYamlHandler.js');

function createProject(args, cb) {
  var bag = {
    resBody: {},
    internalUrl: global.config.internalUrl,
    name: 'default',
    sourceId: 'default_source_id',
    configPropertyBag: {},
  };

  bag.who = util.format('projects|%s', self.name);
  logger.info(bag.who, 'Starting');

  async.series([
      _generateSSHKeys.bind(null, bag),
      _getExistingProject.bind(null, bag),
      _getExistingProvider.bind(null, bag),
      _getConfigs.bind(null, bag),
      _post.bind(null, bag),
      _getProject.bind(null, bag),
      _getSystemCodes.bind(null, bag),
      _getProjectIdentity.bind(null, bag),
      _postIdentity.bind(null, bag),
      _getNewProjectIdentity.bind(null, bag),
      _getIdentityRoles.bind(null, bag),
      _postIdentityRole.bind(null, bag),
      _getProjectPermissions.bind(null, bag),
      _postProjectPermissions.bind(null, bag)
    ],
    function (err) {
      logger.info(bag.who, 'Completed');
      cb(err, bag.resBody);
    }
  );
}

function _generateSSHKeys(bag, next) {
  var who = bag.who + '|' + _generateSSHKeys.name;
  logger.verbose(who, 'Inside');

  tmp.file(
    function (err, path) {
      if (err)
        return next(
          new ActErr(who, ActErr.InternalServer,
            'Failed to create tmp file for new project SSH keys.', err)
        );

      sshKeygen({
          location: path,
          comment: bag.callerId,
          password: '',
          read: true
        },
        function (err, out) {
          if (err)
            return next(
              new ActErr(who, ActErr.InternalServer,
                'Error generating SSH keys for project.', err)
            );

          bag.sshKeys = out;
          return next();
        }
      );
    }
  );
}

function _getExistingProject(bag, next) {
  var who = bag.who + '|' + _getExistingProject.name;
  logger.verbose(who, 'Inside');

  var query = util.format('SELECT * FROM "projects" WHERE ' +
    '"name"=\'%s\'', bag.name);

  global.config.client.query(query,
    function (err, projects) {
      if (err)
        return next(
          new ActErr(who, ActErr.DBOperationFailed,
            'Failed to find projects with error: ' + util.inspect(err))
        );

      if (!_.isEmpty(projects.rows) && !_.isEmpty(projects.rows[0]))
        bag.resBody = projects.rows[0];

      return next();
    }
  );
}

function _getExistingProvider(bag, next) {
  if (!_.isEmpty(bag.resBody)) return next();

  var who = bag.who + '|' + _getExistingProvider.name;
  logger.verbose(who, 'Inside');

  var query = util.format('SELECT * FROM "providers" WHERE ' +
    '"url"=\'%s\'', bag.internalUrl);

  global.config.client.query(query,
    function (err, providers) {
      if (err)
        return next(
          new ActErr(who, ActErr.DBOperationFailed,
            'Failed to find providers with error: ' + util.inspect(err))
        );

      if (!_.isEmpty(providers.rows) &&
        !_.isEmpty(providers.rows[0])) {
        bag.provider = providers.rows[0];
        return next();
      } else {
        return next(
          new ActErr(who, ActErr.ParamNotFound,
            'No providers exist. Cannot create project without a provider')
        );
      }

      return next();
    }
  );
}

function _getConfigs(bag, next) {
  var who = bag.who + '|' + _getConfigs.name;
  logger.verbose(who, 'Inside');

  systemYamlHandler.get(
    function (err, config) {
      if (err) {
        return next(
          new ActErr(who, ActErr.DataNotFound,
            'Failed to get ' + bag.component, err)
        );
      }

      if (!config.shared || !config.shared.msg)
        return next(
          new ActErr(who, ActErr.DataNotFound, 'Failed to get shared.msg')
        );

      bag.msgIP = config.shared.msg.ip;
      bag.msgPort = config.shared.msg.port;
      bag.msgAdminPort = config.shared.msg.adminPort;
      bag.msgUser = config.shared.msg.username;
      bag.msgPassword = global.config.msgPassword;
      bag.msgUIUser = config.shared.msg.ui.username;
      bag.msgUIPassword = global.config.msgUIPassword;

      return next();
    }
  );
}

function _post(bag, next) {
  if (!_.isEmpty(bag.resBody)) return next();

  var who = bag.who + '|' + _post.name;
  logger.verbose(who, 'Inside');

  var systemPropertyBag = {
      nodeUserName: passwordGenerator(16, true),
      nodePassword: passwordGenerator(16, false),
      sshPublicKey: bag.sshKeys.pubKey,
      sshPrivateKey: bag.sshKeys.key
  };

  var insertStatement = util.format('INSERT INTO "projects" ' +
    '("sourceId", "name", "providerId", "systemPropertyBag", ' +
    '"configPropertyBag", "createdAt", "updatedAt") ' +
    'values ' +
    '(\'%s\', \'%s\', \'%s\', \'%s\', \'%s\', \'2019-05-01\', \'2019-05-01\')',
    bag.sourceId, bag.name, bag.provider.id,
    JSON.stringify(systemPropertyBag),
    JSON.stringify(bag.configPropertyBag));

  global.config.client.query(insertStatement,
    function (err) {
      if (err)
        return next(
          new ActErr(who, ActErr.DBOperationFailed,
            'Failed to create project with error: ' + util.inspect(err))
        );
      bag.systemPropertyBag = systemPropertyBag;
      return next();
    }
  );
}

function _getProject(bag, next) {
  if (!_.isEmpty(bag.resBody)) return next();

  var who = bag.who + '|' + _getProject.name;
  logger.verbose(who, 'Inside');

  var query = util.format('SELECT * FROM "projects" WHERE ' +
    '"name"=\'%s\'', bag.name);

  global.config.client.query(query,
    function (err, projects) {
      if (err)
        return next(
          new ActErr(who, ActErr.DBOperationFailed,
            'Failed to find project with error: ' + util.inspect(err))
        );

      if (!_.isEmpty(projects.rows) && !_.isEmpty(projects.rows[0])) {
        bag.resBody = projects.rows[0];
        return next();
      }

      return next(
        new ActErr(who, ActErr.DataNotFound,
          'No project found')
      );
    }
  );
}

function _getSystemCodes(bag, next) {
  var who = bag.who + '|' + _getSystemCodes.name;
  logger.verbose(who, 'Inside');

  var query = 'SELECT "name","group","code" FROM "systemCodes" WHERE '+
    '"group"=\'roles\'';

  global.config.client.query(query,
    function (err, result) {
      if (err)
        return next(
          new ActErr(who, ActErr.DBOperationFailed,
            'Failed to fetch roleCodes', err)
        );

      if (_.isEmpty(result.rows) || !result.rows[0].code)
        return next(
          new ActErr(who, ActErr.OperationFailed,
            'No system codes found')
        );

      bag.projectIdentitySystemCode = _.findWhere(result.rows,
        {name: 'project'});
      bag.memberRoleCode = _.findWhere(result.rows,
        {name: 'member'});
      bag.adminRoleCode = _.findWhere(result.rows,
        {name: 'admin'});
      bag.projectUserRoleCode = _.findWhere(result.rows,
        {name: 'projectUser'});

      if (_.isEmpty(bag.projectIdentitySystemCode))
        return next(
          new ActErr(who, ActErr.OperationFailed,
            'No project role system code found')
        );

      if (_.isEmpty(bag.memberRoleCode))
        return next(
          new ActErr(who, ActErr.OperationFailed,
            'No member role system code found')
        );

      if (_.isEmpty(bag.adminRoleCode))
        return next(
          new ActErr(who, ActErr.OperationFailed,
            'No admin role system code found')
        );

      if (_.isEmpty(bag.projectUserRoleCode))
        return next(
          new ActErr(who, ActErr.OperationFailed,
            'No projectUser role system code found')
        );

      return next();
    }
  );
}

function _getProjectIdentity(bag, next) {
  var who = bag.who + '|' + _getProjectIdentity.name;
  logger.verbose(who, 'Inside');

  var query = util.format('SELECT * FROM "identities" WHERE ' +
    '"name"=\'%s\' AND "type"=%s',
    bag.resBody.sourceId, bag.projectIdentitySystemCode.code);

  global.config.client.query(query,
    function (err, identities) {
      if (err)
        return next(
          new ActErr(who, ActErr.DBOperationFailed,
            'Failed to find identity with error: ' + util.inspect(err))
        );

      if (!_.isEmpty(identities.rows) &&
        !_.isEmpty(identities.rows[0]))
        bag.projectIdentity = identities.rows[0];

      return next();
    }
  );
}

function _postIdentity(bag, next) {
  if (bag.projectIdentity) return next();

  var who = bag.who + '|' + _postIdentity.name;
  logger.verbose(who, 'Inside');

  var insertStatement = util.format('INSERT INTO "identities" ' +
    '("name", "type", "createdAt", "updatedAt") values ' +
    '(\'%s\', %s, \'2019-05-01\', \'2019-05-01\')',
    bag.resBody.sourceId, bag.projectIdentitySystemCode.code);

  global.config.client.query(insertStatement,
    function (err) {
      if (err)
        return next(
          new ActErr(who, ActErr.DBOperationFailed,
            'Failed to create project identity with error: ' +
            util.inspect(err))
        );
      return next();
    }
  );
}

function _getNewProjectIdentity(bag, next) {
  var who = bag.who + '|' + _getNewProjectIdentity.name;
  logger.verbose(who, 'Inside');

  var query = util.format('SELECT * FROM "identities" WHERE ' +
    '"name"=\'%s\' AND "type"=%s',
    bag.resBody.sourceId, bag.projectIdentitySystemCode.code);

  global.config.client.query(query,
    function (err, identities) {
      if (err)
        return next(
          new ActErr(who, ActErr.DBOperationFailed,
            'Failed to find new identity with error: ' + util.inspect(err))
        );

      if (!_.isEmpty(identities.rows) &&
        !_.isEmpty(identities.rows[0])) {
        bag.projectIdentity = identities.rows[0];
        return next();
      }

      return next(
        new ActErr(who, ActErr.DataNotFound,
          'No project identity found')
      );
    }
  );
}

function _getIdentityRoles(bag, next) {
  var who = bag.who + '|' + _getIdentityRoles.name;
  logger.verbose(who, 'Inside');

  var query = util.format('SELECT * FROM "identityRoles" WHERE ' +
    '"identityId"=%s AND "roleCode"=%s',
    bag.projectIdentity.id, bag.projectUserRoleCode.code);

  global.config.client.query(query,
    function (err, roles) {
      if (err)
        return next(
          new ActErr(who, ActErr.DBOperationFailed,
            'Failed to find identity roles with error: ' + util.inspect(err))
        );

      if (!_.isEmpty(roles.rows) && !_.isEmpty(roles.rows[0]))
        bag.projectIdentityRole = roles.rows[0];

      return next();
    }
  );
}

function _postIdentityRole(bag, next) {
  if (bag.projectIdentityRole) return next();

  var who = bag.who + '|' + _postIdentityRole.name;
  logger.verbose(who, 'Inside');

  var insertStatement = util.format('INSERT INTO "identityRoles" ' +
    '("identityId", "roleCode", "createdAt", "updatedAt") values ' +
    '(\'%s\', %s, \'2019-05-01\', \'2019-05-01\')',
    bag.projectIdentity.id, bag.projectUserRoleCode.code);

  global.config.client.query(insertStatement,
    function (err) {
      if (err)
        return next(
          new ActErr(who, ActErr.DBOperationFailed,
            'Failed to create project identity role with error: ' +
            util.inspect(err))
        );

      return next();
    }
  );
}

function _getProjectPermissions(bag, next) {
  var who = bag.who + '|' + _getProjectPermissions.name;
  logger.verbose(who, 'Inside');

  var query = util.format('SELECT "roleCode" FROM "projectPermissions" ' +
    'WHERE "identityId"=%s AND "projectId"=%s',
    bag.projectIdentity.id, bag.resBody.id);

  global.config.client.query(query,
    function (err, projectPermissions) {
      if (err)
        return next(
          new ActErr(who, ActErr.DBOperationFailed,
            'Failed to find permissions with error: ' + util.inspect(err))
        );

      if (!_.isEmpty(projectPermissions.rows) &&
        !_.isEmpty(projectPermissions.rows[0]))
        bag.projectPermissions = projectPermissions.rows;

      return next();
    }
  );
}

function _postProjectPermissions(bag, next) {
  var who = bag.who + '|' + _postProjectPermissions.name;
  logger.verbose(who, 'Inside');

  var roleCodes = [bag.memberRoleCode.code, bag.adminRoleCode.code];

  async.eachSeries(roleCodes,
    function (roleCode, done) {
      var existingPermissions = _.findWhere(bag.projectPermissions,
        {roleCode: roleCode});
      if (existingPermissions)
        return done();

      var insertStatement = util.format('INSERT INTO "projectPermissions" ' +
        '("identityId", "projectId", "roleCode", "createdAt", "updatedAt") ' +
        'values ' +
        '(%s, %s, %s, \'2019-05-01\', \'2019-05-01\')',
        bag.projectIdentity.id, bag.resBody.id, roleCode);

      global.config.client.query(insertStatement,
        function (err) {
          if (err)
            return done(
              new ActErr(who, ActErr.DBOperationFailed,
                'Failed to create project permission with error: ' +
                util.inspect(err))
            );
          return done();
        }
      );
    },
    function (err) {
      return next(err);
    }
  );
}

if (require.main === module)
  createProject();
