'use strict';

var self = createService;
module.exports = self;

var async = require('async');
var path = require('path');
var _ = require('underscore');
var spawn = require('child_process').spawn;
var fs = require('fs');

var apiConfig = require('./apiConfig.js');
var wwwConfig = require('./wwwConfig.js');
var frontendConfig = require('./frontendConfig.js');
var microConfig = require('./microConfig.js');
var nodePoolServiceConfig = require('./nodePoolServiceConfig');
var stepServiceConfig = require('./stepServiceConfig');
var runServiceConfig = require('./runServiceConfig');
var logServiceConfig = require('./logServiceConfig');
var analyticsServiceConfig = require('./analyticsServiceConfig');

function createService(name, cb) {
  var bag = {
    name: name,
    resBody: {},
    params: {},
    tmpScript: '/tmp/service.sh',
  };

  bag.who = util.format('services|%s', self.name);
  logger.info(bag.who, 'Starting');

  async.series([
    _checkInputParams.bind(null, bag),
    _generateServiceConfig.bind(null, bag),
    _generateInitializeEnvs.bind(null, bag),
    _generateScript.bind(null, bag),
    _writeScriptToFile.bind(null, bag),
    _bootService.bind(null, bag)
  ],
    function (err) {
      logger.info(bag.who, 'Completed');
      cb(err);
    }
  );
}

function _checkInputParams(bag, next) {
  var who = bag.who + '|' + _checkInputParams.name;
  logger.verbose(who, 'Inside');
  if (_.isEmpty(bag.name))
    return next(
      new ActErr(who, ActErr.ParamNotFound,
        'Data not found :name')
    );

  return next();
}

function _generateServiceConfig(bag, next) {
  var who = bag.who + '|' + _generateServiceConfig.name;
  logger.verbose(who, 'Inside');
  var configGenerator = null;
  if (bag.name === 'api')
    configGenerator = apiConfig;
  else if (bag.name === 'www')
    configGenerator = wwwConfig;
  else if (bag.name === 'frontend')
    configGenerator = frontendConfig;
  else if (bag.name === 'nodepoolservice')
    configGenerator = nodePoolServiceConfig;
  else if (bag.name === 'stepservice')
    configGenerator = stepServiceConfig;
  else if (bag.name === 'runservice')
    configGenerator = runServiceConfig;
  else if (bag.name === 'logservice')
    configGenerator = logServiceConfig;
  else if (bag.name === 'analyticsservice')
    configGenerator = analyticsServiceConfig;
  else 
    configGenerator = microConfig;
  var params = {
    name: bag.name,
  };

  if (!configGenerator)
    return next(
      new ActErr(who, ActErr.ParamNotFound,
        'No config generator for service: ' + bag.name)
    );

  configGenerator(params,
    function (err, runCommand) {
      if (err)
        return next(
          new ActErr(who, ActErr.OperationFailed,
            'Failed to generate config for service: ' + params.name +
            ' with error: ' + err.message)
        );

      bag.runCommand = runCommand;
      logger.debug('Service run command: ' + util.inspect(runCommand));
      return next();
    }
  );

}

function _generateInitializeEnvs(bag, next) {
  var who = bag.who + '|' + _generateInitializeEnvs.name;
  logger.verbose(who, 'Inside');

  bag.scriptEnvs = {
    'LOGS_DIR': global.config.logsDir,
    'SCRIPTS_DIR': global.config.scriptsDir,
    'CONFIG_DIR': global.config.containerConfigDir,
    'SERVICE_NAME': bag.name,
    'RUN_COMMAND': bag.runCommand,
    'PIPELINES_CORE_RUNMODE': global.config.runMode,
    'PATH': process.env.PATH
  };

  return next();
}

function _generateScript(bag, next) {
  var who = bag.who + '|' + _generateScript.name;
  logger.verbose(who, 'Inside');

  var script = '';
  //attach header
  var filePath = path.join(global.config.scriptsDir, 'bootService.sh');
  script = script.concat(__applyTemplate(filePath, bag.params));

  bag.script = script;
  return next();
}

function _writeScriptToFile(bag, next) {
  var who = bag.who + '|' + _writeScriptToFile.name;
  logger.debug(who, 'Inside');

  fs.writeFile(bag.tmpScript,
    bag.script,
    function (err) {
      if (err) {
        var msg = util.format('%s, Failed with err:%s', who, err);
        return next(
          new ActErr(
            who, ActErr.OperationFailed, msg)
        );
      }
      fs.chmodSync(bag.tmpScript, '755');
      return next();
    }
  );
}

function _bootService(bag, next) {
  var who = bag.who + '|' + _bootService.name;
  logger.verbose(who, 'Inside');

  var exec = spawn('/bin/bash',
    ['-c', bag.script],
    {
      env: bag.scriptEnvs
    }
  );

  exec.stdout.on('data',
    function (data) {
      logger.debug(who, data.toString());
    }
  );

  exec.stderr.on('data',
    function (data) {
      logger.error(who, data.toString());
    }
  );

  exec.on('close',
    function (exitCode) {
      if (exitCode > 0)
        return next(
          new ActErr(who, ActErr.OperationFailed,
            'Script returned code: ' + exitCode)
        );
      return next();
    }
  );
}

//local function to apply vars to template
function __applyTemplate(filePath, dataObj) {
  var fileContent = fs.readFileSync(filePath).toString();
  var template = _.template(fileContent);

  return template({ obj: dataObj });
}

if (require.main === module)
  createService();
