'use strict';

var self = createUser;
module.exports = self;

var async = require('async');
var _ = require('underscore');
var path = require('path');
var fs = require('fs-extra');
var spawn = require('child_process').spawn;
var crypto = require('crypto');

function createUser(args, cb) {
  var bag = {
    resBody: {},
  };

  bag.who = util.format('db|%s', self.name);
  logger.info(bag.who, 'Starting');

  async.series([
      _checkInputParams.bind(null, bag),
      _templateServiceUserIdentityFile.bind(null, bag),
      _upsertServiceUserIdentity.bind(null, bag)
    ],
    function (err) {
      logger.info(bag.who, 'Completed');
      cb(err, bag.resBody);
    }
  );
}

function _checkInputParams(bag, next) {
  var who = bag.who + '|' + _checkInputParams.name;
  logger.verbose(who, 'Inside');

  return next();
}

function _templateServiceUserIdentityFile(bag, next) {
  var who = bag.who + '|' + _templateServiceUserIdentityFile.name;
  logger.verbose(who, 'Inside');

  var filePath = util.format('%s/db/service_user_identity.sql',
    global.config.containerConfigDir);
  var templatePath =
    util.format('%s/configs/service_user_identity.sql.template',
      global.config.scriptsDir);

  var installerAuthToken = global.config.installerAuthToken ;
  var hash = crypto.createHash('sha256');
  hash.update(installerAuthToken);

  var expiresAt = new Date('2063-4-5').getTime();
  expiresAt = new Date(expiresAt).toISOString();
  var dataObj = {
    expiresAt: expiresAt,
    name: 'serviceUser',
    installerAuthToken: installerAuthToken,
    installerTokenHash: hash.digest('hex')
  };

  var script = {
    tmpScriptFilename: filePath,
    script: __applyTemplate(templatePath, dataObj)
  };

  _writeScriptToFile(script,
    function (err) {
      if (err)
        return next(
          new ActErr(who, ActErr.OperationFailed, 'Failed to write script', err)
        );

      return next();
    }
  );
}

function _upsertServiceUserIdentity(bag, next) {
  var who = bag.who + '|' + _upsertServiceUserIdentity.name;
  logger.verbose(who, 'Inside');

  let scriptEnvs = {
    'LOGS_DIR': global.config.logsDir,
    'CONFIG_DIR': global.config.containerConfigDir,
    'SCRIPTS_DIR': global.config.scriptsDir,
    'DBCONNECTIONSTRING': global.config.dbConnectionString,
    'PATH': '/usr/bin'
  };

  if (process.env.ONEBOX === 'true') {
    /* jshint -W069 */
    scriptEnvs['PATH'] = process.env.PATH;
    /* jshint +W069 */
  }

  _copyAndRunScript({
      who: who,
      params: {},
      script: '',
      scriptPath: 'createServiceUserIdentity.sh',
      tmpScriptFilename: '/tmp/serviceUserIdentity.sh',
      scriptEnvs: scriptEnvs
    },
    function (err) {
      return next(err);
    }
  );
}

function _copyAndRunScript(seriesBag, next) {
  var who = seriesBag.who + '|' + _copyAndRunScript.name;
  logger.verbose(who, 'Inside');

  async.series([
      _generateScript.bind(null, seriesBag),
      _writeScriptToFile.bind(null, seriesBag),
      _runScript.bind(null, seriesBag)
    ],
    function (err) {
      fs.remove(seriesBag.tmpScriptFilename,
        function (error) {
          if (error)
            logger.warn(
              util.format('%s, Failed to remove %s %s', who, path, error)
            );
        }
      );
      return next(err);
    }
  );
}

function _generateScript(seriesBag, next) {
  var who = seriesBag.who + '|' + _generateScript.name;
  logger.verbose(who, 'Inside');

  var script = '';
  var filePath = path.join(global.config.scriptsDir, seriesBag.scriptPath);
  script = script.concat(__applyTemplate(filePath, seriesBag.params));

  seriesBag.script = script;
  return next();
}

function _writeScriptToFile(seriesBag, next) {
  var who = seriesBag.who + '|' + _writeScriptToFile.name;
  logger.debug(who, 'Inside');

  fs.writeFile(seriesBag.tmpScriptFilename, seriesBag.script,
    function (err) {
      if (err) {
        var msg = util.format('%s, Failed with err:%s', who, err);
        return next(
          new ActErr(who, ActErr.OperationFailed, msg)
        );
      }
      fs.chmodSync(seriesBag.tmpScriptFilename, '755');
      return next();
    }
  );
}

function _runScript(seriesBag, next) {
  var who = seriesBag.who + '|' + _runScript.name;
  logger.verbose(who, 'Inside');

  seriesBag.scriptEnvs = seriesBag.scriptEnvs || {};
  if (process.env.http_proxy)
    seriesBag.scriptEnvs.http_proxy = process.env.http_proxy;

  if (process.env.https_proxy)
    seriesBag.scriptEnvs.https_proxy = process.env.https_proxy;

  if (process.env.no_proxy)
    seriesBag.scriptEnvs.no_proxy = process.env.no_proxy;

  var exec = spawn('/bin/bash',
    ['-c', seriesBag.tmpScriptFilename],
    {
      cwd: '/',
      env: seriesBag.scriptEnvs
    }
  );

  exec.stdout.on('data',
    function (data)  {
      logger.debug(who, data.toString());
    }
  );

  exec.stderr.on('data',
    function (data)  {
      logger.error(who, data.toString());
    }
  );

  exec.on('close',
    function (exitCode)  {
      if (exitCode > 0)
        return next(
          new ActErr(who, ActErr.OperationFailed,
            'Script: ' + seriesBag.tmpScriptFilename + ' returned code: ' +
          exitCode)
        );
      return next();
    }
  );
}

//local function to apply vars to template
function __applyTemplate(filePath, dataObj) {
  var fileContent = fs.readFileSync(filePath).toString();
  var template = _.template(fileContent);

  return template({obj: dataObj});
}

if (require.main === module)
  createUser();
