'use strict';

process.title = 'pipelines.migration';

const _ = require('underscore');
const util = require('util');
const pg = require('pg');
const fs = require('fs');
const path = require('path');

//bigint in postgres gets parsed as a string by default
//https://github.com/sequelize/sequelize/issues/2383#issuecomment-58006083
pg.defaults.parseInt8 = true;

require('./lib/ActErr.js');

process.on('uncaughtException',
  function (err) {
    util.log('uncaughtException thrown:', err);
    process.exit(1);
  }
);

var folderParent = process.argv[2];
var migrationOrderDir = process.argv[3].split(' ');
var installerBin = process.argv[4] || "";

if (_.isEmpty(folderParent) || _.isEmpty(migrationOrderDir)) {
  throw new Error('Migration folderParent and migrationOrderDir cannot be empty');
}

var bag = {
    folderParent: folderParent,
    migrationOrderDir: migrationOrderDir,
    installerBin: installerBin
};

_runMigration(bag);

async function _runMigration(bag){
    try {
        bag.client = await _createClient();
        for (var i in bag.migrationOrderDir) {
          util.log(`### Starting ${bag.migrationOrderDir[i]} migration ###`);
          bag.filePath = `${bag.folderParent}/${bag.migrationOrderDir[i]}/order.txt`;
          await _processFilesAndExecute(bag);
        }
    } finally {
      await bag.client.end();
    }
}

async function _createClient(){
  var dbConnectionString = `${process.env.DBCONNECTIONSTRING}`;

  var connectionDetails = {};
  if (dbConnectionString &&
    (dbConnectionString.includes('sslmode=require') ||
     dbConnectionString.includes('ssl=true') ||
     dbConnectionString.includes('sslmode=prefer'))
    ){  
    connectionDetails = {
      connectionString: dbConnectionString,
      ssl: {
        require: true,
        rejectUnauthorized: false // disable SSL validation for self-signed certificates
      }    
    };
  } else {
    connectionDetails = {
      connectionString: dbConnectionString,
    };
  }

  var client = new pg.Client(connectionDetails);

  await client.connect(
      function (err) {
        return err;
      }
    );
  return client;
}

async function _processFilesAndExecute(bag){
    var orderFileOrDir = bag.filePath;
    var migrationFileList = [];
    var migrationFolder = ""

    if (fs.existsSync(orderFileOrDir) && fs.lstatSync(orderFileOrDir).isDirectory()) {
        migrationFolder = path.resolve(__dirname, orderFileOrDir).trim();
        migrationFileList   = fs.readdirSync(orderFileOrDir);
    } else if (fs.existsSync(orderFileOrDir)) {
        migrationFolder   = path.resolve(__dirname, orderFileOrDir + "/..");
        migrationFileList = fs.readFileSync(orderFileOrDir).toString().split("\n");            
    }

    for(var i in migrationFileList) {
        var migrationFile = migrationFileList[i];
        bag.migrationFile = migrationFile;
        var isJSPromisified = false;

        if (!migrationFile){
          continue;
        }

        if (migrationFile.startsWith("promise_")){
          isJSPromisified = true;
          migrationFile = migrationFile.replace("promise_","");
        }

        var sqlFile = bag.sqlFile = migrationFolder + "/" + _addExtension(migrationFile,"sql");
        var jsFile  = bag.jsFile  = migrationFolder + "/" + _addExtension(migrationFile,"js");

        if (sqlFile.endsWith(".sql") && fs.existsSync(sqlFile)) {
            await _callPg(bag, sqlFile);
        } else if(jsFile.endsWith(".js") && fs.existsSync(jsFile)) {
            try {
              util.log(`Processing js file ${migrationFile}`);
              var migrationsFile = require(jsFile);
              if (isJSPromisified){
                await migrationsFile(bag.client);
              } else {
                var migrateJs = util.promisify(migrationsFile);
                await migrateJs(bag.client);
              }
            } catch (err) {
              util.log(util.format(err));
              util.log(`Problem executing queries in ${jsFile}`);
            }
        }
    }
    return;
}

async function _callPg(bag){
    try {
        var sqlFile = bag.sqlFile;
        util.log(`Processing sql file ${bag.migrationFile}`);
        var sql = fs.readFileSync(sqlFile).toString();
        return await bag.client.query(sql);
      } catch (err) {
        util.log(err);
        util.log(`Problem executing queries in ${sqlFile}`);
        util.log(`Exiting with exit code 1`);
        process.exit(1)
    }
}

function _addExtension(target, postfix) {
  if (path.extname(target)) {
      return target;
  } 
  return `${target}.${postfix}`;
}
