#!/bin/bash
set -e
set -o pipefail

export TABLES_PATH="$SCHEMA_DIR/tables"
export PRE_UPGRADE_PATH="$SCHEMA_DIR/pre_upgrade"
export RELATIONS_PATH="$SCHEMA_DIR/relations"
export POST_UPGRADE_PATH="$SCHEMA_DIR/post_upgrade"
export TEMPLATES_PATH="$SCHEMA_DIR/templates"
export DATA_PATH="$SCHEMA_DIR/data"
export CONTAINER_NODEJS_PATH="node"

__run_migration_script(){
  local migrationList=${1:-'pre_upgrade tables relations data post_upgrade'}
  export DBCONNECTIONSTRING="${PIPELINES_SHARED_DB_CONNECTIONSTRING}"
  export PIPELINEVERSION="${PIPELINES_CORE_CONTROLPLANEVERSION}"
  local installer_image="$PIPELINES_CORE_REGISTRYURL/$PIPELINES_CORE_INSTALLERIMAGE:$PIPELINES_CORE_CONTROLPLANEVERSION"
  local post_upgrade_cmd="docker run --rm \
    --net=host \
    -v $INSTALLER_HOME/scripts:$JF_PIPELINES_HOME/installer/scripts \
    -e DBCONNECTIONSTRING=$PIPELINES_SHARED_DB_CONNECTIONSTRING \
    -e PIPELINEVERSION=$PIPELINES_CORE_CONTROLPLANEVERSION \
    --user=0 \
    --entrypoint $CONTAINER_NODEJS_PATH \
    $installer_image \
    $JF_PIPELINES_HOME/installer/scripts/dbMigrations.js \
    $JF_PIPELINES_HOME/installer/scripts/schema \
    \"${migrationList}\" \
    $INSTALLER_BIN"
  eval "$post_upgrade_cmd"
}

__exec_install_flow() {
  log_proc_marker "Initializing migration"
  local migrationList='tables relations data'
  __run_migration_script "${migrationList}"
}

__exec_all() {
  log_proc_marker "Initializing migration"
  local migrationList='pre_upgrade tables relations data post_upgrade'
  __run_migration_script "${migrationList}"
}

__init_tables_psql() {
  log_proc_marker "Initializing tables"

  local sql_file="$TABLES_PATH/tables.sql"
  if [ -f "$sql_file" ]; then
    rm "$sql_file"
  fi

  for file in $(cat $TABLES_PATH/order.txt); do
    if [[ "$file" == *".sql" ]]; then
      echo "\i $TABLES_PATH/$file" >> "$sql_file"
    fi
  done

  local init_tables_cmd="psql $PIPELINES_SHARED_DB_CONNECTIONSTRING \
    -v ON_ERROR_STOP=1 \
    -f $sql_file > /dev/null"
  eval "$init_tables_cmd"
  rm "$sql_file"
}

__init_relations_psql() {
  log_proc_marker "Initializing relations"

  local sql_file="$RELATIONS_PATH/relations.sql"
  if [ -f "$sql_file" ]; then
    rm "$sql_file"
  fi

  while read table_name; do
    file_path="${RELATIONS_PATH}/${table_name}.sql"
    if [ -f "$file_path" ]; then
      echo "\i $file_path" >> "$sql_file"
    fi
  done < $RELATIONS_PATH/order.txt

  local init_relations_cmd="psql $PIPELINES_SHARED_DB_CONNECTIONSTRING \
    -v ON_ERROR_STOP=1 \
    -f $sql_file > /dev/null"
  eval "$init_relations_cmd"
  rm "$sql_file"
}

__init_data_psql() {
  log_proc_marker "Initializing data"

  local sql_file="${DATA_PATH}/tables.sql"
  if [ -f "$sql_file" ]; then
    rm "$sql_file"
  fi

  while read table_name; do
    file_path="${DATA_PATH}/${table_name}.sql"
    if [ -f "$file_path" ]; then
      echo "\i $file_path" >> "$sql_file"
    fi
  done < $DATA_PATH/order.txt

  local init_data_cmd="psql $PIPELINES_SHARED_DB_CONNECTIONSTRING \
    -v ON_ERROR_STOP=1 \
    -f $sql_file > /dev/null"

  eval "$init_data_cmd"
  rm "$sql_file"

  local version="$PIPELINES_CORE_BUILDPLANEVERSION"
  if [ ! -z "$BUILDPLANE_UPGRADE_VERSION" ]; then
    version="$BUILDPLANE_UPGRADE_VERSION"
  fi

  log_proc_msg "Updating intendedBuildPlane version for all nodes"
  local update_nodes_sql="update nodes set \"intendedBuildPlaneVersion\"='\'$version\'';"
  local update_nodes_cmd="psql $PIPELINES_SHARED_DB_CONNECTIONSTRING \
    -v ON_ERROR_STOP=1 \
    -c '$update_nodes_sql' > /dev/null"
  eval "$update_nodes_cmd"

  log_proc_msg "Updating build plane images"
  local BUILDPLANE_IMAGES_FILE="$INSTALLER_BIN/buildplaneImages.json"
  if [ ! -f "$BUILDPLANE_IMAGES_FILE" ]; then
    log_proc_warn "Build plane images file doesnt exist, skipping update"
    return
  fi

  local build_plane_images=$(cat $BUILDPLANE_IMAGES_FILE)
  if [ -z "$build_plane_images" ]; then
    log_proc_warn "Empty build plane images, skipping update"
    return
  fi

  #TODO:
  # get this from db based on the masterintegration set in json
  # this is currently being set to AWS master integration ID
  local build_plane_images_count=$(echo $build_plane_images \
    | jq '. | length')

  local BPI_TEMPLATE_SCRIPT_PATH="$TEMPLATES_PATH/buildPlaneImages.sql.template"
  local BPI_MIGRATION_SCRIPT_PATH="$TMP_DIR/buildPlaneImages.sql"

  cat "$TEMPLATES_PATH/transactionStart.sql.template" > $BPI_MIGRATION_SCRIPT_PATH

  for i in $(seq 1 $build_plane_images_count); do
    local build_plane_image=$(echo $build_plane_images \
      | jq '.['"$i-1"']')
    export name=$(echo $build_plane_image \
      | jq -r '.name')
    export image_id=$(echo $build_plane_image \
      | jq -r '.imageId')
    export region=$(echo $build_plane_image \
      | jq -r '.region')
    export architecture="$(echo $build_plane_image \
      | jq -r '.architecture')"
    export operating_system="$(echo $build_plane_image \
      | jq -r '.os')"
    export created_at="$(echo $build_plane_image \
      | jq -r '.createdAt')"
    export provider="$(echo $build_plane_image \
      | jq -r '.provider')"
    export metadata_property_bag="$(echo $build_plane_image \
      | jq -rc '.metadataPropertyBag')"
    local get_master_integration_sql="select id from \"masterIntegrations\" where name = '\'$provider\'';"
    local get_master_integration_cmd="psql $PIPELINES_SHARED_DB_CONNECTIONSTRING \
      -v ON_ERROR_STOP=1 \
      -t -c '$get_master_integration_sql'"
    export master_integration_id=$(eval "$get_master_integration_cmd")
    if [ ! -z "$master_integration_id" ]; then
      envsubst < "$BPI_TEMPLATE_SCRIPT_PATH" >> "$BPI_MIGRATION_SCRIPT_PATH"
    else
      log_proc_error "No masterIntegration found for provider name: $provider"
      rm "$BPI_MIGRATION_SCRIPT_PATH"
      exit 1
    fi
  done

  cat "$TEMPLATES_PATH/transactionEnd.sql.template" >> $BPI_MIGRATION_SCRIPT_PATH

  psql $PIPELINES_SHARED_DB_CONNECTIONSTRING \
    -v ON_ERROR_STOP=1 \
    -f "$BPI_MIGRATION_SCRIPT_PATH" > /dev/null

  rm "$BPI_MIGRATION_SCRIPT_PATH"

  ___insert_ootb_extensions
}

___insert_ootb_extensions() {
  local EXTENSIONS_FILE="$INSTALLER_BIN/ootb-extensions.json"
  if [ ! -f "$EXTENSIONS_FILE" ]; then
    log_proc_warn "Extensions files does not exists, skipping update"
    return
  fi

  local ootb_extensions=$(cat $EXTENSIONS_FILE)
  if [ -z "$ootb_extensions" ]; then
    log_proc_warn "Empty extensions, skipping update"
    return
  fi

  local extensions_count=$(echo $ootb_extensions \
    | jq '. | length')

  local EXTENSIONS_TEMPLATE_SCRIPT_PATH="$TEMPLATES_PATH/ootbExtensions.sql.template"
  local EXTENSIONS_MIGRATION_SCRIPT_PATH="$TMP_DIR/extensions.sql"

  cat "$TEMPLATES_PATH/transactionStart.sql.template" > $EXTENSIONS_MIGRATION_SCRIPT_PATH

  for i in $(seq 1 $extensions_count); do
    local ootb_extension=$(echo $ootb_extensions \
      | jq '.['"$i-1"']')
    export name=$(echo $ootb_extension \
      | jq -r '.name')
    export namespace=$(echo $ootb_extension \
      | jq -r '.namespace')
    export resourceType=$(echo $ootb_extension \
      | jq -r '.resourceType')
    export syntaxVersion="$(echo $ootb_extension \
      | jq -r '.syntaxVersion')"
    export createdAt="$(echo $ootb_extension \
      | jq -r '.createdAt')"
    export updatedAt="$(echo $ootb_extension \
      | jq -r '.updatedAt')"
    export platformsPropertyBag="$(echo $ootb_extension \
      | jq -rc '.platformsPropertyBag')"
    export description="$(echo $ootb_extension \
      | jq -rc '.description')"
    envsubst < "$EXTENSIONS_TEMPLATE_SCRIPT_PATH" >> "$EXTENSIONS_MIGRATION_SCRIPT_PATH"
  done

  cat "$TEMPLATES_PATH/transactionEnd.sql.template" >> $EXTENSIONS_MIGRATION_SCRIPT_PATH

  psql $PIPELINES_SHARED_DB_CONNECTIONSTRING \
    -v ON_ERROR_STOP=1 \
    -f "$EXTENSIONS_MIGRATION_SCRIPT_PATH" > /dev/null

  rm "$EXTENSIONS_MIGRATION_SCRIPT_PATH"
}

exec_db_task() {
  local db_task="$@"
  case $db_task in
    installFlow)
      log_proc_msg "Initializing migration"
      __exec_install_flow
      ;;
    all)
      log_proc_msg "Initializing migration"
      __exec_all
      ;;
    initTablesPSQL)
      log_proc_msg "Initializing object tables"
      __init_tables_psql
      ;;
    initRelationsPSQL)
      log_proc_msg "Initializing object relations"
      __init_relations_psql
      ;;
    initDataPSQL)
      log_proc_msg "Initializing seed data"
      __init_data_psql
      ;;
    *)
      log_proc_error "Invalid db task: $db_task"
      exit 1
      ;;
  esac
}
