'use strict';

process.title = 'pipelines.init';
module.exports = init;
// To allow the adapter to make more than five calls at a time:
require('http').globalAgent.maxSockets = 10000;

require('./lib/logger.js');
require('./lib/ActErr.js');

var favicon = require('serve-favicon');

var glob = require('glob');
var async = require('async');
var pg = require('pg');
var path = require('path');
var _ = require('underscore');

global.util = require('util');

var ConfigHandler = require('./lib/ConfigHandler.js');
global.config = {};

process.on('uncaughtException',
  function (err) {
    _logErrorAndExit('uncaughtException thrown:', err);
  }
);

function init() {
  var bag = {
    env: process.env,
    execScript: process.env.EXEC_SCRIPT,
    execScriptArgs: process.env.EXEC_SCRIPT_ARGS || '',
    config: {}
  };

  bag.who = util.format('installer.init|%s', init.name);

  async.series([
      _initializeConfig.bind(null, bag),
      _createClient.bind(null, bag),
      _executeScript.bind(null, bag)
    ],
    function (err) {
      if (err) {
        logger.error('Could not initialize : ' + util.inspect(err));
        process.exit(1);
      } else {
        logger.info(bag.who, 'Completed');
        process.exit();
      }
    }
  );
}

function _initializeConfig(bag, next) {
  /* jshint maxcomplexity: 20 */
  var who = bag.who + '|' + _initializeConfig.name;

  var configErrors = [];

  var globalConfig = new ConfigHandler().globalConfig;

  if (!globalConfig.dbConnectionString)
    configErrors.push(new ActErr(who, ActErr.ParamNotFound,
      'dbConnectionString is not defined'));

  if (!globalConfig.msgPassword)
    configErrors.push(new ActErr(who, ActErr.ParamNotFound,
      'msgPassword is not defined'));

  if (!globalConfig.runMode)
    globalConfig.runMode = 'production';

  if (globalConfig.ignoreTlsErrors &&
    globalConfig.ignoreTlsErrors === true) {
    // disable TLS check for installer itself
    process.env.NODE_TLS_REJECT_UNAUTHORIZED = '0';
  } else {
    globalConfig.ignoreTlsErrors = false;
  }

  if (!globalConfig.controlplaneVersion)
    configErrors.push(new ActErr(who, ActErr.ParamNotFound,
      'controlplaneVersion is not defined'));

  if (!globalConfig.buildplaneVersion)
    configErrors.push(new ActErr(who, ActErr.ParamNotFound,
      'buildplaneVersion is not defined'));

  if (!globalConfig.installerAuthToken)
    configErrors.push(new ActErr(who, ActErr.ParamNotFound,
      'installerAuthToken is not defined'));

  if (globalConfig.shouldReadFromVault && !globalConfig.vaultUrl)
    configErrors.push(new ActErr(who, ActErr.ParamNotFound,
      'vaultUrl is not defined'));

  if (bag.env.SRC_DIR)
    globalConfig.srcDir = bag.env.SRC_DIR;
  else
    configErrors.push(new ActErr(who, ActErr.ParamNotFound,
      'SRC_DIR is not defined'));

  if (bag.env.CONFIG_DIR) {
    globalConfig.hostConfigDir = bag.env.CONFIG_DIR;
    globalConfig.containerConfigDir = '/opt/jfrog/pipelines/var/etc';
  } else {
    configErrors.push(new ActErr(who, ActErr.ParamNotFound,
      'CONFIG_DIR is not defined'));
  }

  if (bag.env.LOGS_DIR)
    globalConfig.logsDir = bag.env.LOGS_DIR;
  else
    configErrors.push(new ActErr(who, ActErr.ParamNotFound,
      'LOGS_DIR is not defined'));

  if (bag.env.TMP_DIR)
    globalConfig.tmpDir = bag.env.TMP_DIR;
  else
    configErrors.push(new ActErr(who, ActErr.ParamNotFound,
      'TMP_DIR is not defined'));

  if (configErrors.length)
    return next(configErrors);
  else
    global.config = globalConfig;

  globalConfig.installerConfig = path.join(globalConfig.containerConfigDir,
    'system.yaml');
  globalConfig.scriptsDir = path.join(globalConfig.srcDir, '/scripts');

  _.extend(bag.config, global.config);
  return next();
}

function _createClient(bag, next) {
  if (bag.execScript === 'createService.js') return next();

  var who = bag.who + '|' + _createClient.name;
  logger.debug(who, 'Inside');

  // node-postgres expects ssl=true in the connection string
  // for ssl enabled postgres. If the connection string has
  // sslmode=require we will replace with ssl=true
  var dbConnectionString = bag.config.dbConnectionString;

  var connectionDetails = {};
  if (dbConnectionString &&
     (dbConnectionString.includes('sslmode=require') ||
      dbConnectionString.includes('ssl=true') ||
      dbConnectionString.includes('sslmode=prefer'))
  ){  
    connectionDetails = {
      connectionString: dbConnectionString,
      ssl: {
        require: true,
        rejectUnauthorized: false // disable SSL validation for self-signed certificates
      }    
    };
  } else {
    connectionDetails = {
      connectionString: dbConnectionString,
    };
  }

  global.config.client = new pg.Client(connectionDetails);
  global.config.client.connect(
    function (err) {
      return next(err);
    }
  );
}

function _executeScript(bag, next) {
  var who = bag.who + '|' + _executeScript.name;
  logger.debug(who, 'Inside');

  if (_.isUndefined(bag.execScript))
    return next(new ActErr(who, ActErr.ParamNotFound,
      'EXEC_SCRIPT env is missing'))

  var scriptPath = path.join(__dirname, bag.execScript);

  require(scriptPath)(bag.execScriptArgs,
    function (err, response) {
      return next(err);
    }
  );
}

function _logErrorAndExit(message, err) {
  logger.error(message);

  if (err && err.message)
    logger.error(err.message);

  if (err && err.stack)
    logger.error(err.stack);

  setTimeout(
    function () {
      process.exit(1);
    },
    3000
  );
}

if (require.main === module)
  init();
