#!/bin/bash
set -e
set -o pipefail

__check_vault_initialization() {
  log_proc_msg "Checking if vault has been initialized"
  _vault_get_initialized_status
  if [ $is_request_success == true ]; then
    log_proc_msg "Successfully checked initialization status"
  else
    log_proc_error "Error checking vault initialization status: $response"
    log_proc_error "Status code: $response_status_code"
    exit 1
  fi

  if [ ! -z "$response" ]; then
    local initialized=$(echo $response | jq '.initialized')
    if [ "$initialized" == "true" ]; then
      log_proc_msg "Vault is already initialized"
      VAULT_INITIALIZED=true
    else
      VAULT_INITIALIZED=false
    fi
  fi
}

__run_vault_initialization() {
  if [ "$VAULT_INITIALIZED" == "false" ]; then
    log_proc_msg "Initializing vault"
    _vault_put_init '{"secret_shares": 5, "secret_threshold": 3}'
    if [ $is_request_success == true ]; then
      log_proc_msg "Successfully initialized vault"
    else
      log_proc_error "Error initializing vault: $response"
      log_proc_error "Status code: $response_status_code"
      exit 1
    fi

    set_system_config "shared.vault.unsealKeys" "[]"
    local index=0
    while [[ $index -lt 5 ]]; do
      local value=$(echo $response | jq -r ".keys[$index]")
      set_system_config "shared.vault.unsealKeys" "$value" "PIPELINES_SHARED_VAULT_UNSEALKEYS_$index" "+="
      index=$(( index + 1 ))
    done

    local root_token=$(echo $response | jq -r '.root_token')
    set_system_config "shared.vault.token" "$root_token"
  fi
}

__check_vault_seal() {
  log_proc_msg "Checking if vault is sealed"
  _vault_get_seal_status
  if [ $is_request_success == true ]; then
    log_proc_msg "Successfully checked seal status"
  else
    log_proc_error "Error checking vault seal status: $response"
    log_proc_error "Status code: $response_status_code"
    exit 1
  fi

  if [ ! -z "$response" ]; then
    local sealed=$(echo $response | jq '.sealed')
    if [ "$sealed" == "true" ]; then
      log_proc_msg "Vault is sealed"
      VAULT_SEALED=true
    else
      VAULT_SEALED=false
    fi
  fi
}

__unseal_vault() {
  if [ "$VAULT_SEALED" == "true" ]; then
    log_proc_msg "Unsealing vault"

    if [ -z $PIPELINES_SHARED_VAULT_UNSEALKEYS_0 ] || [ -z $PIPELINES_SHARED_VAULT_UNSEALKEYS_1 ] || [ -z $PIPELINES_SHARED_VAULT_UNSEALKEYS_2 ]; then
      log_proc_error "Missing unseal key"
      exit 1
    fi

    _vault_put_unseal '{"secret_shares": 3, "key": "'$PIPELINES_SHARED_VAULT_UNSEALKEYS_0'"}'
    if [ $is_request_success != true ]; then
      log_proc_error "Error unsealing vault: $response"
      log_proc_error "Status code: $response_status_code"
      exit 1
    fi

    _vault_put_unseal '{"secret_shares": 3, "key": "'$PIPELINES_SHARED_VAULT_UNSEALKEYS_1'"}'
    if [ $is_request_success != true ]; then
      log_proc_error "Error unsealing vault: $response"
      log_proc_error "Status code: $response_status_code"
      exit 1
    fi

    _vault_put_unseal '{"secret_shares": 3, "key": "'$PIPELINES_SHARED_VAULT_UNSEALKEYS_2'"}'
    if [ $is_request_success != true ]; then
      log_proc_error "Error unsealing vault: $response"
      log_proc_error "Status code: $response_status_code"
      exit 1
    fi

    sleep 3
  fi
}

__create_secrets_mount() {
  log_proc_msg "Creating vault secrets mount"

  _vault_post_mount "pipelines" '{"type": "generic", "description": "Pipelines secrets mount"}'
  if [ $is_request_success != true ] && [ $response_status_code != 400 ]; then
    log_proc_error "Error creating vault secrets mount: $response"
    log_proc_error "Status code: $response_status_code"
    exit 1
  fi
}

__write_test_secret() {
  log_proc_msg "Writing secret to vault"

  _vault_post_secret "pipelines/testPermissions" '{"testValue": "testSecret"}'
  if [ $is_request_success != true ]; then
    log_proc_error "Error writing secret to vault: $response"
    log_proc_error "Status code: $response_status_code"
    exit 1
  fi
}

__get_test_secret() {
  log_proc_msg "Getting secret from vault"

  _vault_get_secret "pipelines/testPermissions"
  if [ $is_request_success != true ]; then
    log_proc_error "Error getting secret from vault: $response"
    log_proc_error "Status code: $response_status_code"
    exit 1
  fi

  if [ ! -z "$response" ]; then
    local test_value=$(echo $response | jq '.data.testValue')
    if [ "$test_value" == "testSecret" ]; then
      log_proc_error "Failed to retrieve secret"
      exit 1
    fi
  fi
}

__delete_test_secret() {
  log_proc_msg "Deleting secret from vault"

  _vault_delete_secret "pipelines/testPermissions"
  if [ $is_request_success != true ]; then
    log_proc_error "Error deleting secret from vault: $response"
    log_proc_error "Status code: $response_status_code"
    exit 1
  fi
}

main() {
  log_proc_msg "Initializing vault"
  if check_vault_variables; then
    __check_vault_initialization
    __run_vault_initialization
    __check_vault_seal
    __unseal_vault
    __create_secrets_mount
    __write_test_secret
    __get_test_secret
    __delete_test_secret
  else
    log_proc_msg "Skipping vault initialization as it is externalized."
    __check_vault_initialization
    __check_vault_seal
    __create_secrets_mount
    __write_test_secret
    __get_test_secret
    __delete_test_secret
  fi
}

main
