#!/bin/bash
set -e
set -o pipefail

export WELCOME_TEXT="
 _______  ___   _______  _______  ___      ___   __    _  _______  _______
|       ||   | |       ||       ||   |    |   | |  |  | ||       ||       |
|    _  ||   | |    _  ||    ___||   |    |   | |   |_| ||    ___||  _____|
|   |_| ||   | |   |_| ||   |___ |   |    |   | |       ||   |___ | |_____
|    ___||   | |    ___||    ___||   |___ |   | |  _    ||    ___||_____  |
|   |    |   | |   |    |   |___ |       ||   | | | |   ||   |___  _____| |
|___|    |___| |___|    |_______||_______||___| |_|  |__||_______||_______|

"
readonly UPDATE_TIME="$(date)"
readonly SECRET_STORAGE_FLAG="--secret-storage"

__inject_defaults() {
  # TODO:
  # This needs to be removed when we start fetching project list from artifactory
  log_proc_marker "Injecting default data"

  log_proc_msg "Creating default user"
  exec_util "CreateUser"

  log_proc_msg "Creating default provider"
  exec_util "CreateProvider"

  log_proc_msg "Creating default project"
  exec_util "CreateProject"

  log_proc_msg "Create internal pipeline source"
  exec_util "CreateInternalPipelineSource"
}

__update_last_update_time() {
  log_proc_msg "Installation complete"
  set_system_config "core.lastUpdateTime" "$UPDATE_TIME"
}

__boot_nodepoolservice() {
  log_proc_marker "Booting NodePoolService"
  exec_util "CreateService" "nodepoolservice"
}

__boot_stepService() {
   log_proc_marker "Booting Step Service"
   exec_util "CreateService" "stepservice"
}

__boot_runService() {
  log_proc_marker "Booting Run Service"
  exec_util "CreateService" "runservice"
}

__boot_logService() {
  log_proc_marker "Booting Log Service"
  exec_util "CreateService" "logservice"
}

__boot_analyticsservice() {
  log_proc_marker "Booting Analytics Service"
  exec_util "CreateService" "analyticsservice"
}

__boot_services() {
  log_proc_marker "Booting services"
  boot_stateless_services
}


__boot_www() {
  log_proc_marker "Booting WWW"
  exec_util "CreateService" "www"
}

__boot_frontend() {
  log_proc_marker "Booting frontend"
  exec_util "CreateService" "frontend"
}

__boot_api() {
  log_proc_marker "Booting API"
  exec_util "CreateService" "api"
}

__initialize_msg() {
  log_proc_msg "Initializing msg"
  source "$INSTALLER_SCRIPTS_DIR/initializeMsg.sh"
}

__initialize_vault() {
  log_proc_msg "Initializing vault"
  source "$INSTALLER_SCRIPTS_DIR/initializeVault.sh"
}

__initialize_db() {
  log_proc_msg "Initializing database (takes about a minute)"
  source "$INSTALLER_SCRIPTS_DIR/dbTasks.sh"
  eval exec_db_task "installFlow"
}

__install_redis() {
  log_proc_msg "Installing redis"
  source "$INSTALLER_SCRIPTS_DIR/installRedis.sh"
}

__install_observability() {
  log_proc_msg "Installing observability"
  source "$INSTALLER_SCRIPTS_DIR/installObservability.sh"
}

__install_msg() {
  log_proc_msg "Installing msg"
  source "$INSTALLER_SCRIPTS_DIR/installMsg.sh"
}

__install_vault() {
  log_proc_msg "Installing vault"
  source "$INSTALLER_SCRIPTS_DIR/installVault.sh"
}

__create_vault_table() {
  log_proc_msg "Creating vault table"
  source "$INSTALLER_SCRIPTS_DIR/createVaultTable.sh"
}

__install_db() {
  log_proc_msg "Installing database"
  source "$INSTALLER_SCRIPTS_DIR/installDb.sh"
}

__set_controlplane_version() {
  if [ ! -z "$PIPELINES_CORE_CONTROLPLANEVERSION" ]; then
    # Update callHomeUrl from bintray.net to bintray.com for versions before 1.5
    if [ "$PIPELINES_CORE_CALLHOMEURL" == "https://api.bintray.net/products/jfrog/pipelines/stats/usage" ]; then
      local major=$(echo "$PIPELINES_CORE_CONTROLPLANEVERSION" | cut -d "." -f 1)
      local minor=$(echo "$PIPELINES_CORE_CONTROLPLANEVERSION" | cut -d "." -f 2)

      if [ $major == 1 ] && [[ $minor -lt 5 ]]; then
        set_system_config "core.callHomeUrl" "https://api.bintray.com/products/jfrog/pipelines/stats/usage"
      fi
    fi
  fi

  log_proc_msg "Set controlplane version"
  local install_version=$(cat "$CONTROLPLANEVERSION_FILE")
  if [ ! -z "$install_version" ]; then
    set_system_config "core.controlplaneVersion" "$install_version"
  else
    set_system_config "core.controlplaneVersion" "master"
  fi
}

__set_buildplane_version() {
  log_proc_msg "Set buildplane version"
  local buildplane_version=$(cat "$BUILDPLANEVERSION_FILE")
  if [ ! -z "$buildplane_version" ]; then
    set_system_config "core.buildplaneVersion" "$buildplane_version"
  else
    set_system_config "core.buildplaneVersion" "master"
  fi
}

__pull_images() {
  log_proc_marker "Pulling images"
  platform_exec "pull_service_images"
}

__stop_services() {
  log_proc_marker "Stopping running services from existing installations"
  stop_stateless_services
  stop_stateful_services
}

__install_router() {
  log_proc_marker "Installing router"
  source "$INSTALLER_SCRIPTS_DIR/installRouter.sh"
}

__restart_router() {
  log_proc_marker "Restarting Router"
  source "$INSTALLER_SCRIPTS_DIR/restartRouter.sh"
}

__generate_service_id() {
  log_proc_msg "Generating service Id for the installation"
  if [ -z "$PIPELINES_SHARED_ARTIFACTORY_SERVICEID" ]; then
    local rt_service_id="jfpip@$(date +%s)"
    set_system_config "shared.artifactory.serviceId" "$rt_service_id"
  else
    log_proc_msg "Service ID already generated: $PIPELINES_SHARED_ARTIFACTORY_SERVICEID"
  fi
}

__generate_master_key() {
  log_proc_msg "Generating master key for the installation"
  ## If Master key is not present, try reading it from the file system
  ## If the master key is not present in the file system too, then generate one
  if [ -z "$PIPELINES_SHARED_SECURITY_MASTERKEY" ]; then
    if [ -f "$SECURITY_DIR/master.key" ]; then
      local key=$(cat "$SECURITY_DIR/master.key")
      export PIPELINES_SHARED_SECURITY_MASTERKEY=$key
      yq e "del(.shared.security.masterKey)" -i $PIPELINES_YAML_CONFIG
    else
      if type openssl &> /dev/null && true; then
        local key=$(openssl rand -hex 32)
        set_system_config "shared.security.masterKey" "$key"
        echo "$key" | tee -a "$SECURITY_DIR/master.key"
      else
        log_proc_error "'openssl' not installed. Please install openssl to continue installation"
        exit 1
      fi
    fi
  else
    log_proc_msg "Master key already generated"
  fi
}

__bootstrap_rt() {
  log_proc_msg "Bootstrapping artifactory"
  source "$INSTALLER_SCRIPTS_DIR/bootstrapRt.sh"
}

__configure_proxy() {
  log_proc_msg "Configuring proxy settings"
  source "$INSTALLER_SCRIPTS_DIR/configureProxy.sh"
}

__set_defaults() {

  if [ -z "$PIPELINES_SHARED_ROUTER_JOINKEY" ]; then
    log_proc_warn "Join key not found. Checking security dir: $SECURITY_DIR"
    if [ -f "$SECURITY_DIR/join.key" ]; then
      log_proc_msg "Found join.key file in security dir"
      export PIPELINES_SHARED_ROUTER_JOINKEY=$(cat $SECURITY_DIR/join.key)
      log_proc_msg "Security dir contains join key length ${#PIPELINES_SHARED_ROUTER_JOINKEY}"
      yq e "del(.shared.router.joinKey)" -i $PIPELINES_YAML_CONFIG
      log_proc_msg "Removing contents at .shared.router.joinKey in favor of join.key file contents"
    else
      log_proc_error "Please provide a join key to connect to a router with --artifactory-joinkey, exiting"
      exit 1
    fi
  fi

  set_system_config "shared.security.joinKey" "$PIPELINES_SHARED_ROUTER_JOINKEY"

  if [ -z "$PIPELINES_CORE_RUNMODE" ]; then
    set_system_config "core.runMode" "onebox"
  fi

  local node_id=$(hostname)
  if [ -z "$PIPELINES_CORE_ID" ] || [ "$PIPELINES_CORE_ID" != "$node_id" ]; then
    if [ "$PIPELINES_CORE_RUNMODE" == "devmode" ]; then
      ## NOTE: this is added so that pipelines and artifactory running on the
      ## same node node do not get the same node_id
      node_id="$node_id"-pipelines-dev
    fi
    set_system_config "core.id" "$node_id"
  fi

  if [ -z "$PIPELINES_CORE_USER" ]; then
    local user=$(id -u)
    local group=$(id -g)
    if [ -z "$user" ]; then
      log_proc_error "Cannot set installation user, empty value returned from 'id -u' "
      exit 1
    else
      set_system_config "core.user" "$user"
      set_system_config "core.group" "$group"
    fi
  fi

  if [ -z "$PIPELINES_CORE_INSTALLERAUTHTOKEN" ]; then
    if [ -z "$GLOBAL_PASSWORD" ]; then
      export GLOBAL_PASSWORD=$(uuidgen)
    fi
    set_system_config "core.installerAuthToken" "$GLOBAL_PASSWORD"
  else
    export GLOBAL_PASSWORD="$PIPELINES_CORE_INSTALLERAUTHTOKEN"
  fi

  if [ -z "$PIPELINES_SHARED_MSG_ERLANGCOOKIE" ]; then
    export PIPELINES_SHARED_MSG_ERLANGCOOKIE=$(uuidgen)
    set_system_config "shared.msg.erlangCookie" "$PIPELINES_SHARED_MSG_ERLANGCOOKIE"
  fi

  local url_parse_path="$INSTALLER_SCRIPTS_DIR/urlParse.sh"

  local accessControlAllowOrigins=$(yq e ".core.accessControlAllowOrigins" $PIPELINES_YAML_CONFIG)
  if [ "$accessControlAllowOrigins" != "null" ]; then
    local index=0
    local containsStar=false
    local no_origin_remaining=false
    while [ $no_origin_remaining != true ]; do
      local origin=$(yq e  ".core.accessControlAllowOrigins[$index]" $PIPELINES_YAML_CONFIG)
      if [ "$origin" = '*' ]; then
        containsStar=true
      fi
      if [ "$origin" == "null" ]; then
        no_origin_remaining=true
      else
        let "index = $index + 1"
      fi
    done

    if [ $containsStar == true ]; then
      log_proc_msg "Access-Control-Allow-Origins includes *. Is this correct? (type y/Y to confirm)"
      read -r confirmation
      confirmation=$(echo "$confirmation" | awk '{print toupper($0)}')

      if [[ "$confirmation" =~ "Y" ]] || [[ "$confirmation" =~ "y" ]]; then
        log_proc_msg "Confirmed"
      else
        log_proc_error "Please run install with --access-control-allow-origins set correctly."
        exit 1
      fi
    fi
  fi

  if [ -d "$PIPELINES_CUSTOM_CERTS_DIR" ]; then
    if [ ! -z "$(ls -A $PIPELINES_CUSTOM_CERTS_DIR)" ]; then
      set_system_config "core.customCACertsPath" "$PIPELINES_CUSTOM_CERTS_DIR"
      if [[ "$PIPELINES_CORE_OS" == *"Ubuntu"* ]]; then
        cp $PIPELINES_CUSTOM_CERTS_DIR/*.crt /usr/local/share/ca-certificates/
        update-ca-certificates
        export CURL_CA_BUNDLE="/etc/ssl/certs/ca-certificates.crt"
      elif [[ "$PIPELINES_CORE_OS" == *"CentOS"* ]] || [[ "$PIPELINES_CORE_OS" == *"RHEL"* ]]; then
        cp $PIPELINES_CUSTOM_CERTS_DIR/*.crt /usr/share/pki/ca-trust-source/anchors/
        update-ca-trust extract
        export CURL_CA_BUNDLE="/etc/ssl/certs/ca-bundle.crt"
      fi
      pushd $PIPELINES_CUSTOM_CERTS_DIR
      if [ -f "pipelines_custom_certs.crt" ]; then
        log_proc_msg "Resetting pipelines_custom_certs.crt"
        rm -f "pipelines_custom_certs.crt"
      fi
      for file in *.crt; do
        if [ -f "$file" ]; then
          if [ "$file" == "pipelines_custom_certs.crt" ]; then
            continue
          else
            log_proc_msg "Adding $file to pipelines custom certs."
            cat $file >> "pipelines_custom_certs.crt"
          fi
        fi
      done
      log_proc_msg "Done configuring pipelines_custom_certs.crt"
      popd
    fi
  fi

  if [ -z "$PIPELINES_CORE_INSTALLERIP" ]; then
    log_proc_error "Installer IP cannot be empty"
    log_proc_error "Please set --installer-ip <installer_host_ip> argument during installation"
    exit 1
  elif [ "$PIPELINES_CORE_INSTALLERIP" == '127.0.0.1' ] || [ "$PIPELINES_CORE_INSTALLERIP" == 'localhost' ]; then
    log_proc_error "Installer IP cannot be set to 127.0.0.1 or localhost"
    log_proc_error "Please set the internal IP address of the installation host"
    exit 1
  fi

  if [ -z "$PIPELINES_SHARED_ROUTER_DATAPORT" ]; then
    set_system_config "shared.router.dataPort" 8082
  fi

  if [ -z "$PIPELINES_SHARED_MSG_PORT" ]; then
    set_system_config "shared.msg.port" 30200
  fi

  if [ -z "$PIPELINES_SHARED_MSG_ADMINPORT" ]; then
    set_system_config "shared.msg.adminPort" 30201
  fi

  if [ -z "$PIPELINES_SHARED_REDIS_PORT" ]; then
    set_system_config "shared.redis.port" 6379
  fi

  if [ "$PIPELINES_CORE_RUNMODE" == "devmode" ]; then
    set_system_config "shared.router.dataPort" "9092"
    set_system_config "core.allowCallHome" "false"
  fi

  # This is to handle pipelines installations which are using external db and
  # running install command again without connection string
  if [ -z "$PIPELINES_SHARED_DB_CONNECTIONSTRING" ] && [ "$PIPELINES_SHARED_DB_ISEXTERNAL" != "true" ] ; then
    set_system_config "shared.db.externalUrl" "$PIPELINES_CORE_INSTALLERIP"
    set_system_config "shared.db.ip" "$PIPELINES_CORE_INSTALLERIP"
  fi

  if ! __is_vault_disabled; then
    local is_vault_ip_correct="true"
    if [ ! -z "$PIPELINES_SHARED_VAULT_IP" ] && [ "$PIPELINES_SHARED_VAULT_IP" != "$PIPELINES_CORE_INSTALLERIP" ]; then
      is_vault_ip_correct="false"
    fi
    if [ -z "$PIPELINES_SHARED_VAULT_URL" ] || [ "$is_vault_ip_correct" == "false" ]; then
      set_system_config "shared.vault.ip" "$PIPELINES_CORE_INSTALLERIP"
      set_system_config "shared.vault.url" "http://$PIPELINES_SHARED_VAULT_IP:$PIPELINES_SHARED_VAULT_PORT"
    fi
  fi


  if [ "$PIPELINES_CORE_INSTALLERIP" != "$PIPELINES_SHARED_MSG_IP" ]; then
    # system yaml copied from a different host, update rabbitmq cluster ip
    export RABBITMQ_CLUSTER_HOST=$PIPELINES_SHARED_MSG_IP
  fi
  set_system_config "shared.msg.ip" "$PIPELINES_CORE_INSTALLERIP"
  if [ "$PIPELINES_SHARED_REDIS_IP" != "$PIPELINES_CORE_INSTALLERIP" ]; then
    # system yaml copied from a different host, update redis cluster ip
    export REDIS_CLUSTER_IP=$PIPELINES_SHARED_REDIS_IP
  fi
  set_system_config "shared.redis.ip" "$PIPELINES_CORE_INSTALLERIP"
  if [ -z "$PIPELINES_SHARED_REDIS_PASSWORD" ]; then
    set_system_config "shared.redis.password" "$GLOBAL_PASSWORD"
  fi
  if [ -z "$PIPELINES_RUNTIME_ROOTBUCKET" ]; then
    local bucket_name="pipelines_$(date +%s)"
    set_system_config "runtime.rootBucket" "$bucket_name"
  fi

  local default_registry_url="releases-docker.jfrog.io"
  if [ -z "$PIPELINES_CORE_REGISTRYURL" ]; then
    if [ ! -f "$REGISTRY_CREDS" ]; then
      set_system_config "core.registryUrl" "$default_registry_url"
    else
      # using the first registry from docker config by default
      local registry_url=$(cat $REGISTRY_CREDS \
        | jq -r '.auths | keys[0]')
      if [ ! -z "$registry_url" ]; then
        set_system_config "core.registryUrl" "$registry_url"
      fi
    fi
  fi

  if [ ! -z "$PIPELINES_SHARED_ARTIFACTORY_BASEURL" ]; then
    # For the router, we don't use the path if there is one.
    local protocol=$($url_parse_path --protocol $PIPELINES_SHARED_ARTIFACTORY_BASEURL)
    local hostname=$($url_parse_path --hostname $PIPELINES_SHARED_ARTIFACTORY_BASEURL)
    local port=$($url_parse_path --port $PIPELINES_SHARED_ARTIFACTORY_BASEURL)

    if [ ! -z "$port" ]; then
      ping_artifactory "$protocol://$hostname:$port" "--base-url"
    else
      ping_artifactory "$protocol://$hostname" "--base-url"
    fi
  fi

  ## fallback values
  if [ -z "$PIPELINES_SHARED_DB_CONNECTIONSTRING" ] && [ -z "$PIPELINES_SHARED_DB_PASSWORD" ]; then
    set_system_config "shared.db.password" "$GLOBAL_PASSWORD"
  fi
  if [ -z "$PIPELINES_SHARED_MSG_PASSWORD" ]; then
    set_system_config "shared.msg.password" "$GLOBAL_PASSWORD"
  fi
  if [ -z "$PIPELINES_SHARED_MSG_UI_PASSWORD" ]; then
    set_system_config "shared.msg.ui.password" "$GLOBAL_PASSWORD"
  fi
  if [ -z "$PIPELINES_CORE_SERVICES_API_EXTERNALURL" ]; then
    log_proc_error "Please set an external API URL with --api-url <api_url> argument during installation"
    log_proc_error "For example, --api-url http://$PIPELINES_CORE_INSTALLERIP:$PIPELINES_SHARED_ROUTER_DATAPORT/pipelines/api"
    exit 1
  else
    local api_external_protocol=$($url_parse_path --protocol $PIPELINES_CORE_SERVICES_API_EXTERNALURL)
    if [ "$api_external_protocol" != "http" ] && [ "$api_external_protocol" != "https" ]; then
      log_proc_error "External API URL must have \"http\" or \"https\" protocol"
      log_proc_error "Set an external API URL with --api-url"
      exit 1
    fi
  fi
  if [ -z "$PIPELINES_CORE_SERVICES_WWW_SESSIONSECRET" ]; then
    set_system_config "core.services.www.sessionSecret" "$(uuidgen)"
  fi

  # Add the default accessControlAllowOrigins if it is both in the YML and empty
  if [ "$accessControlAllowOrigins" != "null" ]; then
    local firstOrigin=$(yq e ".core.accessControlAllowOrigins[0]" $PIPELINES_YAML_CONFIG)
    if ([ "$firstOrigin" == "null" ] || [ -z "$firstOrigin" ]) && [ ! -z "$PIPELINES_SHARED_ARTIFACTORY_BASEURLUI" ]; then
      local ui_url="$PIPELINES_SHARED_ARTIFACTORY_BASEURLUI"
      if [[ $ui_url =~ /$ ]]; then
        ui_url=${ui_url%/}
      fi
      set_system_config "core.accessControlAllowOrigins[0]" "$ui_url" "PIPELINES_CORE_ACCESSCONTROLALLOWORIGINS_0"
    fi
  fi

  add_language_images
  update_services_list
  update_queues_list
}

__validate_run_mode() {
  if [ "$#" -ne 1 ]; then
    log_proc_error "No PIPELINES_CORE_RUNMODE provided for validation, exiting"
    exit 1
  fi

  local new_run_mode="$1"
  if [ ! -z "$PIPELINES_CORE_RUNMODE" ] && [ "$PIPELINES_CORE_RUNMODE" != "$new_run_mode" ]; then
    log_proc_msg "Cannot change PIPELINES_CORE_RUNMODE of an existing installation"
    log_proc_msg "To change PIPELINES_CORE_RUNMODE, clean up the current installation and install again or manually change it"
  fi
}

__validate_user() {
  local user_grps="$1"
  local install_user=$(echo $user_grps | cut -d':' -f1)
  local install_group=$(echo $user_grps | cut -d':' -f2)

  if [ -z "$install_group" ]; then
    install_group="$install_user"
  fi

  ## Only save the ID's of user and group
  install_user=$(id -u $install_user)
  install_group=$(id -g $install_group)

  set_system_config "core.user" "$install_user"
  set_system_config "core.group" "$install_group"
}

__parse_args() {
  if [[ $# -gt 0 ]]; then
    while [[ $# -gt 0 ]]; do
      ARGUMENT="$1"

      case $ARGUMENT in
        -s|--silent)
          export NO_PROMPT=true
          shift
          ;;
        -d|--devmode)
          __validate_run_mode "devmode"
          log_proc_msg "Running installer in devmode"
          set_system_config "core.runMode" "devmode"
          shift
          ;;
        --installer-ip)
          local installer_ip="$2"
          validate_nonempty_option "--installer-ip" $installer_ip
          set_system_config "core.installerIP" "$installer_ip"
          shift
          shift
          ;;
        --global-password)
          if [ -z "$2" ]; then
            log_proc_error "Global password cannot be empty"
            exit 1
          else
            export GLOBAL_PASSWORD="$2"
          fi
          shift
          shift
          ;;
        --install-user)
          local install_user="$2"
          validate_nonempty_option "--install-user" $install_user
          __validate_user "$install_user"
          shift
          shift
          ;;
        --base-url)
          local rt_url="$2"
          validate_nonempty_option "--base-url" $rt_url
          set_system_config "shared.artifactory.baseUrl" "$rt_url"
          shift
          shift
          ;;
        --base-url-ui)
          local rt_ui_url="$2"
          set_system_config "shared.artifactory.baseUrlUI" "$rt_ui_url"
          shift
          shift
          ;;
        --artifactory-service-id)
          local rt_service_id="$2"
          validate_nonempty_option "--artifactory-service-id" $rt_service_id
          set_system_config "shared.artifactory.serviceId" "$rt_service_id"
          shift
          shift
          ;;
        --artifactory-proxy)
          local rt_proxy="$2"
          validate_nonempty_option "--artifactory-proxy" $rt_proxy
          set_system_config "shared.artifactory.proxy" "$rt_proxy"
          shift
          shift
          ;;
        --artifactory-proxy-username)
          local rt_proxy_username="$2"
          validate_nonempty_option "--artifactory-proxy-username" $rt_proxy_username
          set_system_config "shared.artifactory.username" "$rt_proxy_username"
          shift
          shift
          ;;
        --artifactory-proxy-password)
          local rt_proxy_password="$2"
          validate_nonempty_option "--artifactory-proxy-password" $rt_proxy_password
          set_system_config "shared.artifactory.password" "$rt_proxy_password"
          shift
          shift
          ;;
        --artifactory-joinkey)
          local rt_joinkey="$2"
          validate_nonempty_option "--artifactory-joinkey" $rt_joinkey
          set_system_config "shared.router.joinKey" "$rt_joinkey"
          shift
          shift
          ;;
        --artifactory-masterkey)
          ## only required for development environment
          local rt_masterkey="$2"
          validate_nonempty_option "--artifactory-masterkey" $rt_masterkey
          set_system_config "shared.security.masterKey" "$rt_masterkey"
          shift
          shift
          ;;
        --image-registry-url)
          local registry_url="$2"
          validate_nonempty_option "--image-registry-url" $registry_url
          set_system_config "core.registryUrl" "$registry_url"
          shift
          shift
          ;;
        --image-registry-creds)
          local registry_creds_file="$2"
          if [ -f "$registry_creds_file" ]; then
            cp -r "$registry_creds_file" "$REGISTRY_CREDS"
          else
            log_proc_error "Missing registry credentials file: $registry_creds_file"
            exit 1
          fi
          shift
          shift
          ;;
        --build-image-registry-url)
          local registry_url="$2"
          PIPELINES_RUNTIME_LANGUAGEIMAGES_REGISTRYURL="$registry_url"
          local index=0
          local no_image_remaining=false
          while [ $no_image_remaining != true ]; do
            local image=$(yq e ".runtime.languageImages[$index]" $PIPELINES_YAML_CONFIG)
            if [ "$image" == "null" ]; then
              no_image_remaining=true
            else
              set_system_config "runtime.languageImages[$index].registryUrl" "$registry_url" "PIPELINES_RUNTIME_LANGUAGEIMAGES_$index_REGISTRYURL"
              let "index = $index + 1"
            fi
          done
          shift
          shift
          ;;
        --api-url)
          local api_url="$2"
          validate_nonempty_option "--api-url" "$api_url"
          set_system_config "core.services.api.externalUrl" "$api_url"
          shift
          shift
          ;;
        --db-connection-string)
          local connection_string="$2"
          validate_nonempty_option "--db-connection-string" $connection_string
          export PIPELINES_SHARED_DB_CONNECTIONSTRING=$connection_string
          set_system_config "shared.db.isExternal" "true"
          split_connection_string
          shift
          shift
          ;;
        --vault-url)
          local vault_url="$2"
          validate_nonempty_option "--vault-url" $vault_url
          set_system_config "shared.vault.url" "$vault_url"
          shift
          shift
          ;;
        --vault-root-token)
          local vault_root_token="$2"
          validate_nonempty_option "--vault-root-token" $vault_root_token
          set_system_config "shared.vault.token" "$vault_root_token"
          shift
          shift
          ;;
        --skip-image-pull)
          export SKIP_IMAGE_PULL=true
          shift
          ;;
        --state-bucket)
          local state_bucket="$2"
          validate_nonempty_option "--state-bucket" $state_bucket
          set_system_config "runtime.rootBucket" "$state_bucket"
          shift
          shift
          ;;
        --global-http-proxy)
          local global_http_proxy="$2"
          validate_nonempty_option "--global-http-proxy" $global_http_proxy
          set_system_config "core.proxy.httpProxy" "$global_http_proxy"
          export http_proxy=$global_http_proxy
          shift
          shift
          ;;
        --global-https-proxy)
          local global_https_proxy="$2"
          validate_nonempty_option "--global-https-proxy" $global_https_proxy
          set_system_config "core.proxy.httpsProxy" "$global_https_proxy"
          export https_proxy=$global_https_proxy
          shift
          shift
          ;;
        --global-proxy-username)
          local global_proxy_username="$2"
          validate_nonempty_option "--global-proxy-username" $global_proxy_username
          set_system_config "core.proxy.username" "$global_proxy_username"
          shift
          shift
          ;;
        --global-proxy-password)
          local global_proxy_password="$2"
          validate_nonempty_option "--global-proxy-password" $global_proxy_password
          set_system_config "core.proxy.password" "$global_proxy_password"
          shift
          shift
          ;;
        --no-verify-ssl)
          local no_verify_ssl="$2"
          validate_nonempty_option "--no-verify-ssl" $no_verify_ssl
          set_system_config "core.noVerifySsl" "$no_verify_ssl"
          shift
          shift
          ;;
        --global-no-proxy)
          local global_no_proxy="$2"
          validate_nonempty_option "--global-no-proxy" $global_no_proxy
          set_system_config "core.proxy.noProxy" "$global_no_proxy"
          export no_proxy=$global_no_proxy
          shift
          shift
          ;;
        $SECRET_STORAGE_FLAG)
          local secret="$2"
          validate_nonempty_option "$SECRET_STORAGE_FLAG" $secret
          if [[ "$secret" == "access" ]] ; then
            local enableVaultToAccessMigration="false"
            local shouldJustUpdateAccess="true"
            local shouldReadFromVault="false"
            __set_access_vault_flags "${enableVaultToAccessMigration}" "${shouldJustUpdateAccess}" "${shouldReadFromVault}"            
          else
            log_proc_error "The supported value for --secret-storage is \"access\""
            exit 1
          fi
          shift
          shift
          ;;
        --access-control-allow-origins)
          local access_control_allow_origins="$2"
          validate_nonempty_option "--access-control-allow-origins" $access_control_allow_origins

          local allowedOrigins="$access_control_allow_origins"
          local index=0
          set_system_config "core.accessControlAllowOrigins" "[]"

          while [ ! -z "$allowedOrigins" ]; do
            # start of string up to ","
            origin=${allowedOrigins%%,*}
            if [ ! -z "$origin" ]; then
              set_system_config "core.accessControlAllowOrigins[$index]" "$origin" "PIPELINES_CORE_ACCESSCONTROLALLOWORIGINS_$index"
              index=$(( index + 1 ))
            fi
            if [ "$origin" == "$allowedOrigins" ]; then
              allowedOrigins=""
            else
              allowedOrigins="${allowedOrigins#*,}"
            fi
          done

          shift
          shift
          ;;
        --config-backup-count)
          local config_backup_count="$2"
          validate_nonempty_option "--config-backup-count" $config_backup_count
          set_system_config "core.configBackupCount" "$config_backup_count"
          shift
          shift
          ;;
        --enable-call-home)
          set_system_config "core.allowCallHome" "true"
          shift
          ;;
        --disable-call-home)
          set_system_config "core.allowCallHome" "false"
          shift
          ;;
        --rabbitmq-health-check-interval)
          local rabbitmq_health_check_interval="$2"
          validate_nonempty_option "--rabbitmq-health-check-interval" $rabbitmq_health_check_interval
          set_system_config "core.rabbitmqHealthCheckIntervalInMins" "$rabbitmq_health_check_interval"
          shift
          shift
          ;;
        --artifactory-health-check-interval)
          local artifactory_health_check_interval="$2"
          validate_nonempty_option "--artifactory-health-check-interval" $artifactory_health_check_interval
          set_system_config "core.artifactoryHealthCheckIntervalInMins" "$artifactory_health_check_interval"
          shift
          shift
          ;;
        --db-health-check-interval)
          local db_health_check_interval="$2"
          validate_nonempty_option "--db-health-check-interval" $db_health_check_interval
          set_system_config "core.dbHealthCheckIntervalInMins" "$db_health_check_interval"
          shift
          shift
          ;;
        *)
          echo "Invalid option: $ARGUMENT"
          print_help
          ;;
      esac
    done
  fi
  __set_defaults
}

__is_vault_disabled() {
  if [[ ! -z "${PIPELINES_SHARED_ACCESS_SHOULDREADFROMVAULT}" ]] && [[ "${PIPELINES_SHARED_ACCESS_SHOULDREADFROMVAULT}" == "false" ]]; then
    return 0;
  else
    return 1;
  fi
}

__set_access_vault_flags() {
  local do_not_exit_on_failure=${4:-false}
  if [[ ! -z "$PIPELINES_SHARED_ARTIFACTORY_BASEURL" ]] && [[ "$PIPELINES_SHARED_ARTIFACTORY_BASEURL" == "https"* ]]; then
    log_proc_msg "Disabling usage of vault"
    local enableVaultToAccessMigration="$1"
    local shouldJustUpdateAccess="$2"
    local shouldReadFromVault="$3"
    set_system_config "shared.access.enableVaultToAccessMigration" "${enableVaultToAccessMigration}"
    set_system_config "shared.access.shouldJustUpdateAccess" "${shouldJustUpdateAccess}"
    set_system_config "shared.access.shouldReadFromVault" "${shouldReadFromVault}"  
  elif ! $do_not_exit_on_failure; then
    log_proc_error "To enable $SECRET_STORAGE_FLAG, the platform should be enabled with tls"
    exit 1;
  fi
}

__set_default_access_flags(){
  if [[ -z "${PIPELINES_SHARED_ACCESS_ENABLEVAULTTOACCESSMIGRATION}" ]] \
      && [[ -z "${PIPELINES_SHARED_ACCESS_SHOULDJUSTUPDATEACCESS}" ]] \
      && [[ -z "${PIPELINES_SHARED_ACCESS_SHOULDREADFROMVAULT}" ]]; then
        local enableVaultToAccessMigration="false"
        local shouldJustUpdateAccess="false"
        local shouldReadFromVault="true"
        __set_access_vault_flags "${enableVaultToAccessMigration}" "${shouldJustUpdateAccess}" "${shouldReadFromVault}"    
  fi
}

main() {
  log_proc_marker "Installing pipelines"
  echo "$WELCOME_TEXT"
  __set_controlplane_version
  __set_buildplane_version
  __parse_args "$@"
  __set_default_access_flags
  __generate_master_key

  #validations and configuration
  __pull_images
  __generate_service_id
  __stop_services

  # Create mounted volume dirs with permissions for pipelines user
  create_volume_dirs

  __install_router
  __install_observability
  __bootstrap_rt
  __configure_proxy

  # Setup environment
  __install_db
  if ! __is_vault_disabled; then
    __create_vault_table
    __install_vault
  fi
  __install_msg
  __install_redis

  ## restart router to encrypt system yaml
  __restart_router

  # Initialize core components
  __initialize_db
  if ! __is_vault_disabled; then
    __initialize_vault
  fi
  __initialize_msg
  __inject_defaults

  # Initialize services
  __boot_api
  __boot_www
  __boot_services

  __update_last_update_time
}

{
  main "$@"
} 2>&1 | tee "$LOG_FILE" ; ( exit ${PIPESTATUS[0]} )