#!/bin/bash
set -e
set -o pipefail

export COMPONENT="redis"
export REDIS_DATA_DIR="$RUNTIME_DIR/data/$COMPONENT"
export REDIS_CONFIG_DIR="$CONFIG_DIR/$COMPONENT"
export LOGS_FILE="$LOGS_DIR/$COMPONENT.log"
export REDIS_INSTALL_SCRIPT="installRedis.sh"
export PIPELINES_SHARED_REDIS_IMAGE="$PIPELINES_CORE_REGISTRYURL/$PIPELINES_SHARED_REDIS_IMAGE:$PIPELINES_CORE_CONTROLPLANEVERSION"

## Write logs of this script to component specific file
exec &> >(tee -a "$LOGS_FILE")

__validate_redis_envs() {
  log_proc_msg "Initializing redis environment variables"
  log_proc_msg "STATE_CONFIG_DIR: $STATE_CONFIG_DIR"
  log_proc_msg "PIPELINES_SHARED_REDIS_IP: $PIPELINES_SHARED_REDIS_IP"
  log_proc_msg "PIPELINES_SHARED_REDIS_PORT: $PIPELINES_SHARED_REDIS_PORT"
  log_proc_msg "PIPELINES_CORE_REGISTRYURL: $PIPELINES_CORE_REGISTRYURL"
}

__validate_redis_mounts() {
  log_proc_msg "Validating redis mounts"
  if [ ! -d "$REDIS_DATA_DIR" ]; then
    log_proc_msg "Creating data directory $REDIS_DATA_DIR"
    mkdir -p $REDIS_DATA_DIR
  else
    log_proc_msg "Data directory already present: $REDIS_DATA_DIR"
  fi

  if [ ! -d "$REDIS_CONFIG_DIR" ]; then
    log_proc_msg "Creating config directory $REDIS_CONFIG_DIR"
    mkdir -p $REDIS_CONFIG_DIR
  else
    log_proc_msg "Config directory already present: $REDIS_CONFIG_DIR"
  fi

  mkdir -p $REDIS_CONFIG_DIR/scripts
}

__update_redis_config() {
  log_proc_msg "Updating redis config files"
  cp -vr $INSTALLER_SCRIPTS_DIR/configs/redis.conf $REDIS_CONFIG_DIR/redis.conf
}

__find_cluster() {
  log_proc_msg "Checking for existing nodes"

  _router_get_topology

  if [ $is_request_success != true ]; then
    log_proc_msg "Error getting topology: $response"
    log_proc_msg "Status code: $response_status_code"
    exit 1 # This means the router isn't available.
  else
    local topology=$(echo $response | jq '.nodes')
    local router_addresses=$(echo $topology | jq 'keys[]')
    local router_count=$(echo $topology | jq 'length')

    local url_parse_path="$INSTALLER_SCRIPTS_DIR/urlParse.sh"
    local redis_cluster_ip=""

    for router in $router_addresses; do
      if [ -z "$redis_cluster_ip" ]; then
        local services=$(echo $topology | jq ".$router.services[]")
        local pipelines_service=""
        if [ ! -z "$services" ] && [ "$services" != "null" ]; then
          pipelines_service=$(echo $services | jq 'select(.service_id | startswith("jfpip"))')
        fi

        if [ ! -z "$pipelines_service" ] && [ "$pipelines_service" != "null" ]; then
          local ip=$($url_parse_path --hostname $router)

          if [ "$ip" != "$PIPELINES_SHARED_REDIS_IP" ]; then
            log_proc_msg "Checking connection to $ip"
            if nc -vz $ip $PIPELINES_SHARED_REDIS_PORT &>/dev/null; then
              log_proc_msg "Redis found $ip"
              redis_cluster_ip=$ip
            fi
          fi
        fi
      fi
    done

    if [ ! -z "$redis_cluster_ip" ]; then
      export REDIS_CLUSTER_IP=$redis_cluster_ip
    else
      log_proc_msg "No Redis cluster found"
    fi
  fi
}

__install_redis_compose() {
  source "$INSTALLER_SCRIPTS_DIR/compose/$REDIS_INSTALL_SCRIPT"
}

main() {
  log_proc_marker "Installing Redis"
  __validate_redis_envs
  __validate_redis_mounts
  __update_redis_config
  create_user_volume_dirs "$PIPELINES_SERVICE_UID:$PIPELINES_SERVICE_GID" \
  "$REDIS_DATA_DIR" \
  "$REDIS_CONFIG_DIR"
  if [ -z "$REDIS_CLUSTER_IP" ]; then
    __find_cluster
  fi
  __install_redis_compose
  log_proc_msg "Redis installed successfully"
}

main
