#!/bin/bash
set -e
set -o pipefail

export COMPONENT="vault"
export VAULT_DATA_DIR="$RUNTIME_DIR/data/$COMPONENT"
export VAULT_CONFIG_DIR="$CONFIG_DIR/$COMPONENT"
export LOGS_FILE="$LOGS_DIR/$COMPONENT.log"
export VAULT_INSTALL_SCRIPT="installVault.sh"
export VAULT_CONFIG="$VAULT_CONFIG_DIR/vault_config.hcl"
export PIPELINES_SHARED_VAULT_IMAGE="$PIPELINES_CORE_REGISTRYURL/$PIPELINES_SHARED_VAULT_IMAGE:$PIPELINES_CORE_CONTROLPLANEVERSION"

## Write logs of this script to component specific file
exec &> >(tee -a "$LOGS_FILE")

__validate_vault_envs() {
  log_proc_msg "Initializing vault environment variables"
  log_proc_msg "VAULT_DATA_DIR: $VAULT_DATA_DIR"
  log_proc_msg "VAULT_CONFIG_DIR: $VAULT_CONFIG_DIR"
  log_proc_msg "PIPELINES_SHARED_VAULT_IP: $PIPELINES_SHARED_VAULT_IP"
  log_proc_msg "PIPELINES_SHARED_VAULT_PORT: $PIPELINES_SHARED_VAULT_PORT"
  log_proc_msg "LOGS_FILE: $LOGS_FILE"
}

__validate_vault_mounts() {
  log_proc_msg "Validating vault mounts"
  if [ ! -d "$VAULT_DATA_DIR" ]; then
    log_proc_msg "Creating data directory $VAULT_DATA_DIR"
    mkdir -p $VAULT_DATA_DIR
  else
    log_proc_msg "Data directory already present: $VAULT_DATA_DIR"
  fi

  if [ ! -d "$VAULT_CONFIG_DIR" ]; then
    log_proc_msg "Creating config directory $VAULT_CONFIG_DIR"
    mkdir -p $VAULT_CONFIG_DIR
  else
    log_proc_msg "Config directory already present: $VAULT_CONFIG_DIR"
  fi

  mkdir -p $VAULT_CONFIG_DIR/scripts
}

__update_vault_creds() {
  log_proc_msg "Updating vault to database connection credentials"

  local vault_config_template="$INSTALLER_SCRIPTS_DIR/configs/vault_config.hcl.template"
  cp -vr $vault_config_template $VAULT_CONFIG

  log_proc_msg "Updating db connection string"
  local connection_string=$PIPELINES_SHARED_DB_CONNECTIONSTRING
  local path_end=${connection_string##*/}
  # postgresql server expects sslmode=require in the connection string
  # for ssl enabled postgres. If the connection string has
  # ssl=true we will replace with sslmode=require
  if [[ $connection_string == *"sslmode=require"* ]]; then
    : #passes the statement
  elif [[ $connection_string == *"ssl=true"* ]]; then
    connection_string="${connection_string/ssl=true/sslmode=require}"
  elif [[ $path_end == *"?"* ]]; then
    connection_string="${connection_string}&sslmode=disable"
  else
    connection_string="${connection_string}?sslmode=disable"
  fi

  local escaped_connection_string=$(sed 's/[&/\]/\\&/g' <<< "$connection_string")

  sed -i 's#{{CONNECTION_STRING}}#'$escaped_connection_string'#g' $VAULT_CONFIG

  log_proc_msg "Updating vault port"
  sed -i 's#{{PIPELINES_SHARED_VAULT_PORT}}#'$PIPELINES_SHARED_VAULT_PORT'#g' $VAULT_CONFIG

  log_proc_msg "Updating vault cluster port"
  sed -i 's#{{PIPELINES_SHARED_VAULT_CLUSTERPORT}}#'$PIPELINES_SHARED_VAULT_CLUSTERPORT'#g' $VAULT_CONFIG

  log_proc_msg "Updating vault address"
  sed -i 's#{{PIPELINES_SHARED_VAULT_IP}}#'$PIPELINES_SHARED_VAULT_IP'#g' $VAULT_CONFIG

}

__install_vault_compose() {
    source "$INSTALLER_SCRIPTS_DIR/compose/$VAULT_INSTALL_SCRIPT"
}

main() {
  log_proc_marker "Installing Vault"
  if check_vault_variables; then
    __validate_vault_envs
    __validate_vault_mounts
    __update_vault_creds
    __install_vault_compose
    log_proc_msg "Vault installed successfully"
  else
    log_proc_msg "Skipping vault installation as vault is externalized"
  fi
}

main
