'use strict';
var self = Adapter;
module.exports = self;

var async = require('async');
var request = require('request');

function Adapter(baseUrl, token) {
  this.token = token;
  this.baseUrl = baseUrl;
  if (!baseUrl)
    logger.warn('Vault API URL is required');
}

Adapter.prototype.get = function (relativeUrl, callback) {
  var opts = {
    method: 'GET',
    url: this.baseUrl + relativeUrl,
    headers: {
      'Content-Type': 'application/json; charset=utf-8',
      'X-Vault-Token': this.token,
      'User-Agent': 'JFrog Pipelines API',
      'Accept': 'application/json'
    },
    timeout: 180000
  };

  var bag = {
    opts: opts,
    relativeUrl: relativeUrl,
    token: this.token
  };

  bag.who = util.format('common|vault|%s|GET|url:%s|', self.name, relativeUrl);
  logger.debug(bag.who, 'Starting');

  async.series([
    _performCall.bind(null, bag),
    _parseResponse.bind(null, bag)
  ], function () {
    logger.debug(bag.who, 'Completed');
    return callback(bag.err, bag.parsedBody, bag.headerLinks, bag.res);
  });
};

Adapter.prototype.post = function (relativeUrl, json, callback) {
  var opts = {
    method: 'POST',
    url: this.baseUrl + relativeUrl,
    headers: {
      'Content-Type': 'application/json; charset=utf-8',
      'X-Vault-Token': this.token,
      'User-Agent': 'JFrog Pipelines API',
      'Accept': 'application/json'
    },
    json: json
  };

  var bag = {
    opts: opts,
    relativeUrl: relativeUrl,
    token: this.token
  };

  bag.who = util.format('common|vault|%s|POST|url:%s|', self.name, relativeUrl);
  logger.debug(bag.who, 'Starting');

  async.series([
    _performCall.bind(null, bag),
    _parseResponse.bind(null, bag)
  ], function () {
    logger.debug(bag.who, 'Completed');
    callback(bag.err, bag.parsedBody, bag.headerLinks, bag.res);
  });
};

Adapter.prototype.put = function (relativeUrl, json, callback) {
  var opts = {
    method: 'PUT',
    url: this.baseUrl + relativeUrl,
    headers: {
      'Content-Type': 'application/json; charset=utf-8',
      'X-Vault-Token': this.token,
      'User-Agent': 'JFrog Pipelines API',
      'Accept': 'application/json'
    },
    json: json
  };

  var bag = {
    opts: opts,
    relativeUrl: relativeUrl,
    token: this.token
  };

  bag.who = util.format('common|vault|%s|PUT|url:%s|', self.name, relativeUrl);
  logger.debug(bag.who, 'Starting');

  async.series([
    _performCall.bind(null, bag),
    _parseResponse.bind(null, bag)
  ], function () {
    logger.debug(bag.who, 'Completed');
    callback(bag.err, bag.parsedBody, bag.headerLinks, bag.res);
  });
};

Adapter.prototype.delete = function (relativeUrl, callback) {
  var opts = {
    method: 'DELETE',
    url: this.baseUrl + relativeUrl,
    headers: {
      'Content-Type': 'application/json; charset=utf-8',
      'X-Vault-Token': this.token,
      'User-Agent': 'JFrog Pipelines API',
      'Accept': 'application/json'
    }
  };

  var bag = {
    opts: opts,
    relativeUrl: relativeUrl,
    token: this.token
  };

  bag.who = util.format(
    'common|vault|%s|DELETE|url:%s|', self.name, relativeUrl);
  logger.debug(bag.who, 'Starting');

  async.series([
    _performCall.bind(null, bag),
    _parseResponse.bind(null, bag)
  ], function () {
    logger.debug(bag.who, 'Completed');
    callback(bag.err, bag.parsedBody, bag.res);
  });
};

// Debug
Adapter.prototype.getHealth = function (query, callback) {
  var relativeUrl = util.format('/v1/sys/health?%s', query);
  this.get(relativeUrl, callback);
};

// Secrets
Adapter.prototype.getSecret = function (key, callback) {
  var relativeUrl = util.format('/v1/%s', key);
  this.get(relativeUrl, callback);
};

Adapter.prototype.getSecretsList = function (key, callback) {
  var relativeUrl = util.format('/v1/%s?list=true', key);
  this.get(relativeUrl, callback);
};

Adapter.prototype.postSecret = function (key, secret, callback) {
  var relativeUrl = util.format('/v1/%s', key);
  this.post(relativeUrl, secret, callback);
};

Adapter.prototype.deleteSecret = function (key, callback) {
  var relativeUrl = util.format('/v1/%s', key);
  this.delete(relativeUrl, callback);
};

// Policies
Adapter.prototype.putPolicy = function (name, body, callback) {
  this.put('/v1/sys/policy/' + name, body, callback);
};

// Tokens
Adapter.prototype.postToken = function (body, callback) {
  this.post('/v1/auth/token/create', body, callback);
};

//
// Helper methods
//
function _performCall(bag, next) {
  var who = bag.who + '|' + _performCall.name;
  logger.debug(who, 'Inside');

  bag.startedAt = Date.now();
  var retryOpts = {
    times: 5,
    interval: function (retryCount) {
      return Math.pow(2, retryCount) * 1000;
    }
  };

  async.retry(retryOpts,
    function (callback) {
      request(bag.opts,
        function (err, res, body) {
          var interval = Date.now() - bag.startedAt;

          logger.debug(who, util.format('Request %s %s took %s ms and ' +
              'returned HTTP status %s', bag.opts.method, bag.opts.url,
              interval, res && res.statusCode)
          );

          bag.err = err;
          bag.res = res;
          bag.body = body;
          if (res && res.statusCode > 299) {
            bag.err = err || res.statusCode;
            logger.warn(who, util.format('Vault returned status %s', bag.err));
          }

          if (err || (res && res.statusCode >= 500)) {
            logger.error(who, util.format('Vault returned an error %s with ' +
              'status %s', err, (res && res.statusCode))
            );
            return callback(true);
          }

          return callback();
        }
      );
    },
    function () {
      return next();
    }
  );
}

function _parseResponse(bag, next) {
  var who = bag.who + '|' + _parseResponse.name;
  logger.debug(who, 'Inside');

  if (bag.body) {
    if (typeof bag.body === 'object') {
      bag.parsedBody = bag.body;
    } else {
      try {
        bag.parsedBody = JSON.parse(bag.body);
      } catch (e) {
        // treat body parsing errors only in case there was no other failure
        if (!bag.err) {
          logger.trace(who, 'Unable to parse bag.body', bag.body, e);
          bag.err = e;
        }
      }
    }
  }
  return next();
}
