'use strict';
var path = require('path');
var winston = require('winston');
var WinstonFileTransport = winston.transports.File;
var WinstonConsoleTransport = winston.transports.Console;
// Create the customize variant of `redactLogMessages` utility
var logFormatter = redactLogMessages();

configLevel();

exports = winston;
module.exports = winston;
global.logger = winston;

exports.configLevel = configLevel;

function configLevel(config) {
  winston.clear();

  config = config || {};

  var logsDir = process.env.LOGS_DIR;
  if (!logsDir)
    throw new Error( 'LOGS_DIR is not defined');

  var logLevel = 'debug';
  if (config.runMode  === 'beta') {
    logLevel = 'verbose';
  } else if (config.runMode === 'onebox') {
    logLevel = 'info';
  } else if (config.runMode === 'production') {
    logLevel = 'info';
  }

  winston.add(WinstonConsoleTransport, {
    timestamp: true,
    colorize: true,
    level: logLevel,
    // Pass every message to log formatter which can deal with sensitive
    // information.
    formatter: ({ message }) => logFormatter(message),
  });

  winston.add(WinstonFileTransport, {
    name: 'file#out',
    timestamp: true,
    colorize: true,
    filename: path.join(logsDir, 'installer.log'),
    maxsize: 10 * 1024 * 1024,// maxsize: 10mb
    maxFiles: 20,
    level: logLevel,
    json: false,
    tailable: true
  });
}
/**
 *
 * @param maskValue string
 *
 * `redactLogMessages` is helper method which can help you formatting
 * your logs, without exposing the sensitive information. It can
 * mask the information based on the customization.
 *
 * It uses various strategies for different type of string substitution
 * such as `connection strings`, `key value pair`, or `simple string`.
 */
function redactLogMessages(maskValue) {
  // If the user has not passed the maskValue, use a default one.
  maskValue = maskValue || '[REDACTED]';

  // List of predefined tokens which can be sensitive
  const SENSITIVE_TOKENS = ['passwd', 'password', 'secret', 'token', 'key'];

  // Precomputed list of all the regular expressions against the sensitive
  // tokens
  const SENSITIVE_TOKENS_REGEXPS = SENSITIVE_TOKENS.map(
    token =>
      new RegExp(`(.*)(${token})(:\\s*)([^\\s]+)($|\\s+\\S.*|\\s*)`, 'gi')
  );

  /**
   * All strategies that can deal with different type of string input
   *
   * Data structure -
   *
   *    {
   *      name: string // name of the strategy,
   *      regexp: RegExp (optional) // If the strategy has a regexp to work
   *      validate: function // method which checks if input string
   *                is eligible for the transformation,
   *      transform: function // method which actually mask the
   *                 sensitive information in the input string.
   *    }
   */
  const MASKING_STRATEGIES = [
    /**
     * Connection strings such as `jdbc`, `amqp`, `postgres` etc.
     * Generally looks like as -
     *  protocol://username:password@host:port/entity
     */
    {
      type: 'CONNECTION_STRING',
      regexp: /^(.*)\:\/\/(.*)(:.*)(@.*)$/,
      validate: function (input) {
        return typeof input === 'string' && this.regexp.test(input);
      },
      transform: function (input) {
        return input.replace(this.regexp, `$1://$2:${maskValue}$4`);
      },
    },
    /**
     * String containing key value pair information
     * Generally looks like as -
     *  KEY: VALUE
     */
    {
      name: 'STRING_PAIR',
      validate: input => typeof input === 'string',
      transform: input => {
        let result = input;

        // Mask each and every occurrence of sensitive tokens
        for (const regexp of SENSITIVE_TOKENS_REGEXPS) {
          result = result.replace(regexp, `$1$2$3${maskValue} $5`);
        }

        return result;
      },
    },
  ];

  /**
   * Return a curried method to the user, which when provided
   * with input string, can iterate all the available strategies
   * to mask the input string into best possible way
   */
  return input => {
    for (const strategy of MASKING_STRATEGIES) {
      if (strategy.validate(input)) {
        return strategy.transform(input);
      }
    }

    // If no strategy is valid for the input, return the original input.
    return input;
  };
}
