#!/bin/bash
set -e
set -o pipefail

readonly RABBITMQ_RESPONSE_FILE="$TMP_DIR/rabbitmq_response.tmp"

__rabbitmq_initialize() {
  local username="$1"
  ## NOTE: This logic handles initializing the adapter for prem installation as
  ## well as k8s cluster
  if [ "$username" == "guest" ]; then
    RABBITMQ_API_URL="localhost:$PIPELINES_SHARED_MSG_ADMINPORT/api"
  else
    RABBITMQ_API_URL="$PIPELINES_SHARED_MSG_IP:$PIPELINES_SHARED_MSG_ADMINPORT/api"
  fi
  RESPONSE_CODE=404
  RESPONSE_DATA=""
  CURL_EXIT_CODE=0

  touch $RABBITMQ_RESPONSE_FILE
}

__rabbitmq_get() {
  local username="$2"
  local password="$3"

  if [ -z "$username" ]; then
    log_proc_error "Missing RabbitMQ username"
    exit 1
  fi

  if [ -z "$password" ]; then
    log_proc_error "Missing RabbitMQ password"
    exit 1
  fi

  __rabbitmq_initialize $username
  local url="$RABBITMQ_API_URL/$1"

  local get_cmd="curl \
    -H \"Content-Type: application/json\" \
    -u $username:$password \
    -X GET $url \
    --silent --write-out \"%{http_code}\n\" \
    --output $RABBITMQ_RESPONSE_FILE"

  {
    RESPONSE_CODE=$(eval $get_cmd)
  } || {
    CURL_EXIT_CODE=$(echo $?)
  }

  is_request_success=false
  if [ $RESPONSE_CODE -lt 200 ] || [ $RESPONSE_CODE -gt 299 ]; then
    response="Failed request on $url. Error: $(cat $RABBITMQ_RESPONSE_FILE)"
    response_status_code="$RESPONSE_CODE"
  elif [ $CURL_EXIT_CODE -gt 0 ]; then
    # we are assuming that if curl cmd failed, API is unavailable
    response="curl failed with error code $CURL_EXIT_CODE. RabbitMQ might be down."
    response_status_code=503
  else
    is_request_success=true
    response_status_code="$RESPONSE_CODE"
    response=$(cat $RABBITMQ_RESPONSE_FILE)
  fi
}

__rabbitmq_put() {
  local username="$2"
  local password="$3"
  local update="$4"

  if [ -z "$username" ]; then
    log_proc_error "Missing RabbitMQ username"
    exit 1
  fi

  if [ -z "$password" ]; then
    log_proc_error "Missing RabbitMQ password"
    exit 1
  fi

  __rabbitmq_initialize $username
  local url="$RABBITMQ_API_URL/$1"

  local put_cmd="curl \
    -H \"Content-Type: application/json\" \
    -u $username:$password \
    -X PUT $url \
    -d '"$update"' \
    --silent --write-out \"%{http_code}\n\" \
    --output $RABBITMQ_RESPONSE_FILE"

  {
    RESPONSE_CODE=$(eval $put_cmd)
  } || {
    CURL_EXIT_CODE=$(echo $?)
  }

  is_request_success=false
  if [ $RESPONSE_CODE -lt 200 ] || [ $RESPONSE_CODE -gt 299 ]; then
    response="Failed request on $url. Error: $(cat $RABBITMQ_RESPONSE_FILE)"
    response_status_code="$RESPONSE_CODE"
  elif [ $CURL_EXIT_CODE -gt 0 ]; then
    # we are assuming that if curl cmd failed, API is unavailable
    response="curl failed with error code $CURL_EXIT_CODE. RabbitMQ might be down."
    response_status_code=503
  else
    is_request_success=true
    response_status_code="$RESPONSE_CODE"
    response=$(cat $RABBITMQ_RESPONSE_FILE)
  fi
}

## GET calls ###################################

_rabbitmq_node_healthchecks() {
  if [ "$#" -ne 2 ]; then
    log_proc_error "The number of arguments expected by _rabbitmq_node_healthchecks is 2"
    log_proc_error "current arguments $#"
    exit 1
  fi

  local username="$1"
  local password="$2"

  local ping_endpoint="healthchecks/node"
  __rabbitmq_get "$ping_endpoint" "$username" "$password"
}

## PUT calls ###################################

_rabbitmq_put_user() {
  if [ "$#" -ne 4 ]; then
    log_proc_error "The number of arguments expected by _rabbitmq_put_user is 4"
    log_proc_error "current arguments $#"
    exit 1
  fi

  local username="$1"
  local password="$2"
  local user_name="$3"
  local body="$4"

  if [ -z "$user_name" ]; then
    log_proc_error "User name is required"
    exit 1
  fi

  if [ -z "$body" ]; then
    log_proc_error "User configuration is required"
    exit 1
  fi

  __rabbitmq_put "users/$user_name" "$username" "$password"  "$body"
}

_rabbitmq_put_vhost() {
  if [ "$#" -ne 4 ]; then
    log_proc_error "The number of arguments expected by _rabbitmq_put_vhost is 4"
    log_proc_error "current arguments $#"
    exit 1
  fi

  local username="$1"
  local password="$2"
  local vhost_name="$3"
  local body="$4"

  if [ -z "$vhost_name" ]; then
    log_proc_error "Vhost name is required"
    exit 1
  fi

  if [ -z "$body" ]; then
    log_proc_error "Vhost configuration is required"
    exit 1
  fi

  __rabbitmq_put "vhosts/$vhost_name" "$username" "$password" "$body"
}

_rabbitmq_put_permission() {
  if [ "$#" -ne 5 ]; then
    log_proc_error "The number of arguments expected by _rabbitmq_put_permission is 5"
    log_proc_error "current arguments $#"
    exit 1
  fi

  local username="$1"
  local password="$2"
  local vhost_name="$3"
  local user_name="$4"
  local body="$5"

  if [ -z "$vhost_name" ]; then
    log_proc_error "Vhost name is required"
    exit 1
  fi

  if [ -z "$user_name" ]; then
    log_proc_error "User name is required"
    exit 1
  fi

  if [ -z "$body" ]; then
    log_proc_error "Permission configuration is required"
    exit 1
  fi

  __rabbitmq_put "permissions/$vhost_name/$user_name" "$username" "$password" "$body"
}

_rabbitmq_put_exchange() {
  if [ "$#" -ne 5 ]; then
    log_proc_error "The number of arguments expected by _rabbitmq_put_exchange is 5"
    log_proc_error "current arguments $#"
    exit 1
  fi

  local username="$1"
  local password="$2"
  local vhost_name="$3"
  local exchange_name="$4"
  local body="$5"

  if [ -z "$vhost_name" ]; then
    log_proc_error "Vhost name is required"
    exit 1
  fi

  if [ -z "$exchange_name" ]; then
    log_proc_error "Exchange name is required"
    exit 1
  fi

  if [ -z "$body" ]; then
    log_proc_error "Exhchange configuration is required"
    exit 1
  fi

  __rabbitmq_put "exchanges/$vhost_name/$exchange_name" "$username" "$password" "$body"
}

_rabbitmq_put_policy() {
  if [ "$#" -ne 5 ]; then
    log_proc_error "The number of arguments expected by _rabbitmq_put_policy is 5"
    log_proc_error "current arguments $#"
    exit 1
  fi

  local username="$1"
  local password="$2"
  local vhost_name="$3"
  local policy_name="$4"
  local body="$5"

  if [ -z "$vhost_name" ]; then
    log_proc_error "Vhost name is required"
    exit 1
  fi

  if [ -z "$policy_name" ]; then
    log_proc_error "Policy name is required"
    exit 1
  fi

  if [ -z "$body" ]; then
    log_proc_error "Policy configuration is required"
    exit 1
  fi

  __rabbitmq_put "policies/$vhost_name/$policy_name" "$username" "$password" "$body"
}
