'use strict';
var self = Adapter;
module.exports = self;

var util = require('util');
var async = require('async');
var request = require('request');
var _ = require('underscore');

var parseLinks = require('parse-links');

function Adapter(rtUrl, token) {
  if (_.isUndefined(token)) {
    console.error('Missing admin token, cannot initialize artifactory adapter');
    process.exit(1);
  }
  this.token = token;
  this.baseUrl = rtUrl;

  this.authorizationHeader = 'Bearer ' + token;
}

Adapter.prototype.get = function (relativeUrl, options, callback) {
  var opts = {
    method: 'GET',
    url: relativeUrl.indexOf('http') === 0 ? relativeUrl : this.baseUrl +
    relativeUrl,
    headers: {
      'Authorization': this.authorizationHeader,
      'Content-Type': 'application/json; charset=utf-8',
      'User-Agent': 'Pipelines Installer'
    },
    timeout: 180000
  };

  var bag = {
    opts: opts,
    relativeUrl: relativeUrl,
    token: this.token,
    retry: options && options.retry,
  };
  bag.who = util.format('rtAdapter|%s|GET|url:%s',
    self.name, relativeUrl);
  util.log('Starting', bag.who);

  async.series([
      _performCall.bind(null, bag),
      _parseResponse.bind(null, bag)
    ],
    function () {
      util.log('Completed', bag.who);
      callback(bag.err, bag.parsedBody, bag.headerLinks, bag.res);
    }
  );
};

// generic PUT method
Adapter.prototype.put = function (relativeUrl, json, callback) {
  var opts = {
    method: 'PUT',
    url: relativeUrl.indexOf('http') === 0 ? relativeUrl : this.baseUrl +
    relativeUrl,
    headers: {
      'Authorization': this.authorizationHeader,
      'Content-Type': 'application/json',
      'User-Agent': 'Pipelines Installer',
      'Accept': '*/*'
    },
    json: json,
    timeout: 180000
  };

  var bag = {
    opts: opts,
    relativeUrl: relativeUrl,
    token: this.token
  };
  bag.who = util.format('rtAdapter|%s|PUT|url:%s',
    self.name, relativeUrl);
  util.log('Starting', bag.who);

  async.series([
      _performCall.bind(null, bag),
      _parseResponse.bind(null, bag)
    ],
    function () {
      callback(bag.err, bag.parsedBody, bag.headerLinks, bag.res);
    }
  );
};

Adapter.prototype.delete = function (relativeUrl, callback) {
  var opts = {
    method: 'DELETE',
    url: relativeUrl.indexOf('http') === 0 ? relativeUrl : this.baseUrl +
    relativeUrl,
    headers: {
      'Authorization': this.authorizationHeader,
      'Content-Type': 'application/json',
      'User-Agent': 'Pipelines Installer'
    },
    timeout: 180000
  };

  var bag = {
    opts: opts,
    relativeUrl: relativeUrl,
    token: this.token,
  };
  bag.who = util.format('rtAdapter|%s|DELETE|url:%s',
    self.name, relativeUrl);
  util.log('Starting', bag.who);

  async.series([
      _performCall.bind(null, bag),
      _parseResponse.bind(null, bag)
    ],
    function () {
      util.log('Completed', bag.who);
      callback(bag.err, bag.parsedBody, bag.headerLinks, bag.res);
    }
  );
};

// common helper methods
function _performCall(bag, next) {
  var who = bag.who + '|' + _performCall.name;
  util.log('Inside', who);

  bag.startedAt = Date.now();

  var retryOpts = {
    times: bag.retry ? 5: 1,
    interval: function (retryCount) {
      return Math.pow(2, retryCount) * 1000;
    }
  };

  async.retry(retryOpts,
    function (callback) {
      request(bag.opts, function (err, res, body) {
        var interval = Date.now() - bag.startedAt;
        util.log(who, 'Pipelines request ' + bag.opts.method + ' ' +
          bag.opts.url + ' took ' + interval +
          ' ms and returned HTTP status ' + (res && res.statusCode));

        if (res && res.statusCode > 299)
          err = err || res.statusCode;
        if (err)
          util.log(who, 'Artifactory returned status', err);

        bag.err = err;
        bag.res = res;
        bag.body = body;
        return callback(err);
      });
    },
    function () {
      return next();
    }
  );
}

function _parseResponse(bag, next) {
  var who = bag.who + '|' + _parseResponse.name;
  util.log('Inside', who);

  if (bag.res && bag.res.headers.link) {
    bag.headerLinks = parseLinks(bag.res.headers.link);
  }

  if (bag.body) {
    if (typeof bag.body === 'object') {
      bag.parsedBody = bag.body;
    } else if (typeof bag.body === 'string') {
      bag.parsedBody = {
        response: bag.body
      };
    } else {
      try {
        bag.parsedBody = JSON.parse(bag.body);
      } catch (e) {
        // treat body parsing errors only in case there was no other failure
        if (!bag.err) {
          logger.trace(who, 'Unable to parse bag.body', bag.body, e);
          bag.err = e;
        }
      }
    }
  }
  return next();
}

Adapter.prototype.ping = function (callback) {
  this.get('/api/system/ping', {}, callback);
};

Adapter.prototype.getSystemLicenses = function (callback) {
  this.get('/api/system/licenses', {}, callback);
};

Adapter.prototype.getSystemConfiguration = function (callback) {
  this.get('/api/system/configuration', {}, callback);
};

Adapter.prototype.getProxies = function (callback) {
  this.get('/api/system/configuration/platform/proxies', {}, callback);
};

Adapter.prototype.getMailserver = function (callback) {
  this.get('/api/system/configuration/platform/mail', {}, callback);
};

Adapter.prototype.getServiceID = function (callback) {
  this.get('/api/system/service_id', {}, callback);
};

Adapter.prototype.getRepositories = function (callback) {
  this.get('/api/repositories', {}, callback);
};

Adapter.prototype.getUserInfo = function (callback) {
  this.get('/api/system/gateway/openid/userinfo', {}, callback);
};

Adapter.prototype.getUser = function (name, callback) {
  this.get('/api/users/' + name, {}, callback);
};

Adapter.prototype.getPermissions = function (callback) {
  this.get('/api/security/permissions', {}, callback);
};

Adapter.prototype.getPermissionTarget = function (name, callback) {
  this.get('/api/security/permissions/' + name, {}, callback);
};

Adapter.prototype.listFiles = function (path, callback) {
  var relUrl = util.format('/api/storage/%s?list&mdTimeStamps=1', path);
  this.get(relUrl, {}, callback);
};

Adapter.prototype.putRepository = function (name, body, callback) {
  var relUrl = util.format('/api/repositories/%s', name);
  this.put(relUrl, body, callback);
};

Adapter.prototype.deleteArtifactsByPath = function (path, callback) {
  var relUrl = util.format('/%s', path);
  this.delete(relUrl, callback);
};

Adapter.prototype.cleanFromTrash = function (path, callback) {
  var relUrl = util.format('/api/trash/clean/%s', path);
  this.delete(relUrl, callback);
};
