"use strict"

module.exports = utils;

var util = require('util');
var async = require('async');
var _ = require('underscore');
var fs = require('fs');
var path = require('path');

var jfrogAccess = require('@jfrog/access-nodejs-client');
var ConfigurationManager = require('@jfrog/configuration-manager');

var rtAdapter = require('./rtAdapter.js');
const { KEEP_ALIVE } = require('../constants');

var VALID_LICENSE_TYPES = [
  'Enterprise Plus',
  'Enterprise Plus Trial'
];

function utils() {
  var callUtil = process.env.CALL_UTIL;
  if (_.isUndefined(callUtil)) {
    console.log('Empty CALL_UTIL env, exiting.')
    process.exit(1);
  }

  var bag = {
    who: util.format('util: %s', callUtil),
    accessClient: null,
    accessClientContext: {
      token: ''
    },
    service: 'installer',
    productHome: '/opt/jfrog/pipelines',
    rt: {},
    output: {},
    outfile: '/opt/jfrog/pipelines/var/tmp/utils.out'
  };

  async.series([
      _getConfigs.bind(null, bag),
      _initializeAccessClient.bind(null, bag),
      _createRtToken.bind(null, bag),
      _initializeRtAdapter.bind(null, bag),
    ],
    function (err) {
      if (err) {
        console.error('Could not decrypt configuration: ' + util.inspect(err));
        process.exit(1);
      }
      switch (callUtil) {
        case 'Decrypt':
          console.log('Decrypting secure variable')
          async.series([
              _decryptSecureVariable.bind(null, bag),
              _writeOutput.bind(null, bag)
            ],
            function (utilError) {
              if (utilError) {
                console.log(
                  'Could not decrypt secure variable: ' +
                    util.inspect(utilError));
                process.exit(1);
              }
              process.exit(0);
            }
          );
          break;
        case 'PingArtifactory':
          console.log('Pinging artifactory server')
          async.series([
              _pingArtifactory.bind(null, bag),
              _writeOutput.bind(null, bag)
            ],
            function (utilError) {
              if (utilError) {
                console.log(
                  'Could not ping Artifactory server: ' +
                    util.inspect(utilError));
                process.exit(1);
              }
              process.exit(0);
            }
          );
          break;
        case 'ValidateLicenses':
          console.log('Getting artifactory licenses');
          async.series([
              _validateLicenses.bind(null, bag),
              _writeOutput.bind(null, bag)
            ],
            function (utilError) {
              if (utilError) {
                console.error('Failed to validate Artifactory licenses: ' + util.inspect(utilError));
                process.exit(1);
              }
              process.exit(0);
            }
          );
          break;
        case 'GetServiceID':
          console.log('Getting artifactory service ID');
          async.series([
              _getServiceId.bind(null, bag),
              _writeOutput.bind(null, bag)
            ],
            function (utilError) {
              if (utilError) {
                console.error('Failed to get Artifactory service ID: ' + util.inspect(utilError));
                process.exit(1);
              }
              process.exit(0);
            }
          );
          break;
        case 'GetProxies':
          console.log('Getting proxies');
          async.series([
              _getProxies.bind(null, bag),
              _writeOutput.bind(null, bag)
            ],
            function (utilError) {
              if (utilError) {
                console.error('Failed to get proxy information: ' + util.inspect(utilError));
                process.exit(1);
              }
              process.exit(0);
            }
          );
          break;
        case 'GetMailserverConfig':
          console.log('Getting Mailserver Config');
          async.series([
              _getMailserverConfig.bind(null, bag),
              _writeOutput.bind(null, bag)
            ],
            function (utilError) {
              if (utilError) {
                console.error('Failed to get mailserver information: ' + util.inspect(utilError));
                process.exit(1);
              }
              process.exit(0);
            }
          );
          break;

        default:
          console.log('Invalid or unimplemented CALL_UTIL function, exiting.');
          process.exit(1);
      }
    }
  );
}

function _decryptSecureVariable(bag, next) {
  var who = bag.who + '|' + _decryptSecureVariable.name;
  console.log(who, 'Inside');

  if (_.isUndefined(process.env.EXEC_SCRIPT_ARGS))
    return next('Invalid or no YAML env provided, cannot decrypt');

  bag.output.value = bag.configManager.get(process.env.EXEC_SCRIPT_ARGS);

  return next();
}

function _getMailserverConfig(bag, next) {
  var who = bag.who + '|' + _getMailserverConfig.name;
  console.log(who, 'Inside');

  bag.rtAdapter.getMailserver(
    function (err, data){
      if (err)
        return next(err);

      var mailServer = data.response;
      if (typeof (mailServer) === 'string') {
        try {
          mailServer = JSON.parse(mailServer);
        } catch (e) {
          return next('Unable to parse proxy information ' + util.inspect(e));
        }
      }

      if (!_.isUndefined(mailServer) && (mailServer.enabled === true))
        bag.output = mailServer;
      return next();
    }
  );
}

function _getProxies(bag, next){
  var who = bag.who + '|' + _getProxies.name;
  console.log(who, 'Inside');

  bag.rtAdapter.getProxies(
    function (err, data){
      if (err)
        return next(err);

      var proxies = data.response;
      if (typeof (proxies) === 'string') {
        try {
          proxies = JSON.parse(proxies);
          //nested object
          proxies = proxies.proxies;
        } catch (e) {
          return next('Unable to parse proxy information ' + util.inspect(e));
        }
      }

      if (!_.isEmpty(proxies)) {
        var systemProxy = _.find(proxies,
          function(proxy) {
            return proxy.platform_default == true;
          }
        );

        if (!_.isUndefined(systemProxy))
          bag.output = systemProxy;
      }

      return next();
    }
  );
}

function _validateLicenses(bag, next) {
  var who = bag.who + '|' + _validateLicenses.name;
  console.log(who, 'Inside');

  bag.rtAdapter.getSystemLicenses(
    function (err, data) {
      if (err)
        return next(err);

      var licenses = data.response;
      if (typeof (licenses) === 'string') {
        try {
          licenses = JSON.parse(licenses);
        } catch (e) {
          return next('Unable to parse licenses ' + util.inspect(e));
        }
      }

      if (!_.isUndefined(licenses.type)) {
        // license information returned as an object
        if (_.contains(VALID_LICENSE_TYPES, licenses.type)) {
          console.log('Valid license: ' + licenses.type);
          //set a default license hash if it's not returned from artifactory
          bag.output.licenseHash = 'abcdefghijklmnopqrstuvwxyzabcdefghijklmno';
        }
      } else if (_.isArray(licenses.licenses)) {
        // license information returned as an array
        _.each(licenses.licenses,
          function (license) {
            if (_.contains(VALID_LICENSE_TYPES, license.type)) {
              console.log('Valid license: ' + license.type);
              bag.output.licenseHash = license.licenseHash;
            }
          }
        );
      }

      if (_.isUndefined(bag.output.licenseHash))
        return next('Invalid or no license available for Artifactory');
      return next();
    }
  );
}

function _getServiceId(bag, next) {
  var who = bag.who + '|' + _getServiceId.name;
  console.log(who, 'Inside');

  bag.rtAdapter.getServiceID(
    function (err, data) {
      if (err)
        return next(err);

      var response = data.response;
      if (typeof (response) === 'string')
        bag.output.serviceId = data.response;

      return next();
    }
  );
}

function _pingArtifactory(bag, next) {
  var who = bag.who + '|' + _pingArtifactory.name;
  console.log(who, 'Inside');
  bag.rtAdapter.ping(
    function (err, data){
      if (err)
        return next(err);
      bag.output = data;
      return next();
    }
  );
}

// -------------------- Common helpers for all workflows --------------------//

function _writeOutput(bag, next) {
  var who = bag.who + '|' + _writeOutput.name;
  console.log(who, 'Inside');
  fs.writeFile(bag.outfile, JSON.stringify(bag.output), 'utf8',
    function (err) {
      return next();
    }
  );
}

function _initializeRtAdapter(bag, next) {
  var who = bag.who + '|' + _initializeRtAdapter.name;
  console.log(who, 'Inside');
  bag.rtUrl = 'http://localhost:' + bag.rtRouterAccessPort + '/artifactory';
  bag.rtAdapter = new rtAdapter(bag.rtUrl, bag.rt.accessToken.token);
  return next();
}

function _createRtToken(bag, next) {
  var who = bag.who + '|' + _createRtToken.name;
  console.log(who, 'Inside');

  var request = new jfrogAccess.CreateTokenRequestModel(
    'applied-permissions/groups:* api:*', false, 'admin', 'jfrt@*', 120);

  bag.accessClient.createToken(bag.accessClientContext, request)
    .then(function (response)  {
      bag.rt.accessToken = {
        token: response.accessToken,
        expiration: Date.now() + 90 * 1000
      };
      return next();
    })
    .catch(function (err) {
      console.log(bag.who,
        'Failed to create access token for artifactory', err);
      return next(err);
    }
  );
}

function _getConfigs(bag, next) {
  var who = bag.who + '|' + _getConfigs.name;
  console.log(who, 'Inside');
  var configManager = ConfigurationManager.default({
    productHome: bag.productHome,
    microservice: bag.service,
    aliases: {}
  });

  var rtRouterJoinKey = configManager.get('shared.router.joinKey');
  if (_.isEmpty(rtRouterJoinKey)) {
    console.log(who, 'Reading joinkey from file system');
    var joinKeyPath =
      path.join('/opt/jfrog/pipelines/var/etc/security','join.key');
    if (fs.existsSync(joinKeyPath)) {
      try {
        var encryptedJoinKey = fs.readFileSync(joinKeyPath, 'utf8').trim();
        try {
          rtRouterJoinKey =
            configManager.getDecryptedValue(encryptedJoinKey).trim();
        } catch (err) {
          console.log(who, 'Failed to decrypt joinkey with error : ' + err);
        }
      } catch (err) {
        console.log(who, 'Failed to read joinKey from file ' + joinKeyPath +
          'with error: ' + err);
      }
    } else {
      console.log(who, 'Failed to read joinKey from file ' + joinKeyPath);
      return next(true)
    }
  }
  bag.rtRouterJoinKey = rtRouterJoinKey;

  bag.rtRouterAccessPort = configManager.get('shared.router.accessPort');
  bag.rtServiceId = configManager.get('shared.artifactory.serviceId') || '';
  bag.nodeId = process.env.PIPELINES_NODE_ID || configManager.get('core.id');
  bag.rootBucket = configManager.get('runtime.rootBucket');
  bag.configManager = configManager;

  var splitServiceId = bag.rtServiceId.split('@');
  bag.installerServiceId = splitServiceId[0] + 'installer@' +
    (splitServiceId[1] || '');

  return next();
}

function _initializeAccessClient(bag, next) {
  var who = 'utils.js|' + _initializeAccessClient.name;
  console.log(who, 'Inside');

  var options = {
    joinKey: bag.rtRouterJoinKey,
    nodeId: bag.nodeId,
    serviceId: bag.installerServiceId,
    routerBaseUrl: 'localhost:' + bag.rtRouterAccessPort,
    accessBaseUrl: 'http://localhost:' + bag.rtRouterAccessPort + '/access',
    accessGrpcBaseUrl: 'localhost:' + bag.rtRouterAccessPort,
    numberOfRetries: 5,
    restHeaders: {'Connection': 'keep-alive', 'Keep-Alive': `timeout=${KEEP_ALIVE.TIMEOUT_SEC}, max=${KEEP_ALIVE.MAX_CONNECTIONS}`},
    tokenVerifyCacheTTLInSec: 300
  };

  bag.accessClientContext = {
    token: ''
  };

  bag.accessClient = new jfrogAccess.AccessClient(options);
  bag.accessClient.join(bag.accessClientContext).then(
    function (res) {
      console.log(who, 'Successfully initialized access client');
      bag.accessClientContext.token = res;
      return next();
    }
  ).catch(
    function (err) {
      bag.output = err;
      return next(err);
    }
  );
}

if (require.main === module)
  utils();
