#!/bin/bash
set -e
set -o pipefail

source $INSTALLER_LIB_DIR/rabbitmqAdapter.sh
source $INSTALLER_LIB_DIR/routerAdapter.sh
source $INSTALLER_LIB_DIR/vaultAdapter.sh

# Logger methods ##########################################
###########################################################
log_proc_marker() {
  local prompt="$@"
  echo ""
  echo "# $(date +"%T") #######################################"
  echo "# $prompt"
  echo "##################################################"
}

log_proc_msg() {
  local message="$@"
  echo "|___ $@"
}

log_proc_error() {
  local message="$1"
  local error="$2"
  local bold_red_text='\e[91m'
  local reset_text='\033[0m'

  echo -e "$bold_red_text|___ $message$reset_text"
  echo -e "     $error"
}

log_proc_warn() {
  local message="$@"
  local bold_yellow_text='\e[33m'
  local reset_text='\033[0m'

  echo -e "$bold_yellow_text|___ $message$reset_text"
}

log_proc_success() {
  local message="$@"
  local bold_green_text='\e[32m'
  local reset_text='\033[0m'

  echo -e "$bold_green_text|___ $message$reset_text"
}

print_help() {
  echo "

  Usage:
    ./pipelines <command> [flags]

  Examples:
    ./pipelines install

  Commands:
    install                 Run Pipelines installation
      --installer-ip                       Internal IP of the host [mandatory]
      --base-url-ui                        Unified UI URL
      --artifactory-joinkey                Join key to connect with Artifactory [mandatory]
      --api-url                            Use provided url for API [mandatory]
      --base-url                           Internal Artifactory URL
      --global-password                    Set one password for all services (db, rabbitmq, pipelines). Can be changed later
      --install-user                       User (and Group) that owns the installer generated files and folders ($USER by default) e.g. obie, obie:obie, 1001:obie, 1002:1002
      --artifactory-proxy                  Proxy server to use for connecting to Artifactory
      --artifactory-proxy-username         User for the proxy server
      --artifactory-proxy-password         Password for proxy server
      --artifactory-service-id             Service Id to register with Artifactory. Format 'jft@<id>'
      --image-registry-url                 Docker registry url for Pipelines component images
      --image-registry-creds               Path to a file containing Docker credentials for the image registry as an alternative to --image-registry-url
      --build-image-registry-url           Docker registry url for images used by default in Pipelines steps
      --state-bucket                       Root bucket name for storing state
      --skip-image-pull                    Do not pull images
      --db-connection-string               A connection string to use an existing Postgres database
      --vault-url                          URL to connect to an existing Vault
      --vault-root-token                   Root token of the existing Vault specified with --vault-url
      --no-verify-ssl                      If true, pass -no-verify-ssl flag to services
      --global-http-proxy                  HTTP proxy to be used in Pipelines in place of any proxy information fetched from Artifactory
      --global-https-proxy                 HTTPS proxy to be used in Pipelines in place of any proxy information fetched from Artifactory
      --global-no-proxy                    No proxy settings to be used in Pipelines in place of any proxy information fetched from Artifactory
      --access-control-allow-origins       API will return these as allowed origins. A comma-separated list of origins should be provided.
      --disable-call-home                  Disable call home functionality
      --enable-call-home                   Enables call home functionality if previously disabled
      --rabbitmq-health-check-interval     RabbitMQ health check interval in mins
      --artifactory-health-check-interval  Artifactory health check interval in mins
      --db-health-check-interval           Database health check interval in mins
      --config-backup-count                Number of upgrades for which to keep backup configurations
    upgrade                 Upgrade current installation
    restart                 Restart Pipelines
    clean                   Remove Pipelines components and files
    info                    Print information about current installation on console
      -j | --json                    Print info as json
    version                 Print current installation control and build plane versions
      -j | --json                    Print info as json
    help                    Print this message
  "
  exit 0
}

# Helper methods ##########################################
###########################################################
bootstrap_installer() {
  log_proc_marker "Bootstrapping pipelines"

  if [ ! -f "$CONTROLPLANEVERSION_FILE" ]; then
    log_proc_error "Missing control plane version file: $CONTROLPLANEVERSION_FILE"
    exit 1
  fi

  if [ ! -f "$BUILDPLANEVERSION_FILE" ]; then
    log_proc_error "Missing build plane version file: $BUILDPLANEVERSION_FILE"
    exit 1
  fi

  ################## load config file  #########################
  mkdir -p $SECURITY_DIR
  if [ ! -f "$PIPELINES_YAML_CONFIG" ]; then
    log_proc_msg "$PIPELINES_YAML_CONFIG does not exist, creating"
    cp -vr "$INSTALLER_SCRIPTS_DIR/configs/system.basic-template.yaml" "$PIPELINES_YAML_CONFIG"
    cp -vr "$INSTALLER_SCRIPTS_DIR/configs/system.basic-template.yaml" "$CONFIG_DIR"
    cp -vr "$INSTALLER_SCRIPTS_DIR/configs/system.full-template.yaml" "$CONFIG_DIR"
    log_proc_msg "Successfully created pipelines config"
  else
    log_proc_msg "System YAML file present in $PIPELINES_YAML_CONFIG"
  fi
  ##NOTE: this is required for router to decrypt the yaml
  chmod -cR 666 $PIPELINES_YAML_CONFIG

  ################## create logs directory ########################
  if [ ! -d "$LOGS_DIR" ]; then
    log_proc_msg "Creating logs directory"
    mkdir -p "$LOGS_DIR"
  fi

  ################## create temp directory  ########################
  if [ ! -d "$TMP_DIR" ]; then
    log_proc_msg "Creating tmp directory"
    mkdir -p "$TMP_DIR"
  fi

  ################## create master directory  ########################
  if [ ! -d "$CONFIG_DIR/master" ]; then
    log_proc_msg "Creating master directory"
    mkdir -p "$CONFIG_DIR/master"
  fi
}

bootstrap_platform_config() {
  log_proc_marker "Setting platform config"
  source /etc/os-release

  local architecture=$(uname -m)
  if [[ "$NAME" == "Red Hat Enterprise Linux Server" || "$NAME" == "Red Hat Enterprise Linux" ]]; then
    # For RHEL7, the OS value is defined as "Red Hat Enterprise Linux Server".
    # For RHEL8, the OS value is defined as "Red Hat Enterprise Linux". We want
    # the want this to be consistent with the other OS names, so we will be calling
    # this RHEL. Also in RHEL version comes as major.minor but we want to have
    # only major version in our directory name so trimming to have only major version only.
    local os="RHEL"
    local os_version=$(echo "$VERSION_ID" | cut -f 1 -d '.')
  else
    # For CentOS, the OS value is defined as "CentOS Linux". We only want
    # the first part of it, so ignore everything after a space. Ubuntu defines
    # it as "Ubuntu" so that will not change here.
    local os=$(echo "$NAME" | cut -f 1 -d ' ')
    local os_version="$VERSION_ID"
  fi

  local operating_system=$os"_"$os_version
  local operating_system_distribution

  if [ ! -z $VERSION_CODENAME ]; then
    operating_system_distribution=$VERSION_CODENAME
  elif [ ! -z $VERSION ]; then
    operating_system_distribution=$VERSION
  elif [ ! -z $PRETTY_NAME ]; then
    operating_system_distribution=$PRETTY_NAME
  elif [ ! -z $NAME ]; then
    operating_system_distribution=$NAME
  fi

  ### default architecture & os to x86_64, Ubuntu_18.04, if empty
  if [ -z "$architecture" ]; then
    architecture="x86_64"
  fi
  if [ -z "$operating_system" ]; then
    operating_system="Ubuntu_18.04"
  fi

  local version=$(docker version --format '{{.Server.Version}}')
  # split and extract major.minor version eg. to extract 17.06 from 17.06.0-ce
  local docker_version="${version%.*}"

  log_proc_msg "PIPELINES_CORE_ARCHITECTURE: $architecture"
  log_proc_msg "OS: $operating_system"
  log_proc_msg "DISTRIBUTION: $operating_system_distribution"
  log_proc_msg "PIPELINES_CORE_DOCKERVERSION: $docker_version"

  export PLATFORM_ARCHITECTURE=$architecture
  export PLATFORM_OS=$operating_system
  export PLATFORM_OSDISTRIBUTION=$operating_system_distribution
  export PLATFORM_DOCKER_VERSION=$docker_version
}

platform_exec() {
  local platform_call="$1"
  shift
  ## This is needed because the global IFS converts arguments into a list of
  ## strings, each in a new line
  local platform_call_args=$(echo "$@" | tr '\n' ' ')
  log_proc_msg "Executing $platform_call"

  local platform_script="$INSTALLER_SCRIPTS_DIR/$PLATFORM_ARCHITECTURE/$PLATFORM_OS/$platform_call.sh"
  local platform_helpers="$INSTALLER_SCRIPTS_DIR/$PLATFORM_ARCHITECTURE/$PLATFORM_OS/_helpers.sh"
  if [ -f "$platform_script" ]; then
    ## NOTE: we cannot parse arguments in "source .." command because of which
    ## the function is first source-ed and then eval-ed.
    source "$platform_script"
    eval "$platform_call" "$platform_call_args"
  elif [ -f "$platform_helpers" ]; then
    source "$platform_helpers"
    eval "$platform_call" "$platform_call_args"
  else
    log_proc_msg "Cannot execute $platform_call, missing file $platform_helpers"
    exit 1
  fi
}

set_config() {
  local config_key="$1"
  local config_value="$2"

  if [ -z "$config_key" ]; then
    log_proc_error "Cannot set empty key in config. Syntax 'set_config KEY VALUE'"
    exit 1
  fi

  if [ -z "$config_value" ]; then
    log_proc_error "Cannot set empty value in config for $config_key. Syntax 'set_config KEY VALUE'"
    exit 1
  fi

  sed -i 's#.*'"$config_key"'=.*#'"$config_key"'="'$config_value'"#g' $PIPELINES_CONFIG

  source "$PIPELINES_CONFIG"
}

__read_config() {
  local properties="$1"
  local val=$(yq e ".$properties" $PIPELINES_YAML_CONFIG)
  if [[ "$val" != "null" ]]; then
    echo $val
  else
    echo ""
  fi
}

: '
  purpose:- map json to env variables
  params:-  prefix, json
  description:- This function uses jq filter to flatten down json to env variables with prefix. 
    (e.g 
      input - PIPLINES, {"a":{"b":1}} 
      output - export PIPLINES_A_B = 1
    ) 
'
__obj_to_env_util() {
  local prefix="$1"
  local json="$2"
  eval "$( echo "$json" | jq --arg PREFIX "$prefix" -r 'path(..) as $p | getpath($p) |
    if type != "object" and type != "array" then
    "export " + $PREFIX + "_\([ $p[] | tostring ] | join("_") | ascii_upcase)=\"\(.)\""
    else "" end' )"
}

decrypt_secrets() {
  log_proc_marker "Setting decrypted envs for installation"

  if [ ! -z "$PIPELINES_SHARED_ROUTER_JOINKEY" ]; then
    log_proc_warn "decrypting join key with length ${#PIPELINES_SHARED_ROUTER_JOINKEY}"
    exec_util "Decrypt" "shared.router.joinKey"
    export PIPELINES_SHARED_ROUTER_JOINKEY=$(echo $response \
      | jq -r '.value')
    log_proc_warn "decrypted join key has length ${#PIPELINES_SHARED_ROUTER_JOINKEY}"
  fi

  if [ ! -z "$PIPELINES_CORE_INSTALLERAUTHTOKEN" ]; then
    exec_util "Decrypt" "core.installerAuthToken"
    export PIPELINES_CORE_INSTALLERAUTHTOKEN=$(echo $response \
      | jq -r '.value')
  fi

  if [ ! -z "$PIPELINES_SHARED_DB_PASSWORD" ]; then
    exec_util "Decrypt" "shared.db.password"
    export PIPELINES_SHARED_DB_PASSWORD=$(echo $response \
      | jq -r '.value')
  fi

  if [ ! -z "$PIPELINES_SHARED_REDIS_PASSWORD" ]; then
    exec_util "Decrypt" "shared.redis.password"
    export PIPELINES_SHARED_REDIS_PASSWORD=$(echo $response \
      | jq -r '.value')
  fi  

  if [ -z "$PIPELINES_SHARED_DB_CONNECTIONSTRING" ]; then
    export PIPELINES_SHARED_DB_CONNECTIONSTRING="postgres://$PIPELINES_SHARED_DB_USERNAME:$PIPELINES_SHARED_DB_PASSWORD@$PIPELINES_SHARED_DB_IP:$PIPELINES_SHARED_DB_PORT/$PIPELINES_SHARED_DB_NAME"
  fi

  if [ ! -z "$PIPELINES_SHARED_VAULT_TOKEN" ]; then
    exec_util "Decrypt" "shared.vault.token"
    export PIPELINES_SHARED_VAULT_TOKEN=$(echo $response \
      | jq -r '.value')
  fi

  if [ ! -z "$PIPELINES_SHARED_MSG_PASSWORD" ]; then
    exec_util "Decrypt" "shared.msg.password"
    export PIPELINES_SHARED_MSG_PASSWORD=$(echo $response \
      | jq -r '.value')
  fi

  if [ ! -z "$PIPELINES_SHARED_MSG_UI_PASSWORD" ]; then
    exec_util "Decrypt" "shared.msg.ui.password"
    export PIPELINES_SHARED_MSG_UI_PASSWORD=$(echo $response \
      | jq -r '.value')
  fi
}

load_system_config() {
  local json=$(yq e --tojson $PIPELINES_YAML_CONFIG)
  local defaultConfigJson=$(yq e --tojson $INSTALLER_SCRIPTS_DIR/configs/system.basic-template.yaml)
  local jsonVal=$(echo $json | jq -r .shared)
  if [[ "$jsonVal" == "" || "$jsonVal" == "null" ]]; then
    log_proc_error "$PIPELINES_YAML_CONFIG shared section cannot be empty"
    exit 1
  fi
  jsonVal=$(echo $json | jq -r .core)
  if [[ "$jsonVal" == "" || "$jsonVal" == "null" ]]; then
    log_proc_error "$PIPELINES_YAML_CONFIG core section cannot be empty"
    exit 1
  fi
  jsonVal=$(echo $json | jq -r .runtime)
  if [[ "$jsonVal" == "" || "$jsonVal" == "null" ]]; then
    log_proc_error "$PIPELINES_YAML_CONFIG runtime section cannot be empty"
    exit 1
  fi
  jsonVal=$(echo "$json" | jq -r .shared.observability)
  if [[ "$jsonVal" == "" || "$jsonVal" == "null" ]]; then
    local defaultObservabilityImageName=$(echo $defaultConfigJson | jq -r .shared.observability.image)
    local defaultObservabilityImageTag=$(echo $defaultConfigJson | jq -r .shared.observability.tag)
    set_system_config "shared.observability.image" "$defaultObservabilityImageName"
    set_system_config "shared.observability.tag" "$defaultObservabilityImageTag"
  fi
  jsonVal=$(echo $json | jq -r .core.allowCallHome)
  if [[ "$jsonVal" == "" || "$jsonVal" == "null" ]]; then
    local defaultCallHome=$(echo $defaultConfigJson | jq -r .core.allowCallHome)
    local runMode=$(echo $json | jq -r .core.runMode)
    if [[ "$runMode" == "devmode" ]]; then
      set_system_config "core.allowCallHome" "false"
    else
      set_system_config "core.allowCallHome" "$defaultCallHome"
    fi
  fi
  jsonVal=$(echo $json | jq -r .core.callHomeUrl)
  if [[ "$jsonVal" == "" || "$jsonVal" == "null" ]]; then
    local defaultCallHomeUrl=$(echo $defaultConfigJson | jq -r '.core.callHomeUrl')
    set_system_config "core.callHomeUrl" "$defaultCallHomeUrl"
  fi
  jsonVal=$(echo $json | jq -r .core.rabbitmqHealthCheckIntervalInMins)
  if [[ "$jsonVal" == "" || "$jsonVal" == "null" ]]; then
    local defaultRabbitmqHealthCheckIntervalInMins=$(echo $defaultConfigJson | jq -r '.core.rabbitmqHealthCheckIntervalInMins')
    set_system_config "core.rabbitmqHealthCheckIntervalInMins" "$defaultRabbitmqHealthCheckIntervalInMins"
  fi
  jsonVal=$(echo $json | jq -r .core.artifactoryHealthCheckIntervalInMins)
  if [[ "$jsonVal" == "" || "$jsonVal" == "null" ]]; then
    local defaultArtifactoryHealthCheckIntervalInMins=$(echo $defaultConfigJson | jq -r '.core.artifactoryHealthCheckIntervalInMins')
    set_system_config "core.artifactoryHealthCheckIntervalInMins" "$defaultArtifactoryHealthCheckIntervalInMins"
  fi
  jsonVal=$(echo $json | jq -r .core.dbHealthCheckIntervalInMins)
  if [[ "$jsonVal" == "" || "$jsonVal" == "null" ]]; then
    local defaultDbHealthCheckIntervalInMins=$(echo $defaultConfigJson | jq -r '.core.dbHealthCheckIntervalInMins')
    set_system_config "core.dbHealthCheckIntervalInMins" "$defaultDbHealthCheckIntervalInMins"
  fi
  jsonVal=$(echo $json | jq -r .core.dbHealthCheckTimeoutInSeconds)
  if [[ "$jsonVal" == "" || "$jsonVal" == "null" ]]; then
    local defaultDbHealthCheckTimeoutInSeconds=$(echo $defaultConfigJson | jq -r '.core.dbHealthCheckTimeoutInSeconds')
    set_system_config "core.dbHealthCheckTimeoutInSeconds" "$defaultDbHealthCheckTimeoutInSeconds"
  fi
  jsonVal=$(echo $json | jq -r .runtime.nodePollerIntervalMS)
  if [[ "$jsonVal" == "" || "$jsonVal" == "null" ]]; then
    local defaultNodePollerIntervalMS=$(echo $defaultConfigJson | jq -r '.runtime.nodePollerIntervalMS')
    set_system_config "runtime.nodePollerIntervalMS" "$defaultNodePollerIntervalMS"
  fi
  __obj_to_env_util "PIPELINES" "$json"
}

set_system_config() {
  local yml_key="$1"
  local val=$(echo -e "$2")
  local env_key="$3"
  local operator="$4"

  if [ $# -lt 2 ]; then
    log_proc_error "Invalid number of arguments for setting system config"
    log_proc_error "Arguments passed: key: $yml_key, val: $val"
    log_proc_error "Syntax: set_system_config 'key' 'val' ['env']"
    exit 1
  fi

  if [ -z "$operator" ]; then
    operator="="
  fi

  if [[ "$val" == "[]" || "$val" == "true" || "$val" == "false" ]]; then
    yq e ".$yml_key $operator $val" -i $PIPELINES_YAML_CONFIG
  else
    yq e ".$yml_key $operator \"$val\"" -i $PIPELINES_YAML_CONFIG
  fi

  if [ -z "$env_key" ]; then
    env_key=$(echo "pipelines."$yml_key | tr . _)
    env_key=${env_key^^}
  fi
  eval "export $env_key=\"$val\""
}

set_platform_config() {
  if [ -z "$PIPELINES_CORE_OS" ]; then
    set_system_config "core.os" "$PLATFORM_OS"
  fi

  if [ -z "$PIPELINES_CORE_OSDISTRIBUTION" ]; then
    set_system_config "core.osDistribution" "$PLATFORM_OSDISTRIBUTION"
  fi

  if [ -z "$PIPELINES_CORE_ARCHITECTURE" ]; then
    set_system_config "core.architecture" "$PLATFORM_ARCHITECTURE"
  fi

  if [ -z "$PIPELINES_CORE_DOCKERVERSION" ]; then
    set_system_config "core.dockerVersion" "$PLATFORM_DOCKER_VERSION"
  fi
}

split_connection_string() {
  # connection string will always contain username and password in it
  # as we dont support any other way for pipelines to authenticate
  # with the db
  local db_type=$(echo $PIPELINES_SHARED_DB_CONNECTIONSTRING | cut -d ':' -f 1)
  set_system_config "shared.db.type" "$db_type"

  local db_without_type="$(echo ${PIPELINES_SHARED_DB_CONNECTIONSTRING/$db_type:\/\//})"
  local db_username=$(echo $db_without_type | cut -d ':' -f 1)
  set_system_config "shared.db.username" "$db_username"

  local db_without_username="$(echo ${db_without_type/$db_username:/})"
  local db_password=$(echo $db_without_username | cut -d '@' -f 1)
  set_system_config "shared.db.password" "$db_password"

  local db_without_password="$(echo ${db_without_username/$db_password@/})"
  local db_ip_port=$(echo $db_without_password | cut -d '/' -f 1)
  local db_name=$(echo $db_without_password | cut -d '/' -f 2)
  set_system_config "shared.db.name" "$db_name"

  local db_port=$(echo $db_ip_port | grep ':' | cut -d ':' -f 2)
  if [ ! -z $db_port ]; then
    local db_ip=$(echo $db_ip_port | cut -d ':' -f 1)
    set_system_config "shared.db.ip" "$db_ip"
    set_system_config "shared.db.port" "$db_port"
  else
    set_system_config "shared.db.ip" "$db_ip_port"
    set_system_config "shared.db.port" ""
  fi
}

validate_ip() {
  local ip=$1
  local rx='([1-9]?[0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])'
  [[ $ip =~ ^(http(s)?://)?([a-zA-Z0-9-]+(\.[a-zA-Z0-9-]+)+[a-zA-Z0-9/?]*)$ ||  $ip =~ ^$rx\.$rx\.$rx\.$rx$ ]];
}

validate_db_password() {
  local pwd=$1
  if [[ "$pwd" =~ [^-_+=a-zA-Z0-9] ]]; then
    return 1
  fi
  return 0
}

validate_nonempty_option() {
  local option_name="$1"
  local option_value="$2"
  if [[ $option_value == --* ]] || [ -z "$option_value" ]; then
    log_proc_error "$option_name cannot be empty"
    exit 1
  fi
}


add_language_images() {
  log_proc_marker "Adding languageImages to system.yaml"
  local system_yml_template="$INSTALLER_SCRIPTS_DIR/configs/system.full-template.yaml"
  local langugageImages=$(yq e ".runtime.languageImages" $PIPELINES_YAML_CONFIG)
  local language_image_count=$(echo "$langugageImages" | yq e "length" -)

  local index=0
  local no_image_remaining=false
  while [ $no_image_remaining != true ]; do
    local image=$(yq e ".runtime.languageImages[$index]" $system_yml_template)
    if [ "$image" == "null" ]; then
      no_image_remaining=true
    else
      local architecture=$(yq e ".runtime.languageImages[$index].architecture" $system_yml_template)
      local os=$(yq e ".runtime.languageImages[$index].os" $system_yml_template)
      local language=$(yq e ".runtime.languageImages[$index].language" $system_yml_template)

      local existing_yaml_index=0
      local image_found=false
      while [ $image_found != true ] && [ $existing_yaml_index -lt $language_image_count ]; do
        local existing_architecture=$(yq e ".runtime.languageImages[$existing_yaml_index].architecture" $PIPELINES_YAML_CONFIG)
        local existing_os=$(yq e ".runtime.languageImages[$existing_yaml_index].os" $PIPELINES_YAML_CONFIG)
        local existing_language=$(yq e  ".runtime.languageImages[$existing_yaml_index].language" $PIPELINES_YAML_CONFIG)
        if [ $existing_architecture == $architecture ] && [ $existing_os == $os ] && [ $existing_language == $language ]; then
          image_found=true
        fi
        let "existing_yaml_index = $existing_yaml_index + 1"
      done

      if [ $image_found == false ]; then
        yq e ".runtime.languageImages[$language_image_count].architecture = \"$architecture\"" -i $PIPELINES_YAML_CONFIG
        yq e ".runtime.languageImages[$language_image_count].os = \"$os\"" -i $PIPELINES_YAML_CONFIG
        yq e ".runtime.languageImages[$language_image_count].language = \"$language\"" -i $PIPELINES_YAML_CONFIG
        if [ -z "$PIPELINES_RUNTIME_LANGUAGEIMAGES_REGISTRYURL" ]; then
          local registryUrl=$(yq e ".runtime.languageImages[$index].registryUrl" $system_yml_template)
          yq e ".runtime.languageImages[$language_image_count].registryUrl = \"$registryUrl\"" -i $PIPELINES_YAML_CONFIG
        else
          yq e ".runtime.languageImages[$language_image_count].registryUrl = \"$PIPELINES_RUNTIME_LANGUAGEIMAGES_REGISTRYURL\"" -i $PIPELINES_YAML_CONFIG
        fi
        local imageValue=$(yq e ".runtime.languageImages[$index].image" $system_yml_template)
        yq e ".runtime.languageImages[$language_image_count].image = \"$imageValue\"" -i $PIPELINES_YAML_CONFIG
        local is_default=$(yq e ".runtime.languageImages[$index].isDefault" $system_yml_template)
        if [ $is_default == true ]; then
          yq e ".runtime.languageImages[$language_image_count].isDefault = $is_default" -i $PIPELINES_YAML_CONFIG
        fi
        local defaultVersion=$(yq e ".runtime.languageImages[$index].defaultVersion" $system_yml_template)
        yq e ".runtime.languageImages[$language_image_count].defaultVersion = \"$defaultVersion\"" -i $PIPELINES_YAML_CONFIG
        let "language_image_count = $language_image_count + 1"
      fi
    fi
    let "index = $index + 1"
  done
}

update_services_list() {
  log_proc_marker "Updating services in system.yaml"
  local system_yml_template="$INSTALLER_SCRIPTS_DIR/configs/system.full-template.yaml"

  # Add services in the template and not the system.yaml
  local service_names=$(yq e ".core.services.*.name" $system_yml_template)
  echo "$service_names" | while read service_name; do
    local existing_service=$(yq e ".core.services.$service_name" $PIPELINES_YAML_CONFIG)
    if [ "$existing_service" == "null" ]; then
      local service_image=$(yq e ".core.services.$service_name.image" $system_yml_template)
      local service_port=$(yq e ".core.services.$service_name.port" $system_yml_template)
      set_system_config "core.services.$service_name.name" "$service_name"
      set_system_config "core.services.$service_name.image" "$service_image"

      if [ "$service_port" != "null" ]; then
        set_system_config "core.services.$service_name.port" "$service_port"
      fi
    fi
  done

  # Remove services in the system.yaml and not the template
  local service_names=$(yq e ".core.services.*.name" $PIPELINES_YAML_CONFIG)
  echo "$service_names" | while read service_name; do
    local template_service=$(yq e ".core.services.$service_name" $system_yml_template)
    if [ "$template_service" == "null" ]; then
      yq e "del(.core.services.$service_name)" -i $PIPELINES_YAML_CONFIG
    fi
  done
}

update_queues_list() {
  log_proc_marker "Updating queues in system.yaml"
  local system_yml_template="$INSTALLER_SCRIPTS_DIR/configs/system.full-template.yaml"

  # Reset queues
  set_system_config "shared.msg.queues" "[]"

  # Add queues from the template
  local queue_count=0
  local all_queues_present=false
  while [ $all_queues_present != true ]; do
    local queue_name=$(yq e ".shared.msg.queues[$queue_count]" $system_yml_template)
    if [ "$queue_name" == "null" ]; then
      all_queues_present=true
    else
      set_system_config "shared.msg.queues" "$queue_name" "PIPELINES_SHARED_MSG_QUEUES_$queue_count" "+="
      let "queue_count = $queue_count + 1"
    fi
  done
}

boot_stateless_services() {
  local service_names=$(yq e ".core.services.*.name" $PIPELINES_YAML_CONFIG)
  echo "$service_names" | while read service_name; do
    if [ "$service_name" != "api" ] && [ "$service_name" != "www" ]; then
      log_proc_msg "Booting: $service_name"
      exec_util "CreateService" "$service_name"
    fi
  done
}

stop_stateless_services() {
  local service_names=$(yq e ".core.services.*.name" $PIPELINES_YAML_CONFIG)
  echo "$service_names" | while read service_name; do
    if [ "$service_name" != "api" ] && [ "$service_name" != "www" ]; then
      local service=$(echo "$service_name" | awk '{print tolower($0)}')
      log_proc_msg "Stopping: $service"
      docker rm -f $service || true
    fi
  done
}

stop_stateful_services() {
  local services=("api" "www")
  for service in ${services[@]}; do
    log_proc_msg "Stopping: $service"
    # These services must be allowed to shut down before removal.
    docker stop $service || true
    docker rm -f $service || true
  done
}

check_vault_variables() {
  # If these variables are set, we installed, or should install, the vault.
  if [ -z "$PIPELINES_SHARED_VAULT_IP" ]; then
    return 1
  fi

  return 0
}

check_db_variables() {
  # If these variables are set, we installed, or should install, the database.
  if [ "$PIPELINES_SHARED_DB_ISEXTERNAL" == "true" ]; then
    return 1
  fi

  if [ -z "$PIPELINES_SHARED_DB_IP" ]; then
    return 1
  fi

  if [ -z "$PIPELINES_SHARED_DB_PORT" ]; then
    return 1
  fi

  if [ -z "$PIPELINES_SHARED_DB_USERNAME" ]; then
    return 1
  fi

  if [ -z "$PIPELINES_SHARED_DB_NAME" ]; then
    return 1
  fi

  if [ -z "$PIPELINES_SHARED_DB_PASSWORD" ]; then
    return 1
  fi

  if [ -z "$PIPELINES_SHARED_DB_TYPE" ]; then
    return 1
  fi

  return 0
}

test_db_connection() {
  local create_table_cmd="psql $PIPELINES_SHARED_DB_CONNECTIONSTRING \
    -v ON_ERROR_STOP=1 \
    -c 'CREATE TABLE \"pipelinesTestTable\" (\"testColumn\" integer PRIMARY KEY);'"
  local drop_table_cmd="psql $PIPELINES_SHARED_DB_CONNECTIONSTRING \
    -v ON_ERROR_STOP=1 \
    -c 'DROP TABLE \"pipelinesTestTable\";'"

  log_proc_msg "Creating a test table to verify database connection"
  eval "$create_table_cmd"
  log_proc_msg "Dropping a test table"
  eval "$drop_table_cmd"
  log_proc_msg "Successfully tested connection to Postgres"
}

ping_artifactory() {
  local url="$1"
  if [ -z "$url" ]; then
    log_proc_error "No URL provided for Artifactory"
    exit 1
  else
    if [[ $url =~ /$ ]]; then
      url=${url%/}
    fi
    url="$url/artifactory/api/system/ping"
  fi

  local url_setting="$2"

  if [ -z "$url_setting" ]; then
    url_setting="the CLI settings"
  fi

  local artifactory_response_file=$TMP_DIR/artifactory_response.tmp
  local artifactory_response_code=0

  touch $artifactory_response_file
  echo "" > $artifactory_response_file
  {
    if [ "$PIPELINES_CORE_NOVERIFYSSL" == "true" ]; then
      artifactory_response_code=$(curl $additional_curl_options \
        -H "Content-Type: application/json" \
        -X GET "$url" --insecure \
        --silent --write-out "%{http_code}\n" \
        --output $artifactory_response_file)
    else
      artifactory_response_code=$(curl $additional_curl_options \
        -H "Content-Type: application/json" \
        -X GET "$url" \
        --silent --write-out "%{http_code}\n" \
        --output $artifactory_response_file)
    fi
  } || {
    log_proc_error "Artifactory server ping failed for URL $url"
    log_proc_error "Please check that the URL provided in $url_setting is correct."
    log_proc_error "For example, http://<artifactory host>:8082."
    exit 1
  }

  if [ $artifactory_response_code -ne 200 ] || [ "$(cat $artifactory_response_file)" != "OK" ]; then
    log_proc_error "Artifactory server ping failed for URL $url with status $artifactory_response_code and response: \n$(cat $artifactory_response_file)"
    log_proc_error "Please check that the URL provided in $url_setting is correct."
    log_proc_error "For example, http://<artifactory host>:8082."
    exit 1
  fi

  rm $artifactory_response_file
}

exec_util() {
	local util="$1"
	if [ -z "$util" ]; then
		log_proc_error "No utility function provided"
		exit 1
	fi

  local args="$2"

  local image_tag="$PIPELINES_CORE_CONTROLPLANEVERSION"
  local envs=" -e CONFIG_DIR=$CONFIG_DIR \
    -e LOGS_DIR=/opt/jfrog/pipelines/var/log"

	if [ ! -z "$args" ]; then
    envs="$envs -e EXEC_SCRIPT_ARGS=$args"
	fi

  if [ ! -z "$UPGRADE_VERSION" ]; then
    image_tag="$UPGRADE_VERSION" # Defined during upgrades
  fi

  if [ ! -z "$PIPELINES_CORE_PROXY_HTTPPROXY" ]; then
    envs="$envs \
      -e http_proxy=$PIPELINES_CORE_PROXY_HTTPPROXY"
  fi

  if [ ! -z "$PIPELINES_CORE_PROXY_HTTPSPROXY" ]; then
    envs="$envs \
      -e https_proxy=$PIPELINES_CORE_PROXY_HTTPSPROXY"
  fi

  if [ ! -z "$PIPELINES_CORE_PROXY_NOPROXY" ]; then
    envs="$envs \
      -e no_proxy=$PIPELINES_CORE_PROXY_NOPROXY"
  fi

  if [ "$PIPELINES_CORE_NOVERIFYSSL" == true ]; then
    envs="$envs \
      -e NODE_TLS_REJECT_UNAUTHORIZED=0"
  fi

  local mounts="-v $RUNTIME_DIR:/opt/jfrog/pipelines/var \
    -v $CONFIG_DIR:/opt/jfrog/pipelines/var/etc:rw \
    -v $DATA_DIR:/opt/jfrog/pipelines/var/data:rw \
    -v $LOGS_DIR:/opt/jfrog/pipelines/var/log:rw \
    -v /var/run/docker.sock:/var/run/docker.sock"

  local installer_image="$PIPELINES_CORE_REGISTRYURL/$PIPELINES_CORE_INSTALLERIMAGE:$image_tag"
  local out_file="$TMP_DIR/utils.out"

  local run_cmd="docker run \
		$envs \
    $mounts \
    --rm \
    --net=host \
    --user=0 \
    --name=pipelines-init \
    $installer_image $util"

  eval $run_cmd
  response=$(cat $out_file)
}

create_volume_dirs() {
  log_proc_msg "Creating mounted volume directories"
  mkdir -p \
  "$CONFIG_DIR" \
  "$LOGS_DIR"

  chown -R "$PIPELINES_SERVICE_UID:$PIPELINES_SERVICE_GID" \
  "$CONFIG_DIR" \
  "$LOGS_DIR"
}

# accepts arguments $host $port $serviceName $timeout
# $timeout is optional and defaults to 60 seconds if not defined
check_service_connection() {

  if [ "$#" != "3" ] && [ "$#" != "4" ]; then
    log_proc_error "invalid arguments to check_service_connection()"
    log_proc_error "Usage: check_service_connection <host> <port> <serviceName> <timeout (optional)>"
    exit 1
  fi

  local timeout=$4
  if [ -z "$timeout" ]; then
    timeout=60
  fi
  local host=$1
  if [ -z "$host" ]; then
    log_proc_error "got empty host in check_service_connection(), exiting..."
    exit 1
  fi
  local port=$2
  if [ -z "$port" ]; then
    log_proc_error "got empty port in check_service_connection(), exiting..."
    exit 1
  fi
  local service=$3
  if [ -z "$service" ]; then
    log_proc_error "got empty service name in check_service_connection(), exiting..."
    exit 1
  fi
  local interval=3
  local counter=0
  local service_booted=false

  while [ $service_booted != true ] && [ $counter -lt $timeout ]; do
    if nc -vz $host $port &>/dev/null; then
      log_proc_msg "$service found"
      sleep 5
      service_booted=true
    else
      log_proc_msg "Waiting for $service to start"
      let "counter = $counter + $interval"
      sleep $interval
    fi
  done
  if [ $service_booted = false ]; then
    log_proc_error "Could not detect $service container for host:$host, port:$port"
    exit 1
  fi
}

log_system_info() {
  local format="$1"
  if [ "$format" == "--json" ] || [ "$format" == "-j" ]; then
    format="json"
  fi
  if [ "$format" == "json" ]; then
    local jq_args=( )
    local jq_query='.'

    jq_args+=( --arg "controlplaneVersion" "$PIPELINES_CORE_CONTROLPLANEVERSION" )
    jq_query+=" | .controlplaneVersion=\$controlplaneVersion"

    jq_args+=( --arg "buildplaneVersion" "$PIPELINES_CORE_BUILDPLANEVERSION" )
    jq_query+=" | .buildplaneVersion=\$buildplaneVersion"

    jq_args+=( --arg "installMode" "$PIPELINES_CORE_RUNMODE" )
    jq_query+=" | .installMode=\$installMode"

    jq_args+=( --arg "artifactoryUrl" "$PIPELINES_SHARED_ARTIFACTORY_BASEURL" )
    jq_query+=" | .artifactoryUrl=\$artifactoryUrl"

    jq_args+=( --arg "unifiedUIUrl" "$PIPELINES_SHARED_ARTIFACTORY_BASEURLUI" )
    jq_query+=" | .unifiedUIUrl=\$unifiedUIUrl"

    jq_args+=( --arg "serviceId" "$PIPELINES_SHARED_ARTIFACTORY_SERVICEID" )
    jq_query+=" | .serviceId=\$serviceId"

    jq_args+=( --arg "apiUrl" "$PIPELINES_CORE_SERVICES_API_EXTERNALURL" )
    jq_query+=" | .apiUrl=\$apiUrl"

    jq_args+=( --arg "databaseEndpoint" "$PIPELINES_SHARED_DB_CONNECTIONSTRING" )
    jq_query+=" | .databaseEndpoint=\$databaseEndpoint"

    jq_args+=( --arg "vaultServer" "$PIPELINES_SHARED_VAULT_IP" )
    jq_query+=" | .vaultServer=\$vaultServer"

    jq_args+=( --arg "rabbitMQServer" "$PIPELINES_SHARED_MSG_IP" )
    jq_query+=" | .rabbitMQServer=\$rabbitMQServer"

    jq_args+=( --arg "redisServer" "$PIPELINES_SHARED_REDIS_IP" )
    jq_query+=" | .redisServer=\$redisServer"

    jq_args+=( --arg "lastUpgrade" "$PIPELINES_CORE_LASTUPDATETIME" )
    jq_query+=" | .lastUpgrade=\$lastUpgrade"

    jq_args+=( --arg "operatingSystem" "$PIPELINES_CORE_OS" )
    jq_query+=" | .operatingSystem=\$operatingSystem"

    jq_args+=( --arg "distribution" "$PIPELINES_CORE_OSDISTRIBUTION" )
    jq_query+=" | .operatingSystemDistribution=\$distribution"

    jq_args+=( --arg "architecture" "$PIPELINES_CORE_ARCHITECTURE" )
    jq_query+=" | .architecture=\$architecture"

    jq "${jq_args[@]}" "$jq_query" <<<'{}'
  else
    log_proc_marker "System information"
    log_proc_msg "Release: $PIPELINES_CORE_CONTROLPLANEVERSION"
    log_proc_msg "Control plane version: $PIPELINES_CORE_CONTROLPLANEVERSION"
    log_proc_msg "Build plane version: $PIPELINES_CORE_BUILDPLANEVERSION"
    log_proc_msg "Install mode: $PIPELINES_CORE_RUNMODE"
    log_proc_msg "Last upgrade: $PIPELINES_CORE_LASTUPDATETIME"
    log_proc_msg "Operating system: $PIPELINES_CORE_OS"
    log_proc_msg "Operating system distribution: $PIPELINES_CORE_OSDISTRIBUTION"
    log_proc_msg "Architecture: $PIPELINES_CORE_ARCHITECTURE"
    log_proc_msg "Artifactory URL: $PIPELINES_SHARED_ARTIFACTORY_BASEURL"
    log_proc_msg "Unified UI URL: $PIPELINES_SHARED_ARTIFACTORY_BASEURLUI"
    log_proc_msg "Service ID: $PIPELINES_SHARED_ARTIFACTORY_SERVICEID"
    if [ ! -z "$PIPELINES_CORE_SERVICES_API_EXTERNALURL" ]; then
      log_proc_msg "API URL: $PIPELINES_CORE_SERVICES_API_EXTERNALURL"
    fi
    # Since the connections string contains the password, we should mask it first before logging it.
    log_proc_msg "Database Endpoint: $(mask_connection_string $PIPELINES_SHARED_DB_CONNECTIONSTRING)"
    log_proc_msg "Vault Server: $PIPELINES_SHARED_VAULT_IP"
    log_proc_msg "RabbitMQ Server: $PIPELINES_SHARED_MSG_IP"
    log_proc_msg "Redis Server: $PIPELINES_SHARED_REDIS_IP"
  fi
}

log_versions() {
  local format="$1"
  if [ "$format" == "--json" ] || [ "$format" == "-j" ]; then
    format="json"
  fi
  if [ "$format" == "json" ]; then
    local jq_args=( )
    local jq_query='.'

    jq_args+=( --arg "controlplaneVersion" "$PIPELINES_CORE_CONTROLPLANEVERSION" )
    jq_query+=" | .controlplaneVersion=\$controlplaneVersion"

    jq_args+=( --arg "buildplaneVersion" "$PIPELINES_CORE_BUILDPLANEVERSION" )
    jq_query+=" | .buildplaneVersion=\$buildplaneVersion"

    jq "${jq_args[@]}" "$jq_query" <<<'{}'
  else
    log_proc_marker "Pipelines Version"
    log_proc_msg "Control plane version: $PIPELINES_CORE_CONTROLPLANEVERSION"
    log_proc_msg "Build plane version: $PIPELINES_CORE_BUILDPLANEVERSION"
  fi
}

check_semver() {
  local installed_version="$1"

  local installed_major=$(echo $installed_version | cut -d '.' -f 1 | sed 's/[^0-9]*//g')
  local installed_minor=$(echo $installed_version | cut -d '.' -f 2 | sed 's/[^0-9]*//g')
  local installed_patch=$(echo $installed_version | cut -d '.' -f 3 | sed 's/[^0-9]*//g')
  if [ -z $installed_patch ]; then
    installed_patch=0
  fi

  if ! [[ $installed_major =~ ^[0-9]+$ ]] || ! [[ $installed_minor =~ ^[0-9]+$ ]] || ! [[ $installed_patch =~ ^[0-9]+$ ]]; then
    log_proc_error "Unable to determine dependency version"
    exit 1
  fi
}

compare_semver() {
  local installed_version="$1"
  local requested_version="$2"

  local installed_major=$(echo $installed_version | cut -d '.' -f 1 | sed 's/[^0-9]*//g')
  local installed_minor=$(echo $installed_version | cut -d '.' -f 2 | sed 's/[^0-9]*//g')
  local installed_patch=$(echo $installed_version | cut -d '.' -f 3 | sed 's/[^0-9]*//g')
  if [ -z $installed_patch ]; then
    installed_patch=0
  fi

  local requested_major=$(echo $requested_version | cut -d '.' -f 1 | sed 's/[^0-9]*//g')
  local requested_minor=$(echo $requested_version | cut -d '.' -f 2 | sed 's/[^0-9]*//g')
  local requested_patch=$(echo $requested_version | cut -d '.' -f 3 | sed 's/[^0-9]*//g')
  if [ -z $requested_patch ]; then
    requested_patch=0
  fi

  if [ $installed_major -gt $requested_major ]; then
    :
  elif [ $installed_major -lt $requested_major ]; then
    return 0
  elif [ $installed_minor -gt $requested_minor ]; then
    :
  elif [ $installed_minor -lt $requested_minor ]; then
    return 0
  elif [ $installed_patch -lt $requested_patch ]; then
    return 0
  fi

  return 1
}

# A helper method to mask the sensitive information such as password, in a connection string.
#
# Example -> "postgres://user:password@host:port/db" ---> "postgres://user:[REDACTED]@host:port/db"
# Example -> "postgres://host:port/db" ---> "postgres://host:port/db"
# Example -> "postgres://host:port" ---> "postgres://host:port"
# Example -> "postgres://host/db" ---> "postgres://host/db"
# Example -> "postgres://host" ---> "postgres://host"
#
# You can pass the mask string as a second argument, which should replace the sensitive information.
# by default it will use "[REDACTED]".
mask_connection_string() {
  input=$1
  mask_default_value=$2
  mask_value="${mask_default_value:-[REDACTED]}"

  local masked_result="echo $input | sed -En 's/^(.*)\:\/\/(.*)(:.*)(@.*)$/\1:\/\/\2:$mask_value\4/p'"
  local result=$(eval $masked_result)

  result="${result:-$input}"
  echo $result
}

create_user_volume_dirs() {
  log_proc_msg "Mounting volumes to service user"
  
  user=$1
  shift
  log_proc_msg "User: $user"
  log_proc_msg "Volumes: $@"

  mkdir -p $@

  chown -R "$user" $@
}
