#!/bin/bash
set -e
set -o pipefail

readonly VAULT_RESPONSE_FILE="$TMP_DIR/vault_response.tmp"

__vault_initialize() {
  VAULT_API_URL="$PIPELINES_SHARED_VAULT_URL"
  RESPONSE_CODE=404
  RESPONSE_DATA=""
  CURL_EXIT_CODE=0

  touch $VAULT_RESPONSE_FILE
}

__vault_get() {
  __vault_initialize

  local url="$VAULT_API_URL/$1"

  local get_cmd="curl \
    -H \"Content-Type: application/json; charset=utf-8\" \
    -X GET $url \
    --silent --write-out \"%{http_code}\n\" \
    --output $VAULT_RESPONSE_FILE"

  if [ ! -z "$PIPELINES_SHARED_VAULT_TOKEN" ]; then
    get_cmd="$get_cmd \
      -H \"X-Vault-Token: $PIPELINES_SHARED_VAULT_TOKEN\""
  fi

  {
    RESPONSE_CODE=$(eval $get_cmd)
  } || {
    CURL_EXIT_CODE=$(echo $?)
  }

  is_request_success=false
  if [ $RESPONSE_CODE -lt 200 ] || [ $RESPONSE_CODE -gt 299 ]; then
    response="Failed request on $url. Error: $(cat $VAULT_RESPONSE_FILE)"
    response_status_code="$RESPONSE_CODE"
  elif [ $CURL_EXIT_CODE -gt 0 ]; then
    # we are assuming that if curl cmd failed, API is unavailable
    response="curl failed with error code $CURL_EXIT_CODE. Vault might be down."
    response_status_code=503
  else
    is_request_success=true
    response_status_code="$RESPONSE_CODE"
    response=$(cat $VAULT_RESPONSE_FILE)
  fi
}

__vault_put() {
  __vault_initialize

  local url="$VAULT_API_URL/$1"
  local update="$2"

  local put_cmd="curl \
    -H \"Content-Type: application/json; charset=utf-8\" \
    -X PUT $url \
    -d '"$update"' \
    --silent --write-out \"%{http_code}\n\" \
    --output $VAULT_RESPONSE_FILE"

  if [ ! -z "$PIPELINES_SHARED_VAULT_TOKEN" ]; then
    put_cmd="$put_cmd \
      -H \"X-Vault-Token: $PIPELINES_SHARED_VAULT_TOKEN\""
  fi

  {
    RESPONSE_CODE=$(eval $put_cmd)
  } || {
    CURL_EXIT_CODE=$(echo $?)
  }

  is_request_success=false
  if [ $RESPONSE_CODE -lt 200 ] || [ $RESPONSE_CODE -gt 299 ]; then
    response="Failed request on $url. Error: $(cat $VAULT_RESPONSE_FILE)"
    response_status_code="$RESPONSE_CODE"
  elif [ $CURL_EXIT_CODE -gt 0 ]; then
    # we are assuming that if curl cmd failed, API is unavailable
    response="curl failed with error code $CURL_EXIT_CODE. Vault might be down."
    response_status_code=503
  else
    is_request_success=true
    response_status_code="$RESPONSE_CODE"
    response=$(cat $VAULT_RESPONSE_FILE)
  fi
}

__vault_post() {
  __vault_initialize

  local url="$VAULT_API_URL/$1"
  local update="$2"

  local post_cmd="curl \
    -H \"Content-Type: application/json; charset=utf-8\" \
    -X POST $url \
    -d '"$update"' \
    --silent --write-out \"%{http_code}\n\" \
    --output $VAULT_RESPONSE_FILE"

  if [ ! -z "$PIPELINES_SHARED_VAULT_TOKEN" ]; then
    post_cmd="$post_cmd \
      -H \"X-Vault-Token: $PIPELINES_SHARED_VAULT_TOKEN\""
  fi

  {
    RESPONSE_CODE=$(eval $post_cmd)
  } || {
    CURL_EXIT_CODE=$(echo $?)
  }

  is_request_success=false
  if [ $RESPONSE_CODE -lt 200 ] || [ $RESPONSE_CODE -gt 299 ]; then
    response="Failed request on $url. Error: $(cat $VAULT_RESPONSE_FILE)"
    response_status_code="$RESPONSE_CODE"
  elif [ $CURL_EXIT_CODE -gt 0 ]; then
    # we are assuming that if curl cmd failed, API is unavailable
    response="curl failed with error code $CURL_EXIT_CODE. Vault might be down."
    response_status_code=503
  else
    is_request_success=true
    response_status_code="$RESPONSE_CODE"
    response=$(cat $VAULT_RESPONSE_FILE)
  fi
}

__vault_delete() {
  __vault_initialize

  local url="$VAULT_API_URL/$1"

  local delete_cmd="curl \
    -H \"Content-Type: application/json; charset=utf-8\" \
    -X DELETE $url \
    --silent --write-out \"%{http_code}\n\" \
    --output $VAULT_RESPONSE_FILE"

  if [ ! -z "$PIPELINES_SHARED_VAULT_TOKEN" ]; then
    delete_cmd="$delete_cmd \
      -H \"X-Vault-Token: $PIPELINES_SHARED_VAULT_TOKEN\""
  fi

  {
    RESPONSE_CODE=$(eval $delete_cmd)
  } || {
    CURL_EXIT_CODE=$(echo $?)
  }

  is_request_success=false
  if [ $RESPONSE_CODE -lt 200 ] || [ $RESPONSE_CODE -gt 299 ]; then
    response="Failed request on $url. Error: $(cat $VAULT_RESPONSE_FILE)"
    response_status_code="$RESPONSE_CODE"
  elif [ $CURL_EXIT_CODE -gt 0 ]; then
    # we are assuming that if curl cmd failed, API is unavailable
    response="curl failed with error code $CURL_EXIT_CODE. Vault might be down."
    response_status_code=503
  else
    is_request_success=true
    response_status_code="$RESPONSE_CODE"
    response=$(cat $VAULT_RESPONSE_FILE)
  fi
}


## GET calls ###################################

_vault_get_health() {
  __vault_get "v1/sys/health"
}

_vault_get_secret() {
  if [ "$#" -ne 1 ]; then
    log_proc_error "The number of arguments expected by _vault_get_secret is 1"
    log_proc_error "current arguments $#"
    exit 1
  fi

  local key="$1"

  if [ -z "$key" ]; then
    log_proc_error "Secret key is required"
    exit 1
  fi

  __vault_get "v1/$key"
}

_vault_get_seal_status() {
  __vault_get "v1/sys/seal-status"
}

_vault_get_initialized_status() {
  __vault_get "v1/sys/init"
}

## POST calls ###################################

_vault_post_secret() {
  if [ "$#" -ne 2 ]; then
    log_proc_error "The number of arguments expected by _vault_post_secret is 2"
    log_proc_error "current arguments $#"
    exit 1
  fi

  local key="$1"
  local secret="$2"

  if [ -z "$key" ]; then
    log_proc_error "Secret key is required"
    exit 1
  fi

  __vault_post "v1/$key" "$secret"
}

_vault_post_mount() {
  if [ "$#" -ne 2 ]; then
    log_proc_error "The number of arguments expected by _vault_post_mount is 2"
    log_proc_error "current arguments $#"
    exit 1
  fi

  local mount_path="$1"
  local body="$2"

  if [ -z "$mount_path" ]; then
    log_proc_error "Mount path is required"
    exit 1
  fi

  if [ -z "$body" ]; then
    log_proc_error "Body is required"
    exit 1
  fi

  __vault_post "v1/sys/mounts/$mount_path" "$body"
}

## PUT calls ###################################

_vault_put_init() {
  if [ "$#" -ne 1 ]; then
    log_proc_error "The number of arguments expected by _vault_put_init is 1"
    log_proc_error "current arguments $#"
    exit 1
  fi

  local body="$1"

  if [ -z "$body" ]; then
    log_proc_error "Configuration is required"
    exit 1
  fi

  __vault_put "v1/sys/init" "$body"
}

_vault_put_unseal() {
  if [ "$#" -ne 1 ]; then
    log_proc_error "The number of arguments expected by _vault_put_unseal is 1"
    log_proc_error "current arguments $#"
    exit 1
  fi

  local body="$1"

  if [ -z "$body" ]; then
    log_proc_error "Body is required"
    exit 1
  fi

  __vault_put "v1/sys/unseal" "$body"
}

## DELETE calls ###################################

_vault_delete_secret() {
  if [ "$#" -ne 1 ]; then
    log_proc_error "The number of arguments expected by _vault_delete_secret is 1"
    log_proc_error "current arguments $#"
    exit 1
  fi

  local key="$1"

  if [ -z "$key" ]; then
    log_proc_error "Secret key is required"
    exit 1
  fi

  __vault_delete "v1/$key"
}
