'use strict';

module.exports = migrateCommitDataInPipelineSyncStatuses;

var _ = require('underscore');
var util = require('util');
var async = require('async');
const CALL_OBJECT_LIMIT = 1000;

function migrateCommitDataInPipelineSyncStatuses(client, callback) {
  var fileName = process.argv[1];
  var bag = {
    who: util.format('installer.migration|%s|%s', fileName,
      migrateCommitDataInPipelineSyncStatuses.name),
    client: client,
    skipMigration: true
  };
  util.log(bag.who, 'Started');

  async.series([
    _checkIfToSkipMigration.bind(null, bag),
    _fetchAndProcessPipelineSyncStatuses.bind(null, bag)
    ],
    function (err) {
      if (err) {
        return callback(err);
      } else {
        util.log(bag.who, 'Completed');
        return callback();
      }
    }
  );
}
function _checkIfToSkipMigration(bag, next) {
  var who = bag.who + '|' + _checkIfToSkipMigration.name;
  util.log(who, 'Inside');

  var query = `SELECT 1 FROM "pipelineSyncStatuses" WHERE ` +
    `"triggeredByResourceVersionId" IS NOT NULL AND "commitData" IS NULL;`;

  bag.client.query(query,
    function (err, syncStatuses) {
      if (err)
        return next(
          new ActErr(who, ActErr.DBOperationFailed,
            `Failed to check if to run migration with error: ` +
            `${util.inspect(err)}`)
        );
      if (syncStatuses.rowCount > 0)
        bag.skipMigration = false;

      return next();
    }
  );
}

function _fetchAndProcessPipelineSyncStatuses(bag, next) {
  if (bag.skipMigration) return next();

  var who = bag.who + '|' + _fetchAndProcessPipelineSyncStatuses.name;
  util.log(who, 'Inside');

  var fetchedAllPipelineSyncStatuses = false;

  async.whilst(
    function () {
      return !fetchedAllPipelineSyncStatuses;
    },
    function (done) {
      var query = `SELECT id, "triggeredByResourceVersionId" FROM ` +
        `"pipelineSyncStatuses" WHERE "triggeredByResourceVersionId" IS NOT ` +
        `NULL AND "commitData" IS NULL LIMIT ${CALL_OBJECT_LIMIT};`;

      bag.client.query(query,
        function (err, pipelineSyncStatuses) {
          if (err)
            return done(
              new ActErr(who, ActErr.DBOperationFailed,
                `Failed to find pipelineSyncStatuses with error: ` +
                  `${util.inspect(err)}`)
            );

          if (pipelineSyncStatuses.rowCount !== CALL_OBJECT_LIMIT)
            fetchedAllPipelineSyncStatuses = true;

          bag.pipelineSyncStatuses = pipelineSyncStatuses.rows;
          var innerBag = {
            who: who,
            client: bag.client,
            pipelineSyncStatuses: bag.pipelineSyncStatuses
          };

          async.series([
              __getResourceVersions.bind(null, innerBag),
              __addShaDataToPipelineSyncStatuses.bind(null, innerBag)
            ],
            function (err) {
              if (err)
                return done(err);
              return done();
            }
          );
        }
      );
    }, function (err) {
      return next(err);
    }
  );
}

function __getResourceVersions(bag, done) {
  if (_.isEmpty(bag.pipelineSyncStatuses))
    return done();

  let who = bag.who + '|' + __getResourceVersions.name;
  util.log(who, 'Inside');

  let resourceVersionIds = _.pluck(bag.pipelineSyncStatuses,
    'triggeredByResourceVersionId');

  let query = `SELECT "id", "contentPropertyBag" FROM "resourceVersions" ` +
    `WHERE id IN (${resourceVersionIds.join(',')}) AND "contentPropertyBag"` +
    ` IS NOT NULL;`;

  bag.client.query(query,
    function (err, resourceVersions) {
      if (err)
        return done(
          new ActErr(who, ActErr.DBOperationFailed,
            `Failed to get resource versions with error: ${util.inspect(err)}`)
        );

      if (resourceVersions.rows === 0)
        return done();

      bag.resourceVersionsById = _.indexBy(resourceVersions.rows, 'id');
      return done();
    }
  );
}

function __addShaDataToPipelineSyncStatuses(bag, done) {
  if (_.isEmpty(bag.resourceVersionsById) ||
    _.isEmpty(bag.pipelineSyncStatuses))
    return done();

  let who = bag.who + '|' + __addShaDataToPipelineSyncStatuses.name;
  util.log(who, 'Inside');
  let counter = 0;

  async.eachLimit(bag.pipelineSyncStatuses, 10,
    function (syncStatus, cb) {
      let resourceVersion =
        bag.resourceVersionsById[syncStatus.triggeredByResourceVersionId];
      let commitData = {};

      if (resourceVersion && resourceVersion.contentPropertyBag) {
        try {
          let contentPropertyBag = JSON.parse(resourceVersion.contentPropertyBag);

          commitData = {
            commitSha: contentPropertyBag.commitSha || null,
            committer: contentPropertyBag.shaData?.lastAuthor?.login ||
              contentPropertyBag.shaData?.lastAuthor?.displayName ||
              null,
            commitMsg: contentPropertyBag.commitMessage || null,
            source: contentPropertyBag.path || null
          };
        } catch (err) {
          util.log(who, `WARNING: Failed to parse contentPropertyBag for ` +
            `resource version with id: ${resourceVersion.id} with error: ` +
            `${util.inspect(err)}. Setting commitData to an empty object for ` +
            `pipelineSyncStatus with id: ${syncStatus.id}`);
          commitData = {};
        }
      }

      let commitDataText;
      try {
        commitDataText = JSON.stringify(commitData);
      } catch (err) {
        util.log(who, `WARNING: Failed to stringify commitData ` +
          `for resource version with id: ${resourceVersion.id} with ` +
          `error: ${util.inspect(err)}`);
        commitDataText = '{}';
      }

      var updateQuery = `UPDATE "pipelineSyncStatuses" SET "commitData" = $1::text WHERE id = $2::integer;`;
      bag.client.query(updateQuery, [commitDataText, syncStatus.id],
        function (err) {
          if (err) {
            let errMsg = `Failed to update pipelineSyncStatuses with ` +
              `error: ${util.inspect(err)} for pipelineSyncStatus with id: ` +
              `${syncStatus.id}`;
            util.log(who, `WARNING: ${errMsg}`);
            return cb(new ActErr(who, ActErr.DBOperationFailed, errMsg));
          }
          counter++;
          return cb();
        }
      );
    },
    function (err) {
      util.log(who, `Number of records migrated: ${counter}`);
      return done(err);
    }
  );
}
