'use strict';

module.exports = migrateGitRepoFullNameInResources;

const _ = require('underscore');
const util = require('util');
const async = require('async');

const query_array_limit = 11242;

function migrateGitRepoFullNameInResources(client, callback) {
  const fileName = process.argv[1];
  console.time('*** Resources Migration');
  const bag = {
    who: util.format('installer.migration|%s', fileName),
    client: client,
    skipResourcesMigration: false,
    migratedResources: 0,
  };

  async.series([
    _checkToMigrate.bind(null, bag),
    _createTempTable.bind(null, bag),
    _getResourcesToMigrate.bind(null, bag),
    _bulkInsertTmpTable.bind(null, bag),
    _updateNewFieldsInResources.bind(null, bag)
  ],
    function (err) {
      if (err) {
        return  callback(err);
      } else {
        console.timeEnd('*** Resources Migration');
        util.log(bag.who, 'Completed');
        return callback();
      }
    }
  );
}

function _checkToMigrate(bag, next) {
  const who = bag.who + '|' + _checkToMigrate.name;
  util.log(who, 'Inside');

  const query = util.format('SELECT id FROM "resources" where "typeCode"=1000' +
    ' and "gitRepoFullName" is null limit 1;');
  bag.client.query(query,
    function (err, resources) {
      if (err)
        return next(
          new ActErr(who, ActErr.DBOperationFailed,
            'Failed to find resources with error: ' +
              util.inspect(err))
        );
      if (resources.rowCount === 0) {
        bag.skipResourcesMigration = true;
      }
      return next();
    }
  );
}

function _createTempTable(bag, next) {
  if (bag.skipResourcesMigration) return next();
  const who = bag.who + '|' + _createTempTable.name;
  util.log(who, 'Inside');

  const query = "create temp table tmpres (\"resourceId\" integer, " +
    "\"gitRepoFullName\" character varying(255), \"gitIncludeBranchPattern\"" +
    " text, \"gitExcludeBranchPattern\" text, \"gitBuildOnCommit\" boolean, " +
    "\"gitBuildOnPullRequestCreate\" boolean, \"gitBuildOnPullRequestClose\" " +
    "boolean, \"gitBuildOnReleaseCreate\" boolean, \"gitBuildOnTagCreate\" boolean);"
  bag.client.query(query,
    function (err) {
      if (err)
        return next(
          new ActErr(who, ActErr.DBOperationFailed,
            'Failed to create tmp table with error: ' +
              util.inspect(err))
        );
      return next();
    }
  );
}

function _getResourcesToMigrate(bag, next) {
  if (bag.skipResourcesMigration) return next();

  const who = bag.who + '|' + _getResourcesToMigrate.name;
  util.log(who, 'Inside');

  const query = util.format(
    'SELECT id, "systemPropertyBag", "ymlConfigPropertyBag"'+
    ' FROM "resources" where "typeCode"=1000 and "gitRepoFullName" is null;');

  bag.client.query(query,
    function (err, resources) {
      if (err)
        return next(
          new ActErr(who, ActErr.DBOperationFailed,
            'Failed to find resources with error: ' +
              util.inspect(err))
        );
      if (resources.rowCount === 0) {
        bag.skipResourcesMigration = true;
        return next();
      }
      const insertQuery = `INSERT INTO tmpres("resourceId", ` +
        `"gitRepoFullName", "gitIncludeBranchPattern", ` +
        `"gitExcludeBranchPattern", "gitBuildOnCommit", ` +
        `"gitBuildOnPullRequestCreate", "gitBuildOnPullRequestClose",` +
        ` "gitBuildOnReleaseCreate", "gitBuildOnTagCreate") VALUES`;
      const length = resources.rows.length;
      let includeIndex = 1;
      bag.resourcesQuery = {
        currentBatchQuery: insertQuery,
        batchesQuery:[],
        currentBranchPatternArray: [],
        branchPatternArray:[]
      };
      _.each(resources.rows,
        function(resource, index) {
          let systemPropertyBag = {};
          let ymlConfigPropertyBag = {};
          try {
            if (resource.systemPropertyBag && _.isString(resource.systemPropertyBag)) {
              systemPropertyBag = JSON.parse(
                resource.systemPropertyBag);
            }
            if (resource.ymlConfigPropertyBag && _.isString(resource.ymlConfigPropertyBag)) {
              ymlConfigPropertyBag = JSON.parse(resource.ymlConfigPropertyBag);
            }
          } catch (error) {
            util.log(who, 'Failed to parse systemPropertyBag or ymlConfigPropertyBag for ' +
              'resource id: ', resource.id);
            return;
          }
          let includeBranch = null;
          let excludeBrach =  null;
          let onCommit = true;
          let onPullReqCreate = false;
          let onPullRequestClose = false;
          let onReleaseCreate = false;
          let onTagCreate = false;
          if (ymlConfigPropertyBag.branches) {
            if (ymlConfigPropertyBag.branches.include) {
              includeBranch = ymlConfigPropertyBag.branches.include;
            }
            if (ymlConfigPropertyBag.branches.exclude) {
              excludeBrach = ymlConfigPropertyBag.branches.exclude;
            }
          }
          if (ymlConfigPropertyBag.buildOn) {
            if (_.has(ymlConfigPropertyBag.buildOn, 'commit')) {
              onCommit = ymlConfigPropertyBag.buildOn.commit;
            }
            if (_.has(ymlConfigPropertyBag.buildOn, 'pullRequestCreate')) {
              onPullReqCreate = ymlConfigPropertyBag.buildOn.pullRequestCreate;
            }
            if (_.has(ymlConfigPropertyBag.buildOn, 'pullRequestClose')) {
              onPullRequestClose = ymlConfigPropertyBag.buildOn.pullRequestClose;
            }
            if (_.has(ymlConfigPropertyBag.buildOn, 'releaseCreate')) {
              onReleaseCreate = ymlConfigPropertyBag.buildOn.releaseCreate;
            }
            if (_.has(ymlConfigPropertyBag.buildOn, 'tagCreate')) {
              onTagCreate = ymlConfigPropertyBag.buildOn.tagCreate;
            }
          }
          if (includeIndex === query_array_limit + 1) {
            bag.resourcesQuery.batchesQuery.push(bag.resourcesQuery.currentBatchQuery);
            bag.resourcesQuery.branchPatternArray.push(bag.resourcesQuery.currentBranchPatternArray);
            includeIndex = 1;
            bag.resourcesQuery.currentBatchQuery = insertQuery;
            bag.resourcesQuery.currentBranchPatternArray = [];
          }
          bag.resourcesQuery.currentBranchPatternArray.push(includeBranch);
          bag.resourcesQuery.currentBranchPatternArray.push(excludeBrach);
          let excludeIndex = includeIndex + 1;

          bag.resourcesQuery.currentBatchQuery = bag.resourcesQuery.currentBatchQuery.concat(
            `(${resource.id}, '${systemPropertyBag.gitRepoFullName}', $${includeIndex}::text,
            $${excludeIndex}::text, ${onCommit}, ${onPullReqCreate},
            ${onPullRequestClose}, ${onReleaseCreate}, ${onTagCreate})`);

          if (index !== length - 1 && excludeIndex !== query_array_limit) {
            bag.resourcesQuery.currentBatchQuery = bag.resourcesQuery.currentBatchQuery.concat(', ');
          }
          if (index === length - 1) {
            bag.resourcesQuery.batchesQuery.push(bag.resourcesQuery.currentBatchQuery);
            bag.resourcesQuery.branchPatternArray.push(bag.resourcesQuery.currentBranchPatternArray);
          }
          includeIndex = includeIndex + 2;
        }
      );
      util.log('Found resources to migrate: ', resources.rows.length);
      return next();
    }
  );
}

function _bulkInsertTmpTable(bag, next) {
  if (bag.skipResourcesMigration) return next();

  const who = bag.who + '|' + _bulkInsertTmpTable.name;
  util.log(who, 'Inside');

  async.eachOfSeries(bag.resourcesQuery.batchesQuery,
    function (insertQuery, index, nextInsertQuery) {
      bag.client.query(insertQuery, bag.resourcesQuery.branchPatternArray[index],
        function (err) {
          if (err) {
            return nextInsertQuery(
              new ActErr(who, ActErr.DBOperationFailed,
                'Failed to insert into tmp table with error: ' +
                  util.inspect(err))
            );
          }
          return nextInsertQuery();
        }
      );
    }, function (err) {
      return next(err);
    }
  );
}

function _updateNewFieldsInResources(bag, next) {
  if (bag.skipResourcesMigration) return next();

  const who = bag.who + '|' + _updateNewFieldsInResources.name;
  util.log(who, 'Inside');

  const updateQuery = "update resources set \"gitRepoFullName\" = " +
    "s.\"gitRepoFullName\", \"gitIncludeBranchPattern\" = s.\"gitIncludeBranchPattern\"," +
    " \"gitExcludeBranchPattern\" = s.\"gitExcludeBranchPattern\", \"gitBuildOnCommit\"" +
    " = s.\"gitBuildOnCommit\", \"gitBuildOnPullRequestCreate\" = " +
    "s.\"gitBuildOnPullRequestCreate\", \"gitBuildOnPullRequestClose\" " +
    "= s.\"gitBuildOnPullRequestClose\", \"gitBuildOnReleaseCreate\" = " +
    "s.\"gitBuildOnReleaseCreate\", \"gitBuildOnTagCreate\" = s.\"gitBuildOnTagCreate\" " +
    "from tmpres s where resources.id = s.\"resourceId\";"
  bag.client.query(updateQuery,
    function (err) {
      if (err)
        return next(
          new ActErr(who, ActErr.DBOperationFailed,
            'Failed to update resources table with error: ' +
              util.inspect(err))
        );
      return next();
    }
  );
}
