'use strict';

module.exports = migrateTriggeredByRunResVerIdInRuns;

var _ = require('underscore');
var util = require('util');
var async = require('async');
var migrationVersion = '1.4';

function migrateTriggeredByRunResVerIdInRuns(client, callback) {
  var fileName = process.argv[1];
  var pipelineVersion;
  if (process.env.PIPELINEVERSION)
    pipelineVersion = process.env.PIPELINEVERSION.split('.');

  migrationVersion = migrationVersion.split('.');
  var pvMajor = parseInt(pipelineVersion[0]);
  var pvMinor = parseInt(pipelineVersion[1]);
  var pvPatch = parseInt(pipelineVersion[2]);
  if (!pvPatch)
    pvPatch = 0;
  var mvMajor = parseInt(migrationVersion[0]);
  var mvMinor = parseInt(migrationVersion[1]);
  var mvPatch = parseInt(migrationVersion[2]);
  if (!mvPatch)
    mvPatch = 0;

  var bag = {
    who: util.format('installer.migration|%s', fileName),
    client: client,
    skipRunsMigration: false,
    migratedRuns: 0
  };

  if (mvMajor < pvMajor) {
    bag.skipRunsMigration = true;
  } else if (mvMajor === pvMajor) {
    if (mvMinor < pvMinor)
      bag.skipRunsMigration = true;
    else if (mvMinor === pvMinor) {
      if (mvPatch <= pvPatch)
        bag.skipRunsMigration = true;
    }
  }

  async.series([
    _getRuns.bind(null, bag),
    _updateTriggeredByRunResVerIdInRuns.bind(null, bag)
  ],
    function (err) {
      if (err) {
        return callback(err);
      } else {
        util.log(bag.who, 'Completed');
        return callback();
      }
    }
  );
}

function _getRuns(bag, next) {
  if (bag.skipRunsMigration) return next();

  var who = bag.who + '|' + _getRuns .name;
  util.log(who, 'Inside');

  var query = util.format('SELECT id, "staticPropertyBag" FROM "runs";');

  bag.client.query(query,
    function (err, runs) {
      if (err)
        return next(
          new ActErr(who, ActErr.DBOperationFailed,
            'Failed to find runs with error: ' +
              util.inspect(err))
        );
      if (runs.rowCount === 0) {
        bag.skipRunsMigration = true;
        return next();
      }
      bag.runs = runs.rows;
      util.log('Found runs to migrate: ', bag.runs.length);
      return next();
    }
  );
}

function _updateTriggeredByRunResVerIdInRuns(bag, next) {
  if (bag.skipRunsMigration) return next();

  var who = bag.who + '|' + _updateTriggeredByRunResVerIdInRuns.name;
  util.log(who, 'Inside');

  async.eachLimit(bag.runs, 10,
    function (run, done) {
      bag.migratedRuns = bag.migratedRuns + 1;
      if (bag.migratedRuns % 1000 === 0)
        util.log('Migrated Runs: ', bag.migratedRuns ,'/', bag.runs.length);
      if (run.staticPropertyBag &&
        _.isString(run.staticPropertyBag)) {
        try {
          bag.staticPropertyBag = JSON.parse(
            run.staticPropertyBag);
        } catch (error) {
            util.log(who, 'Failed to parse staticPropertyBag for ' +
            'run id: ', run.id);
            return process.nextTick(done);
        }
      } else {
        return process.nextTick(done);
      }

      if (!_.has(bag.staticPropertyBag, 'triggeredByResourceName') ||
        _.has(bag.staticPropertyBag, 'triggeredByRunResourceVersionId'))
          return process.nextTick(done);

      var innerBag = {
        who: who,
        runId: run.id,
        staticPropertyBag: bag.staticPropertyBag,
        client: bag.client
      };

      async.series([
          __computeRunResourceVersionId.bind(null, innerBag),
          __updateTriggeredByRunResVerId.bind(null, innerBag)
        ],
        function (err) {
          // This does an argument check and if it's not correct, it will pass
          // the error to the callback. The API updated fairly recently to
          // allow passing arguments to process.nextTick() allowing it to take
          // any arguments passed after the callback to be propagated as the
          // arguments to the callback so you don't have to nest functions.
          return process.nextTick(done, err);
        }
      );
    },
    function (err) {
      return next(err);
    }
  );
}

function __computeRunResourceVersionId(bag, next) {
  if (!bag.runId) return next();

  var who = bag.who + '|' + __computeRunResourceVersionId.name;
  util.log(who, 'Inside');

  var query = util.format('SELECT id from "runResourceVersions" WHERE ' +
    '"runId"=\'%s\' and "resourceName"=\'%s\' order by id limit 1;', bag.runId,
      bag.staticPropertyBag.triggeredByResourceName);

  bag.client.query(query,
    function (err, runResourceVersions) {
      if (err)
        return next(
          new ActErr(who, ActErr.DBOperationFailed,
            'Failed to find runResourceVersions with error: ' +
              util.inspect(err))
        );
      if (runResourceVersions.rowCount === 0) {
        return next();
      }
      bag.runResourceVersionId = runResourceVersions.rows[0].id;
      return next();
    }
  );
}

function __updateTriggeredByRunResVerId(bag, next) {
  if (!bag.runResourceVersionId) return next();

  var who = bag.who + '|' + __updateTriggeredByRunResVerId.name;
  util.log(who, 'Inside');

  bag.staticPropertyBag.triggeredByRunResourceVersionId =
    bag.runResourceVersionId;
  bag.staticPropertyBag = JSON.stringify(bag.staticPropertyBag);

  var updateQuery = util.format('UPDATE "runs" SET ' +
    '"staticPropertyBag"=\'%s\' WHERE id = \'%s\';' ,
      bag.staticPropertyBag, bag.runId);

  bag.client.query(updateQuery,
    function (err) {
      if (err)
        return next(
          new ActErr(who, ActErr.DBOperationFailed,
            'Failed to update run with error: ' +
            util.inspect(err))
        );
      return next();
    }
  );
}
