#!/bin/bash
set -e

__log_status_marker() {
  if [ "$format" != "json" ]; then
    log_proc_marker "$1"
  fi
}

__parse_response() {
  local api_response="$1"
  local api_response_status_code="$2"
  local current_object="$3"

  if [ $api_response_status_code -gt 299 ]; then
    if [ "$format" == "json" ]; then
      local response_object=$(jq --arg "error" "$api_response" \
        ". | .error=\$error" <<<'{}')
      jq_args+=( --argjson "$current_object" "$response_object" )
      jq_query+=" | .$current_object=\$$current_object"
      jq "${jq_args[@]}" "$jq_query" <<<'{}'
    else
      log_proc_error "Response status code: $api_response_status_code"
      log_proc_error "Response: $api_response"
    fi
    exit 1
  fi

  if [ -z "$api_response" ]; then
    if [ "$format" == "json" ]; then
      local response_object=$(jq --arg "error" \
        "No response received from api" ". | .error=\$error" <<<'{}')
      jq_args+=( --argjson "$current_object" "$response_object" )
      jq_query+=" | .$current_object=\$$current_object"
      jq "${jq_args[@]}" "$jq_query" <<<'{}'
    else
      log_proc_error "No response received from api, exiting"
    fi
    exit 1
  fi

  local response=$(echo $api_response | jq '.' 2>&1)
  if [[ $response = *"parse error"* ]]; then
    if [ "$format" == "json" ]; then
      local response_object=$(jq --arg "error" \
        "Invalid response from api: $api_response" \
        ". | .error=\$error" <<<'{}')
      jq_args+=( --argjson "$current_object" "$response_object" )
      jq_query+=" | .$current_object=\$$current_object"
      jq "${jq_args[@]}" "$jq_query" <<<'{}'
    else
      log_proc_error "Invalid response from api: $api_response"
    fi
    exit 1
  fi
}

__print_status() {
  local status_component="$1"
  local status_data="$2"
  local component_ip=$(echo $status_data | jq -r '.ip')
  local component_port=$(echo $status_data | jq -r '.port')
  local component_uptime=$(echo $status_data | jq -r '.uptime')
  local component_reachable=$(echo $status_data | jq -r '.isReachable')

  printf "%2s COMPONENT %5s IP %9s PORT %10s UPTIME %8s REACHABLE %5s Errors\n"
  printf "%10s %15s %10s %20s %10s \n" "$status_component" "$component_ip" \
    "$component_port" "$component_uptime" "$component_reachable"

  printf "\n"
}

__get_uptime() {
  __log_status_marker "Local machine uptime"

  if [ "$format" == "json" ]; then
    local uptime=$(uptime -p)
    jq_args+=( --arg "localUptime" "$uptime" )
    jq_query+=" | .localUptime=\$localUptime"
  else
    uptime -p
  fi
}

__log_system_information() {
  if [ "$format" == "json" ]; then
    local systemInfo=$(log_system_info "json")
    jq_args+=( --argjson "system" "$systemInfo" )
    jq_query+=" | .system=\$system"
  else
    log_system_info
  fi
}

main() {
  load_system_config
  local format="$1"
  if [ "$format" == "json" ]; then
    local jq_args=( )
    local jq_query='.'
  fi

  __get_uptime
  __log_system_information

  if [ "$format" == "json" ]; then
    jq "${jq_args[@]}" "$jq_query" <<<'{}'
  fi
}

main "$@"
