'use strict';

const {
  DEFAULT_NODEPOOL_ARCH,
  DEFAULT_NODEPOOL_OS,
  DEFAULT_NODEPOOL_PERMISSIONS,
  DEFAULT_NODEPOOL_TIMEOUT,
  DEFAULT_NODEPOOL_CACHE_ENABLED,
  DEFAULT_NODEPOOL_AWS_INTERVAL_MINS,
  DEFAULT_NODEPOOL_K8S_INTERVAL_MINS,
  DEFAULT_NODEPOOL_MAX_DISK,
  DEFAULT_NODEPOOL_CREATED_AT,
  DEFAULT_NODEPOOL_UPDATED_AT,
  DEFAULT_NODEPOOL_PROJECT_ID,
  DEFAULT_NODEPOOL_ACCESS_PREFIX,
  MAC_OS_13_5,
  MAC_NODEPOOL_NAME_SUFFIX,
  MAC_NODEPOOL_INTERVAL_MINS,
  MAC_NODEPOOL_INSTANCE_TYPE,
} = require('../constants.js');

let mockConfig;
let mockMasterIntegrations;
let mockProjectIntegrations;
let mockBuildplaneImages;
let mockNodePools;
let mockError;
let querySpy;
let insertNodePoolSpy;
let updateDefaultNodePoolSpy;

describe('createDynamicNodePool', () => {
  beforeEach(() => {
    mockConfig = {
      accountId: 'accountId',
      nodePoolName: 'nodePoolName',
      nodelimit: '42',
      instanceType: 'instanceType',
      vpcId: 'vpcId',
      securityGroupId: 'securityGroupId',
      provider: 'aws',
      keyPairName: 'keyPairName',
      subnetId: 'subnetId',
      region: 'region',
    };
    mockMasterIntegrations = [{id: 'foo'}];
    mockProjectIntegrations = [{id: 'bar'}];
    mockBuildplaneImages = [
      {
        id: 'baz',
        region: mockConfig.region,
        operatingSystem: DEFAULT_NODEPOOL_OS,
        architecture: DEFAULT_NODEPOOL_ARCH,
      },
      {
        id: 'macos',
        region: mockConfig.region,
        operatingSystem: MAC_OS_13_5,
        architecture: DEFAULT_NODEPOOL_ARCH,
      }
    ];
    mockNodePools = [];
    mockError = undefined;
    insertNodePoolSpy = jest.fn();
    updateDefaultNodePoolSpy = jest.fn();

    jest.mock('fs', () => {
        const {INTEGRATION_CONFIG_DIR} = require('../constants.js');
        return {
          readFileSync: (key) => {
            return mockConfig[key.replace(`${INTEGRATION_CONFIG_DIR}/`, '')];
          },
          existsSync: () => {
            return true;
          },
        };
      }
    );
    querySpy = jest.fn((query, cb) => {
      if (query.includes('FROM "masterIntegrations"')) {
        return cb(undefined, {rows: mockMasterIntegrations});
      }
      if (query.includes('FROM "nodePools"')) {
        return cb(undefined, {rows: mockNodePools});
      }
      if (query.includes('FROM "buildPlaneImages"')) {
        return cb(undefined, {rows: mockBuildplaneImages});
      }
      if (query.includes('FROM "projectIntegrations"')) {
        return cb(undefined, {rows: mockProjectIntegrations});
      }
      if (query.includes('INSERT INTO "nodePools"')) {
        insertNodePoolSpy(query);
        return cb(mockError);
      }
      if (query.includes('UPDATE projects SET "configPropertyBag"')) {
        updateDefaultNodePoolSpy(query);
        return cb();
      }
      throw new Error(`unhandled query to mock db client: ${query}`);
    });
    global.config.client = {query: querySpy};
    global.config.isMacOSEnabled = false;
  });

  it('creates an aws dynamic NodePool' +
    ' in the db with the correct values', () => {
    const {formatCreatNodePoolQuery} =
      require('../createDynamicNodePool.js');
    const createDynamicNodePool =
      require('../createDynamicNodePool.js');
    const cb = jest.fn();
    createDynamicNodePool(undefined, cb);

    expect(insertNodePoolSpy).toHaveBeenLastCalledWith(
      formatCreatNodePoolQuery(
        mockConfig.nodePoolName,
        DEFAULT_NODEPOOL_PROJECT_ID,
        true,
        DEFAULT_NODEPOOL_ARCH,
        DEFAULT_NODEPOOL_OS,
        mockBuildplaneImages[0].id,
        mockProjectIntegrations[0].id,
        DEFAULT_NODEPOOL_PERMISSIONS,
        DEFAULT_NODEPOOL_TIMEOUT,
        DEFAULT_NODEPOOL_CACHE_ENABLED,
        DEFAULT_NODEPOOL_AWS_INTERVAL_MINS,
        DEFAULT_NODEPOOL_MAX_DISK,
        mockConfig.nodelimit,
        {
          instanceSize: mockConfig.instanceType,
          securityGroupId: mockConfig.securityGroupId,
          keyPairName: mockConfig.keyPairName,
          vpcId: mockConfig.vpcId,
          subnetId: mockConfig.subnetId,
          region: mockConfig.region
        },
        DEFAULT_NODEPOOL_CREATED_AT,
        DEFAULT_NODEPOOL_UPDATED_AT,
        DEFAULT_NODEPOOL_ACCESS_PREFIX
      ));
    expect(cb).toHaveBeenCalledTimes(1);
    expect(updateDefaultNodePoolSpy).toHaveBeenCalledTimes(1);
  });

  it('creates one default and one MacOS aws dynamic NodePool' +
    ' in the db with the correct values when MacOS feature flag is' +
    ' enabled', () => {
    global.config.isMacOSEnabled = true;
    global.config.macOSNodeLimit = 42;

    const {formatCreatNodePoolQuery} =
      require('../createDynamicNodePool.js');
    const createDynamicNodePool =
      require('../createDynamicNodePool.js');

    const cb = jest.fn();
    createDynamicNodePool({isMacOSEnabled: true}, cb);

    const macQuery = formatCreatNodePoolQuery(
      mockConfig.nodePoolName + MAC_NODEPOOL_NAME_SUFFIX,
      DEFAULT_NODEPOOL_PROJECT_ID,
      true,
      DEFAULT_NODEPOOL_ARCH,
      MAC_OS_13_5,
      mockBuildplaneImages[1].id,
      mockProjectIntegrations[0].id,
      DEFAULT_NODEPOOL_PERMISSIONS,
      DEFAULT_NODEPOOL_TIMEOUT,
      DEFAULT_NODEPOOL_CACHE_ENABLED,
      MAC_NODEPOOL_INTERVAL_MINS,
      DEFAULT_NODEPOOL_MAX_DISK,
      global.config.macOSNodeLimit,
      {
        instanceSize: MAC_NODEPOOL_INSTANCE_TYPE,
        securityGroupId: mockConfig.securityGroupId,
        keyPairName: mockConfig.keyPairName,
        vpcId: mockConfig.vpcId,
        subnetId: mockConfig.subnetId,
        region: mockConfig.region
      },
      DEFAULT_NODEPOOL_CREATED_AT,
      DEFAULT_NODEPOOL_UPDATED_AT,
      DEFAULT_NODEPOOL_ACCESS_PREFIX
    );

    const defaultNodeQuery = formatCreatNodePoolQuery(
      mockConfig.nodePoolName,
      DEFAULT_NODEPOOL_PROJECT_ID,
      true,
      DEFAULT_NODEPOOL_ARCH,
      DEFAULT_NODEPOOL_OS,
      mockBuildplaneImages[0].id,
      mockProjectIntegrations[0].id,
      DEFAULT_NODEPOOL_PERMISSIONS,
      DEFAULT_NODEPOOL_TIMEOUT,
      DEFAULT_NODEPOOL_CACHE_ENABLED,
      DEFAULT_NODEPOOL_AWS_INTERVAL_MINS,
      DEFAULT_NODEPOOL_MAX_DISK,
      mockConfig.nodelimit,
      {
        instanceSize: mockConfig.instanceType,
        securityGroupId: mockConfig.securityGroupId,
        keyPairName: mockConfig.keyPairName,
        vpcId: mockConfig.vpcId,
        subnetId: mockConfig.subnetId,
        region: mockConfig.region
      },
      DEFAULT_NODEPOOL_CREATED_AT,
      DEFAULT_NODEPOOL_UPDATED_AT,
      DEFAULT_NODEPOOL_ACCESS_PREFIX
    );

    expect(insertNodePoolSpy).toHaveBeenCalledTimes(2);
    expect(insertNodePoolSpy).toHaveBeenLastCalledWith(macQuery);
    expect(insertNodePoolSpy).toHaveBeenNthCalledWith(1, defaultNodeQuery);
    expect(cb).toHaveBeenCalledTimes(1);
    expect(updateDefaultNodePoolSpy).toHaveBeenCalledTimes(2);
  });

  it('creates a k8s dynamic NodePool' +
    ' in the db with the correct values', () => {
    mockConfig = {
      accountId: 'accountId',
      nodePoolName: 'nodePoolName',
      nodelimit: '42',
      provider: 'k8s',
      cpu: 'cpu',
      memory: 'memory',
      storageClass: 'storageClass',
      namespace: 'namespace',
      labels: '{}',
    };

    const {formatCreatNodePoolQuery} =
      require('../createDynamicNodePool.js');
    const createDynamicNodePool =
      require('../createDynamicNodePool.js');

    const cb = jest.fn();
    createDynamicNodePool(undefined, cb);

    expect(insertNodePoolSpy).toHaveBeenLastCalledWith(
      formatCreatNodePoolQuery(
        mockConfig.nodePoolName,
        DEFAULT_NODEPOOL_PROJECT_ID,
        true,
        DEFAULT_NODEPOOL_ARCH,
        DEFAULT_NODEPOOL_OS,
        null,
        mockProjectIntegrations[0].id,
        DEFAULT_NODEPOOL_PERMISSIONS,
        DEFAULT_NODEPOOL_TIMEOUT,
        DEFAULT_NODEPOOL_CACHE_ENABLED,
        DEFAULT_NODEPOOL_K8S_INTERVAL_MINS,
        DEFAULT_NODEPOOL_MAX_DISK,
        mockConfig.nodelimit,
        {
          storageClass: mockConfig.storageClass,
          labels: {},
          cpu: mockConfig.cpu,
          memory: mockConfig.memory,
          namespace: mockConfig.namespace,
        },
        DEFAULT_NODEPOOL_CREATED_AT,
        DEFAULT_NODEPOOL_UPDATED_AT,
        DEFAULT_NODEPOOL_ACCESS_PREFIX
      ));
    expect(cb).toHaveBeenCalledTimes(1);
    expect(updateDefaultNodePoolSpy).toHaveBeenCalledTimes(1);
  });

  it('does not create a nodePool and throws an error ' +
    'if gcp is specified as the provider', () => {
    const createDynamicNodePool =
      require('../createDynamicNodePool.js');
    mockConfig.provider = 'gcp';
    createDynamicNodePool(undefined, () => {
    });
    expect(global.ActErr).toHaveBeenCalledTimes(1);
  });

  it('does not create a nodePool and throws an error ' +
    'if no master integration exists', () => {
    const createDynamicNodePool =
      require('../createDynamicNodePool.js');
    mockMasterIntegrations = [];
    createDynamicNodePool(undefined, () => {
    });
    expect(global.ActErr).toHaveBeenCalledTimes(1);
  });

  it('does not create a nodePool ' +
    'if one already exists and isMacOSEnabled is false', () => {
    const createDynamicNodePool =
      require('../createDynamicNodePool.js');
    mockNodePools = [{id: 'foo'}];
    const cb = jest.fn();
    createDynamicNodePool(undefined, cb);
    expect(global.ActErr).not.toHaveBeenCalled();
    expect(cb).toHaveBeenCalledTimes(1);
    expect(insertNodePoolSpy).not.toHaveBeenCalled();
  });

  it('does not create a nodePool ' +
    'if one already exists and isMacOSEnabled is true', () => {
    global.config.isMacOSEnabled = true;
    const createDynamicNodePool =
      require('../createDynamicNodePool.js');
    mockNodePools = [{id: 'foo'}];
    const cb = jest.fn();
    createDynamicNodePool(undefined, cb);
    expect(global.ActErr).not.toHaveBeenCalled();
    expect(cb).toHaveBeenCalledTimes(1);
    expect(insertNodePoolSpy).not.toHaveBeenCalled();
  });

  it('does not create a nodePool and throws an error ' +
    'if an invalid provider is configured', () => {
    const createDynamicNodePool =
      require('../createDynamicNodePool.js');
    mockConfig = {
      accountId: 'accountId',
      nodePoolName: 'nodePoolName',
      nodelimit: '42',
      instanceType: 'instanceType',
      vpcId: 'vpcId',
      securityGroupId: 'securityGroupId',
      provider: 'bogus provider',
      keyPairName: 'keyPairName',
      subnetId: 'subnetId',
      region: 'region',
    };
    const cb = jest.fn();
    createDynamicNodePool(undefined, cb);
    expect(global.ActErr).toHaveBeenCalledTimes(1);
    expect(cb).toHaveBeenCalledTimes(1);
    expect(insertNodePoolSpy).not.toHaveBeenCalled();
  });
});
