'use strict';

var self = updateDynamicNodePools;
module.exports = self;

var async = require('async');
var path = require('path');
var fs = require('fs');
var _ = require('underscore');

const {
  MAC_OS_13_5,
  DEFAULT_NODEPOOL_AWS_INTERVAL_MINS,
  KUBERNETES_MASTER_INTEGRATION_NAME
} = require('./constants.js');

function updateDynamicNodePools(args, cb) {
  var bag = {
    integrationConfigDir: 'buildplane-config'
  };

  bag.configPath = path.join(global.config.containerConfigDir,
    bag.integrationConfigDir);

  bag.who = util.format('nodePools|%s', self.name);
  logger.info(bag.who, 'Starting');

  async.series([
      _getConfigs.bind(null, bag),
      _getNodePools.bind(null, bag),
      _getServiceUserId.bind(null, bag),
      _updateNodePools.bind(null, bag),
      _updateSaaSNodePoolsIdleInterval.bind(null, bag)
    ],
    function (err) {
      logger.info(bag.who, 'Completed');
      return cb(err);
    }
  );
}

function _getConfigs(bag, next) {
  var who = bag.who + '|' + _getConfigs.name;
  logger.verbose(who, 'Inside');

  var providerPath = path.join(bag.configPath, 'provider');
  if (!fs.existsSync(providerPath)) {
    logger.debug('No provider found at', providerPath);
    return next();
  }

  bag.provider = fs.readFileSync(providerPath, 'utf-8').trim();

  if (bag.provider === 'aws') {
    var subnetIdPath = path.join(bag.configPath, 'subnetId');
    bag.subnetId = Buffer.from(
      fs.readFileSync(subnetIdPath, 'utf-8')).toString('utf-8').trim();

    const instanceSizePath = path.join(bag.configPath, 'instanceType');
    bag.instanceSize = Buffer.from(
      fs.readFileSync(instanceSizePath, 'utf-8')).toString('utf-8').trim();

  } else if (bag.provider === 'gcp') {
    return next();
  } else if (bag.provider === 'k8s') {
    return next();
  }

  return next();
}

function _getNodePools(bag, next) {
  if (bag.provider !== 'aws') return next();

  var who = bag.who + '|' + _getNodePools.name;
  logger.verbose(who, 'Inside');

  var query = util.format('SELECT id,"providerMetadataPropertyBag","projectIntegrationId" ' +
    'FROM "nodePools" WHERE "isOnDemand"=true');

  global.config.client.query(query,
    function (err, nodePools) {
      if (err)
        return next(
          new ActErr(who, ActErr.DBOperationFailed,
            'Failed to find nodePools with error: ' + util.inspect(err))
        );

      if (_.isArray(nodePools.rows))
        bag.nodePools = nodePools.rows;

      return next();
    }
  );
}

function _getServiceUserId(bag, next) {
  if (bag.provider !== 'aws') return next();

  var who = bag.who + '|' + _getServiceUserId.name;
  logger.verbose(who, 'Inside');

  var query = util.format(`SELECT id FROM "identities" WHERE "name"='Service User'`);

  global.config.client.query(query,
    function (err, identities) {
      if (err || !identities.rows || identities.rows.length === 0)
        return next(
          new ActErr(who, ActErr.DBOperationFailed,
            'Failed to find Service User with error: ' + util.inspect(err))
        );

      bag.serviceUserId = identities.rows[0].id;

      return next();
    }
  );
}

function _updateNodePools(bag, next) {
  if (_.isEmpty(bag.nodePools)) return next();
  var who = bag.who + '|' + _updateNodePools.name;
  logger.verbose(who, 'Inside');

  async.eachSeries(bag.nodePools,
    function (nodePool, done) {
      if (!nodePool.providerMetadataPropertyBag)
        return process.nextTick(done);

      var providerMetadataPropertyBag;
      try {
        providerMetadataPropertyBag =
          JSON.parse(nodePool.providerMetadataPropertyBag);
      } catch (e) {
        logger.debug(who,
          'Unable to parse providerMetadataPropertyBag for nodePool:',
          nodePool.id, e);
        return process.nextTick(done);
      }
      var innerBag = {
        subnetId: bag.subnetId,
        instanceSize: bag.instanceSize,
        provider: bag.provider,
        providerMetadataPropertyBag,
        serviceUserId: bag.serviceUserId,
        nodePool,
      }
      async.series([
          _getProjectIntegration.bind(null, innerBag),
          _updateNodePool.bind(null, innerBag),
        ],
        function (err) {
          if (err) {
            return done(err);
          }
          return process.nextTick(done);
        }
      );
    },
    function (err) {
      return next(err);
    }
  );
}

function _getProjectIntegration(innerBag, next) {
  var query = util.format(`SELECT "createdBy" from "projectIntegrations" where` +
    ` id = '${innerBag.nodePool.projectIntegrationId}'`);

  global.config.client.query(query,
    function (err, integrations) {
      if (err || !integrations.rows || integrations.rows.length !== 1)
        return next(
          new ActErr(ActErr.DBOperationFailed,
            'Failed to get integration for node pool' + innerBag.nodePool.name + ' with error: ' + util.inspect(err))
        );
      innerBag.projectIntegrationCreatedBy = integrations.rows[0].createdBy
      return next();
    }
  );
}

function _updateNodePool(innerBag, next) {
  if (innerBag.provider === 'aws') {
    if ((!innerBag.instanceSize || innerBag.instanceSize === innerBag.providerMetadataPropertyBag.instanceSize) && (!innerBag.subnetId || innerBag.subnetId === innerBag.providerMetadataPropertyBag.subnetId))
      return process.nextTick(next);
    if (innerBag.projectIntegrationCreatedBy !== innerBag.serviceUserId)
      return process.nextTick(next);
    innerBag.providerMetadataPropertyBag.subnetId = innerBag.subnetId;
    innerBag.providerMetadataPropertyBag.instanceSize = innerBag.instanceSize;

  } else if (innerBag.provider === 'gcp') {
    return process.nextTick(next);
  } else if (innerBag.provider === 'k8s') {
    return process.nextTick(next);
  } else {
    return process.nextTick(next);
  }

  var updateStatement = util.format('UPDATE "nodePools" SET ' +
    '"providerMetadataPropertyBag"=\'%s\' WHERE id=%s',
    JSON.stringify(innerBag.providerMetadataPropertyBag), innerBag.nodePool.id);

  global.config.client.query(updateStatement,
    function (err) {
      if (err)
        return next(
          new ActErr(ActErr.DBOperationFailed,
            'Failed to update node pool with error: ' + util.inspect(err))
        );
      return next();
    }
  );
}

function _updateSaaSNodePoolsIdleInterval(bag, next) {
  if (!global.config.isSaas) return next();
  if (!bag.serviceUserId) return next();

  var who = bag.who + '|' + _updateSaaSNodePoolsIdleInterval.name;
  logger.verbose(who, 'Inside');

  var updateStatement = util.format(
    'UPDATE "nodePools" np ' +
    'SET "nodeIdleIntervalInMins" = %s ' +
    'FROM "projectIntegrations" pi ' +
    'WHERE np."projectIntegrationId" = pi.id ' +
    'AND np."isOnDemand" = true ' +
    'AND np."operatingSystem" != \'%s\' ' +
    'AND pi."createdBy" = %s ' +
    'AND pi."masterIntegrationName" != \'%s\'',
    DEFAULT_NODEPOOL_AWS_INTERVAL_MINS, MAC_OS_13_5,
    bag.serviceUserId,
    KUBERNETES_MASTER_INTEGRATION_NAME
  );

  global.config.client.query(updateStatement,
    function (err) {
      if (err)
        return next(
          new ActErr(who, ActErr.DBOperationFailed,
            'Failed to update node pool with error: ' + util.inspect(err))
        );
      return next();
    }
  );
}

if (require.main === module)
  updateDynamicNodePools();