#!/bin/bash
set -e
set -o pipefail

export UPGRADE_VERSION=""
export BUILDPLANE_UPGRADE_VERSION=""
readonly UPDATE_TIME="$(date)"
readonly ENABLE_MIGRATION_FLAG="--enable-vault-migration"
readonly SECRET_STORAGE_FLAG="--secret-storage"
__parse_args() {
  if [[ $# -gt 0 ]]; then
    while [[ $# -gt 0 ]]; do
      ARGUMENT="$1"
      case $ARGUMENT in
        --enable-call-home)
          set_system_config "core.allowCallHome" "true"
          shift
          ;;
        --disable-call-home)
          set_system_config "core.allowCallHome" "false"
          shift
          ;;
        $ENABLE_MIGRATION_FLAG)
          __set_migration_flag
          shift
          ;;
        $SECRET_STORAGE_FLAG)
          local secret="$2"
          validate_nonempty_option "--secret-storage" $secret
          if [[ "$secret" == "access" ]] ; then
            __use_only_access
          else
            log_proc_error "The supported value for $SECRET_STORAGE_FLAG is \"access\""
            exit 1
          fi
          shift
          shift
          ;;
        --build-image-registry-url)
          local registry_url="$2"
          PIPELINES_RUNTIME_LANGUAGEIMAGES_REGISTRYURL="$registry_url"
          local index=0
          local no_image_remaining=false
          while [ $no_image_remaining != true ]; do
            local image=$(yq e ".runtime.languageImages[$index]" $PIPELINES_YAML_CONFIG)
            if [ "$image" == "null" ]; then
              no_image_remaining=true
            else
              set_system_config "runtime.languageImages[$index].registryUrl" "$registry_url" "PIPELINES_RUNTIME_LANGUAGEIMAGES_$index_REGISTRYURL"
              let "index = $index + 1"
            fi
          done
          shift
          shift
          ;;
        *)
          echo "Invalid option: $ARGUMENT"
          print_help
          ;;
      esac
    done
  fi
}

__set_migration_flag() {
  if [[ ! -z  "${PIPELINES_SHARED_ACCESS_ENABLEVAULTTOACCESSMIGRATION}" ]] && [[ "${PIPELINES_SHARED_ACCESS_ENABLEVAULTTOACCESSMIGRATION}" == "true" ]]; then
    log_proc_msg "Vault migration is already enabled in system.yaml, ignoring $ENABLE_MIGRATION_FLAG flag"
    return 0;
  fi
  local enableVaultToAccessMigration="true"
  local shouldJustUpdateAccess="false"
  local shouldReadFromVault="true"
  __set_access_vault_flags "${enableVaultToAccessMigration}" "${shouldJustUpdateAccess}" "${shouldReadFromVault}"
}

__set_default_access_flags(){
  if [[ -z "${PIPELINES_SHARED_ACCESS_ENABLEVAULTTOACCESSMIGRATION}" ]] \
      && [[ -z "${PIPELINES_SHARED_ACCESS_SHOULDJUSTUPDATEACCESS}" ]] \
      && [[ -z "${PIPELINES_SHARED_ACCESS_SHOULDREADFROMVAULT}" ]]; then
        local enableVaultToAccessMigration="false"
        local shouldJustUpdateAccess="false"
        local shouldReadFromVault="true"
        __set_access_vault_flags "${enableVaultToAccessMigration}" "${shouldJustUpdateAccess}" "${shouldReadFromVault}"    
  fi
}

__use_only_access() {
  if __is_vault_disabled; then
    log_proc_msg "Vault is already disabled, ignoring $SECRET_STORAGE_FLAG flag"
    return;
  fi
  if [[ ! -z  "${PIPELINES_SHARED_ACCESS_ENABLEVAULTTOACCESSMIGRATION}" ]] && [[ "${PIPELINES_SHARED_ACCESS_ENABLEVAULTTOACCESSMIGRATION}" == "true" ]]; then
    local enableVaultToAccessMigration="false"
    local shouldJustUpdateAccess="true"
    local shouldReadFromVault="false"
    __set_access_vault_flags "${enableVaultToAccessMigration}" "${shouldJustUpdateAccess}" "${shouldReadFromVault}"
  else
    log_proc_error "Vault should be disabled only after migrating existing data from vault to access, use flag $ENABLE_MIGRATION_FLAG to enable migration during upgrade"
    exit 1;
  fi
}

__set_access_vault_flags() {
  if [[ ! -z "$PIPELINES_SHARED_ARTIFACTORY_BASEURL" ]] && [[ "$PIPELINES_SHARED_ARTIFACTORY_BASEURL" == "https"* ]]; then
    local enableVaultToAccessMigration="$1"
    local shouldJustUpdateAccess="$2"
    local shouldReadFromVault="$3"
    set_system_config "shared.access.enableVaultToAccessMigration" "${enableVaultToAccessMigration}"
    set_system_config "shared.access.shouldJustUpdateAccess" "${shouldJustUpdateAccess}"
    set_system_config "shared.access.shouldReadFromVault" "${shouldReadFromVault}"  
  else
    log_proc_error "To enable $ENABLE_MIGRATION_FLAG or $SECRET_STORAGE_FLAG, the platform should be enabled with tls"
    exit 1;
  fi
}

__is_vault_disabled() {
  if [[ ! -z "${PIPELINES_SHARED_ACCESS_SHOULDREADFROMVAULT}" ]] && [[ "${PIPELINES_SHARED_ACCESS_SHOULDREADFROMVAULT}" == "false" ]]; then
    return 0;
  else
    return 1;
  fi
}

__load_config() {
  log_proc_msg "Loading system configuration"
  bootstrap_platform_config
  load_system_config
  log_system_info
}

__archive_files() {
  log_proc_marker "Backing up config and logs"
  mkdir -p $TMP_DIR/backup
  cp -r $LOGS_DIR $TMP_DIR/backup/logs
  if [ -f $PIPELINES_CONFIG ]; then
    mv $PIPELINES_CONFIG $TMP_DIR/backup/pipelines.env
  fi
  if [ -f $PIPELINES_YAML_CONFIG ]; then
    cp $PIPELINES_YAML_CONFIG $TMP_DIR/backup/system.yaml
  fi
  if [ -f $REGISTRY_CREDS ]; then
    cp $REGISTRY_CREDS $TMP_DIR/backup/registry_creds.json
  fi
  if [ -f $KUBECONFIG ]; then
    cp $KUBECONFIG $TMP_DIR/backup/kubernetes.conf
  fi
  if [ -f $SYSTEM_ENV ]; then
    cp $SYSTEM_ENV $TMP_DIR/backup/environment
  fi

  local backup_directory=$RUNTIME_DIR/archives
  local archive_file="$backup_directory/$TIMESTAMP.tar.gz"

  mkdir -p $backup_directory
  tar -czf $archive_file -C $TMP_DIR/backup .

  local config_backup_count=$PIPELINES_CORE_CONFIGBACKUPCOUNT

  if [ -z "$config_backup_count" ]; then
    config_backup_count=5
  fi

  ls -t $backup_directory | awk "NR>$config_backup_count" | \
    awk -v dir=$backup_directory '{printf("%s/%s\n", dir, $1); }' | \
    xargs rm -f

  rm -r $TMP_DIR/backup

  log_proc_success "Saved previous configuration in $archive_file"
}

__set_upgrade_version() {
  log_proc_marker "Setting upgrade version"
  if [ -f  "$CONTROLPLANEVERSION_FILE" ]; then
    local upgrade_version=$(cat "$CONTROLPLANEVERSION_FILE")
    if [ ! -z "$upgrade_version" ]; then
      UPGRADE_VERSION="$upgrade_version"
    else
      log_proc_error "Unknown upgrade version"
      exit 1
    fi
    local buildplane_upgrade_version=$(cat "$BUILDPLANEVERSION_FILE")
    if [ ! -z "$buildplane_upgrade_version" ]; then
      export BUILDPLANE_UPGRADE_VERSION="$buildplane_upgrade_version"
    else
      log_proc_error "Unknown upgrade version"
      exit 1
    fi
  else
    log_proc_error "Missing version file: $BUILDPLANEVERSION_FILE"
    exit 1
  fi
  log_proc_success "Upgrading from $PIPELINES_CORE_CONTROLPLANEVERSION to $UPGRADE_VERSION"
}

__update_license_hash() {
  log_proc_marker "Validating licenses with artifactory"
  exec_util "ValidateLicenses"

  local license_hash=$(echo $response \
    | jq -r '.licenseHash')
  if [ ! -z "$license_hash" ]; then
    log_proc_msg "Updating license hash"
    set_system_config "shared.artifactory.licenseId" "$license_hash"
  else
    log_proc_error "Missing license hash, exiting"
    exit 1
  fi
}

__update_artifactory_service_id() {
  log_proc_marker "Updating Artifactory service ID"
  exec_util "GetServiceID"

  local artifactory_service_id=$(echo $response \
    | jq -r '.serviceId')
  if [ ! -z "$artifactory_service_id" ]; then
    log_proc_msg "Updating Artifactory service ID"
    set_system_config "shared.artifactory.artifactoryServiceId" "$artifactory_service_id"
  fi
}

__set_os_distribution() {
  if [ -z "$PIPELINES_CORE_OSDISTRIBUTION" ]; then
    set_system_config "core.osDistribution" "$PLATFORM_OSDISTRIBUTION"
  fi
}

__set_rabbitmq_erlangcookie() {
  if [ -z "$PIPELINES_SHARED_MSG_ERLANGCOOKIE" ]; then
    local erlang_cookie=$(uuidgen)
    set_system_config "shared.msg.erlangCookie" "$erlang_cookie"
  fi
}

__upgrade_redis() {
  log_proc_msg "Installing redis"
  source "$INSTALLER_SCRIPTS_DIR/upgradeRedis.sh"
}

__set_redis_password_and_upgrade(){
  if [ -z "$PIPELINES_SHARED_REDIS_PASSWORD" ]; then
    export GLOBAL_PASSWORD=$(uuidgen)
    export PIPELINES_SHARED_REDIS_PASSWORD="$GLOBAL_PASSWORD"
  fi
  if [ ! -z "$PIPELINES_SHARED_REDIS_PASSWORD" ]; then
    set_system_config "shared.redis.password" "$PIPELINES_SHARED_REDIS_PASSWORD"
    __upgrade_redis
  fi
}

__set_db_connection_details() {
  if check_db_variables; then
    if [ -z "$PIPELINES_SHARED_DB_CONNECTIONSTRING" ]; then
      export PIPELINES_SHARED_DB_CONNECTIONSTRING="postgres://$PIPELINES_SHARED_DB_USERNAME:$PIPELINES_SHARED_DB_PASSWORD@$PIPELINES_SHARED_DB_IP:$PIPELINES_SHARED_DB_PORT/$PIPELINES_SHARED_DB_NAME"
    fi
  else
    set_system_config "shared.db.isExternal" "true"
    if [ ! -z "$PIPELINES_SHARED_DB_CONNECTIONSTRING" ]; then
      split_connection_string
    fi
  fi
  __remove_db_connection_string
}

__remove_db_connection_string() {
  yq e "del(.shared.db.connectionString)" -i $PIPELINES_YAML_CONFIG
}

__set_vault_url() {
  if __is_vault_disabled; then
    return 0;
  fi
  if [ -z "$PIPELINES_SHARED_VAULT_URL" ]; then
    set_system_config "shared.vault.url" "http://$PIPELINES_SHARED_VAULT_IP:$PIPELINES_SHARED_VAULT_PORT"
  fi
}

__update_call_home_url() {
  # Update callHomeUrl from bintray.net to bintray.com for versions before 1.5
  if [ "$PIPELINES_CORE_CALLHOMEURL" == "https://api.bintray.net/products/jfrog/pipelines/stats/usage" ]; then
    local major=$(echo "$PIPELINES_CORE_CONTROLPLANEVERSION" | cut -d "." -f 1)
    local minor=$(echo "$PIPELINES_CORE_CONTROLPLANEVERSION" | cut -d "." -f 2)

    if [ $major == 1 ] && [[ $minor -lt 5 ]]; then
      set_system_config "core.callHomeUrl" "https://api.bintray.com/products/jfrog/pipelines/stats/usage"
    fi
  fi
}

__pull_images() {
  log_proc_marker "Pulling images for tag: $UPGRADE_VERSION"
  platform_exec "pull_service_images" "$UPGRADE_VERSION"
}

__stop_stateless_services() {
  log_proc_marker "Stopping stateless services"
  stop_stateless_services
}

__stop_stateful_services() {
  stop_stateful_services
}

__run_migrations() {
  source "$INSTALLER_SCRIPTS_DIR/dbTasks.sh"
  eval exec_db_task "all"
}

__create_internal_pipeline_source() {
  log_proc_marker "Creating internal pipeline source, if not exists"
  exec_util "CreateInternalPipelineSource"
}

__boot_router() {
  log_proc_marker "Booting router"
  source "$INSTALLER_SCRIPTS_DIR/upgradeRouter.sh"
}

__boot_observability() {
  log_proc_marker "Booting observability"
  source "$INSTALLER_SCRIPTS_DIR/upgradeObservability.sh"
}

__boot_api() {
  log_proc_marker "Booting API"
  exec_util "CreateService" "api"
}

__boot_www() {
  log_proc_marker "Booting WWW"
  exec_util "CreateService" "www"
}

__boot_frontend() {
  log_proc_marker "Booting frontend"
  exec_util "CreateService" "frontend"
}

__boot_nodepoolservice() {
  log_proc_marker "Booting NodePoolService"
  exec_util "CreateService" "nodepoolservice"
}

__boot_stepService() {
  log_proc_marker "Booting Step Service"
  exec_util "CreateService" "stepservice"
}

__boot_runService() {
  log_proc_marker "Booting Run Service"
  exec_util "CreateService" "runservice"
}

__boot_logService() {
  log_proc_marker "Booting Log Service"
  exec_util "CreateService" "logservice"
}

__boot_services() {
  log_proc_marker "Booting services"
  boot_stateless_services
}

__boot_analyticsservice() {
  log_proc_marker "Booting Analytics Service"
  exec_util "CreateService" "analyticsservice"
}

__update_release_version() {
  log_proc_marker "All migrations complete, updating release version in config to: $UPGRADE_VERSION"
  if [ -f "$PIPELINES_CONFIG" ]; then
    set_config "PIPELINES_CORE_CONTROLPLANEVERSION" "$UPGRADE_VERSION"
    set_config "PIPELINES_CORE_LASTUPDATETIME" "$UPDATE_TIME"
  fi
  if [ -f "$PIPELINES_YAML_CONFIG" ]; then
    set_system_config "core.controlplaneVersion" "$UPGRADE_VERSION"
    set_system_config "core.lastUpdateTime" "$UPDATE_TIME"
    set_system_config "core.buildplaneVersion" "$BUILDPLANE_UPGRADE_VERSION"
    log_proc_msg "Setting join key value in yaml at shared.security.joinKey with length ${#PIPELINES_SHARED_ROUTER_JOINKEY}"
    set_system_config "shared.security.joinKey" "$PIPELINES_SHARED_ROUTER_JOINKEY"
  fi
}

main() {
  log_proc_marker "Upgrading Pipelines"
  __load_config
  __set_default_access_flags
  __parse_args "$@"
  __load_config
  __archive_files
  __set_upgrade_version
  update_services_list
  __load_config
  __pull_images
  decrypt_secrets
  __update_license_hash
  __update_artifactory_service_id
  __set_os_distribution
  __set_redis_password_and_upgrade
  __set_rabbitmq_erlangcookie
  __set_db_connection_details
  __set_vault_url
  __update_call_home_url
  __stop_stateless_services
  add_language_images
  __run_migrations
  __create_internal_pipeline_source
  __update_release_version
  __stop_stateful_services
  update_queues_list
  create_volume_dirs
  __boot_router
  __boot_observability
  __boot_api
  __boot_www
  __boot_services
  log_proc_marker "All upgrade steps complete for release version: $UPGRADE_VERSION"
}

{
  main "$@"
} 2>&1 | tee "$LOG_FILE" ; ( exit ${PIPESTATUS[0]} )
