#!/bin/bash
set -e
set -o pipefail

export PIPELINES_SHARED_REDIS_IMAGE="$PIPELINES_CORE_REGISTRYURL/$PIPELINES_SHARED_REDIS_IMAGE:$UPGRADE_VERSION"
export COMPONENT="redis"
export REDIS_DATA_DIR="$RUNTIME_DIR/data/$COMPONENT"
export REDIS_CONFIG_DIR="$CONFIG_DIR/$COMPONENT"
export REDIS_COMPOSE="$REDIS_CONFIG_DIR/redis_compose.yml"
export LOGS_FILE="$LOGS_DIR/$COMPONENT.log"

__validate_redis_mounts() {
  log_proc_msg "Validating redis mounts"
  if [ ! -d "$REDIS_DATA_DIR" ]; then
    log_proc_msg "Creating data directory $REDIS_DATA_DIR"
    mkdir -p $REDIS_DATA_DIR
  else
    log_proc_msg "Data directory already present: $REDIS_DATA_DIR"
  fi

  if [ ! -d "$REDIS_CONFIG_DIR" ]; then
    log_proc_msg "Creating config directory $REDIS_CONFIG_DIR"
    mkdir -p $REDIS_CONFIG_DIR
  else
    log_proc_msg "Config directory already present: $REDIS_CONFIG_DIR"
  fi

  mkdir -p $REDIS_CONFIG_DIR/scripts
}
__generate_compose_config() {
  log_proc_msg "Generating redis compose config"

  local compose_template="$INSTALLER_SCRIPTS_DIR/configs/redis_compose.yml.template"
  cp -vr $compose_template $REDIS_COMPOSE

  log_proc_msg "Updating component name"
  sed -i 's#{{COMPONENT}}#'$COMPONENT'#g' $REDIS_COMPOSE

  log_proc_msg "Updating redis image"
  sed -i 's#{{PIPELINES_SHARED_REDIS_IMAGE}}#'$PIPELINES_SHARED_REDIS_IMAGE'#g' $REDIS_COMPOSE

  log_proc_msg "Updating redis port"
  sed -i 's#{{PIPELINES_SHARED_REDIS_PORT}}#'$PIPELINES_SHARED_REDIS_PORT'#g' $REDIS_COMPOSE

  log_proc_msg "Updating redis data directory"
  sed -i 's#{{REDIS_DATA_DIR}}#'$REDIS_DATA_DIR'#g' $REDIS_COMPOSE

  log_proc_msg "Updating redis config directory"
  sed -i 's#{{REDIS_CONFIG_DIR}}#'$REDIS_CONFIG_DIR'#g' $REDIS_COMPOSE

  log_proc_msg "Updating node IP"
  sed -i 's#{{PIPELINES_CORE_INSTALLERIP}}#'$PIPELINES_CORE_INSTALLERIP'#g' $REDIS_COMPOSE

	log_proc_msg "Updating cluster IP"
	if [ -z "$REDIS_CLUSTER_IP" ]; then
		REDIS_CLUSTER_IP=""
	fi
	sed -i 's#{{REDIS_CLUSTER_IP}}#'$REDIS_CLUSTER_IP'#g' $REDIS_COMPOSE
	if [ ! -z "$PIPELINES_SHARED_REDIS_PASSWORD" ]; then
    log_proc_msg "Adding REDIS_PASSWORD to docker-compose.yaml"
    REDIS_PASSWORD_ENV="REDIS_PASSWORD: $PIPELINES_SHARED_REDIS_PASSWORD"
    sed -i "s/^    environment:$/    environment:\n      $REDIS_PASSWORD_ENV/" "$REDIS_COMPOSE"
	fi
}

__stop_redis() {
  log_proc_msg "stopping redis container"
  docker-compose \
    --file $REDIS_COMPOSE down || true
}

__run_redis() {
  docker-compose \
    --file $REDIS_COMPOSE \
    up \
    --detach $COMPONENT

  log_proc_msg "Redis container successfully running"
}

__check_redis() {
  log_proc_msg "Checking redis container status on: $PIPELINES_SHARED_REDIS_IP:$PIPELINES_SHARED_REDIS_PORT"
  check_service_connection "$PIPELINES_SHARED_REDIS_IP" "$PIPELINES_SHARED_REDIS_PORT" "$COMPONENT"
}

main() {
  log_proc_marker "Installing redis"
  __validate_redis_mounts
  __generate_compose_config
  __stop_redis
  __run_redis
  __check_redis
}

main
