#!/bin/bash -e

# Helper methods ##########################################
###########################################################

export PLATFORM_DEPENDENCIES_DIR="$DEPENDENCIES_DIR/x86_64"

__install_curl() {
  ################## Install curl  #######################################
  if type curl &> /dev/null && true; then
    log_proc_msg "'curl' already installed"
    local required_curl_version=7.29.0
    local installed_curl_version=$(curl --version | head -n 1 | cut -d ' ' -f 2)

    check_semver $installed_curl_version
    log_proc_msg "Found curl version $installed_curl_version"

    if compare_semver $installed_curl_version $required_curl_version; then
      log_proc_error "The installed version of curl is not supported. curl should be at least version $required_curl_version"
      exit 1
    fi
  else
    log_proc_msg "Installing 'curl'"
    local curl_rpm="$PLATFORM_DEPENDENCIES_DIR/yum/curl-7.29.0-59.el7.x86_64.rpm"
    rpm -i --force "$curl_rpm"
  fi
}

__install_nc() {
  ################## Install nc  #######################################
  if type nc &> /dev/null && true; then
    log_proc_msg "'nc' already installed"
  else
    log_proc_msg "Installing 'nc'"
    local nc_zip="$PLATFORM_DEPENDENCIES_DIR/yum/nmap-ncat-6.40-16.tar.gz"
    local nc_yum="$TMP_DIR/nc"
    mkdir -p "$nc_yum"
    tar -xvzf "$nc_zip" -C "$nc_yum"
    pushd $nc_yum
    rpm -i --force *.rpm
    popd
  fi
}

__install_psql() {
  if type psql &> /dev/null && true; then
    log_proc_msg "'psql' already installed"
    local required_psql_version=12.9
    local installed_psql_version=$(psql --version | cut -d ' ' -f 3)

    check_semver $installed_psql_version
    log_proc_msg "Found psql version $installed_psql_version"

    if compare_semver $installed_psql_version $required_psql_version; then
      log_proc_error "The installed version of psql is not supported. psql should be at least version $required_psql_version"
      exit 1
    fi
  else
    log_proc_msg "Installing psql client $required_psql_version"
    local psql_archive="$PLATFORM_DEPENDENCIES_DIR/yum/postgresql12-12.9.tar.gz"
    local psql_dnf="$TMP_DIR/psql"
    mkdir -p "$psql_dnf"
    tar -xvf "$psql_archive" -C "$psql_dnf"
    pushd "$psql_dnf"
    rpm -i --force *.rpm
    popd
    psql --version
    log_proc_msg "psql $required_psql_version successfully installed"
  fi
}

__install_docker() {
  ################## Install Docker  #####################################
  if type docker &> /dev/null && true; then
    log_proc_msg "'docker' already installed"
    local required_docker_version=18.03
    local installed_docker_version=$(docker version --format {{.Server.Version}})

    check_semver $installed_docker_version
    log_proc_msg "Found docker version $installed_docker_version"

    if compare_semver $installed_docker_version $required_docker_version; then
      log_proc_error "The installed version of docker is not supported. Docker should be at least version $required_docker_version"
      exit 1
    fi
  else
    log_proc_error "'docker' not installed. Please install docker to continue installation"
    exit 1
  fi
}

install_dependencies() {
  log_proc_marker "Installing dependencies"
  __install_docker
  if [ "$EUID" -ne 0 ]; then
    log_proc_warn "Skipping dependency installation since user doesn't have admin privileges"
  else
    __install_curl
    __install_nc
    __install_psql
  fi
}

pull_service_images() {
  if [ "$SKIP_IMAGE_PULL" == true ]; then
    return
  fi

  log_proc_marker "Pulling service images"
  local image_tag="$PIPELINES_CORE_CONTROLPLANEVERSION"
  if [ ! -z "$1" ]; then
    image_tag="$1"
  fi

  if check_db_variables; then
    # Installing database
    docker pull "$PIPELINES_CORE_REGISTRYURL/$PIPELINES_SHARED_DB_IMAGE:$image_tag"
  fi

  docker pull "$PIPELINES_CORE_REGISTRYURL/$PIPELINES_SHARED_ROUTER_IMAGE:$image_tag"
  docker pull "$PIPELINES_CORE_REGISTRYURL/$PIPELINES_SHARED_VAULT_IMAGE:$image_tag"
  docker pull "$PIPELINES_CORE_REGISTRYURL/$PIPELINES_SHARED_MSG_IMAGE:$image_tag"
  docker pull "$PIPELINES_CORE_REGISTRYURL/$PIPELINES_SHARED_REDIS_IMAGE:$image_tag"
  docker pull "$PIPELINES_CORE_REGISTRYURL/$PIPELINES_SHARED_OBSERVABILITY_IMAGE:$image_tag"
  docker pull "$PIPELINES_CORE_REGISTRYURL/$PIPELINES_CORE_INSTALLERIMAGE:$image_tag"
  docker pull "$PIPELINES_CORE_REGISTRYURL/$PIPELINES_CORE_SERVICES_API_IMAGE:$image_tag"
  docker pull "$PIPELINES_CORE_REGISTRYURL/$PIPELINES_CORE_SERVICES_WWW_IMAGE:$image_tag"
  docker pull "$PIPELINES_CORE_REGISTRYURL/$PIPELINES_CORE_SERVICES_FRONTEND_IMAGE:$image_tag"
  docker pull "$PIPELINES_CORE_REGISTRYURL/$PIPELINES_CORE_SERVICES_NODEPOOLSERVICE_IMAGE:$image_tag"
  docker pull "$PIPELINES_CORE_REGISTRYURL/$PIPELINES_CORE_SERVICES_PIPELINESYNC_IMAGE:$image_tag"
  docker pull "$PIPELINES_CORE_REGISTRYURL/$PIPELINES_CORE_SERVICES_STEPSERVICE_IMAGE:$image_tag"
  docker pull "$PIPELINES_CORE_REGISTRYURL/$PIPELINES_CORE_SERVICES_RUNSERVICE_IMAGE:$image_tag"
  docker pull "$PIPELINES_CORE_REGISTRYURL/$PIPELINES_CORE_SERVICES_LOGSERVICE_IMAGE:$image_tag"
  docker pull "$PIPELINES_CORE_REGISTRYURL/$PIPELINES_CORE_SERVICES_ANALYTICSSERVICE_IMAGE:$image_tag"
  
  log_proc_marker "All images pulled successfully"
}
