#!/bin/bash

# Configuration
artContainerName="artifactory"
artPort="8081"
artRouterPort="8082"

printHelp() {
  echo "
  Usage:
    testInstallerLocally.sh <Public Host IP> <Path to License File> [options]

  Example:
    testInstallerLocally.sh 10.0.0.22 /workspace/licenses/mylicense.txt
      --arturl https://entplus.jfrog.io/artifactory --artuser user --artpassword password
      --packagepath pipelines-installers/installer/pipelines-0.0.0-m000.tar.gz

  Options:
    --controlplane-version    Control plane version. Default: 0.0.0-m000
    --buildplane-version      Build plane version. Default: 0.0.0-m000
    --art-docker-image        Artifactory docker image to use.
                              Default: docker.jfrog.io/jfrog/artifactory-pro:latest
    --arturl                  Artifactory service to download installer package from.
                              Default: https://entplus.jfrog.io/artifactory
    --artuser                 Artifactory user
    --artpassword             Artifactory password
    --packagepath             Path to installer package
    --db-connection-string    Provide the postgres db details
                              postgres://postgres_user:postgres_password@postgres_host:5432/dbName
    --devmode                 Skips installer package download step and use local installer
                              instead. Default: false
"

  exit 1
}

parseArgs() {
  shift
  shift
  if [[ $# -gt 0 ]]; then
    while [[ $# -gt 0 ]]; do
      ARGUMENT="$1"

      case $ARGUMENT in
        --controlplane-version)
          CONTROL_PLANE_VERSION=$2
          shift
          shift
          ;;
        --buildplane-version)
          BUILD_PLANE_VERSION=$2
          shift
          shift
          ;;
        --art-docker-image)
          ART_DOCKER_IMAGE=$2
          shift
          shift
          ;;
        --arturl)
          ART_URL=$2
          shift
          shift
          ;;
        --artuser)
          ART_USER=$2
          shift
          shift
          ;;
        --artpassword)
          ART_PASSWORD=$2
          shift
          shift
          ;;
        --packagepath)
          PACKAGE_PATH=$2
          shift
          shift
          ;;
        --db-connection-string)
          DB_CONNECTIONSTRING=$2
          shift
          shift
          ;;
        --devmode)
          DEVMODE=true
          shift
          ;;
        *)
          echo "Invalid option: $ARGUMENT"
          printHelp
          ;;
      esac
    done
  fi
}

validateArgs() {
  if [[ "$DEVMODE" != true ]]; then
    case $PACKAGE_PATH in *.tar.gz) PACKAGE_TYPE="tar.gz";; esac
    case $PACKAGE_PATH in *.deb) PACKAGE_TYPE="deb";; esac
    case $PACKAGE_PATH in *.rpm) PACKAGE_TYPE="rpm";; esac

    if [[ "$PACKAGE_TYPE" != "tar.gz" && "$PACKAGE_TYPE" != "deb" && "$PACKAGE_TYPE" != "rpm" ]]; then
      echo "Failed to get package type from package path: $PACKAGE_PATH. Should end on 'tar.gz|deb|rpm'"
      printHelp
    fi
  fi
}

setDefaults() {
  if [[ -z $CONTROL_PLANE_VERSION ]]; then
    CONTROL_PLANE_VERSION="0.0.0-m000"
  fi

  if [[ -z $BUILD_PLANE_VERSION ]]; then
    BUILD_PLANE_VERSION="0.0.0-m000"
  fi

  if [[ -z $ART_URL ]]; then
    ART_URL="https://entplus.jfrog.io/artifactory"
  fi

  if [[ -z $ART_DOCKER_IMAGE ]]; then
    ART_DOCKER_IMAGE="docker.jfrog.io/jfrog/artifactory-pro:latest"
  fi

}

removeContainerIfPresent() {
  echo "Removing container $1 if present"
  running_container=$(docker ps -aq --filter "Name=$1*")
  if [[ "$running_container" ]]; then
    echo $running_container | xargs docker rm -f
  fi
}

getHttpStatus() {
  url=$1
  curl -s -o /dev/null -w "%{http_code}" $url
}

checkdependencies() {
  if ! which docker >/dev/null; then
    echo "docker not installed!"
    exit 7
  fi
  if ! which docker-compose >/dev/null; then
    echo "docker-compose not installed!"
    exit 7
  fi
  if ! which jq >/dev/null; then
    echo "jq not installed!"
    exit 7
  fi
  if ! which curl >/dev/null; then
    echo "curl not installed!"
    exit 7
  fi
  if ! which jfrog >/dev/null; then
    echo "jfrog not installed!"
    exit 7
  fi
}

fetchInstaller() {
  echo "Fetching Installer Package #############################"
  if [[ "$DEVMODE" == true ]]; then
    echo "Devmode is on. Using local installer."
    installerScriptFolder=$(cd $scriptFolder/.. && pwd)
  else
    installerPackageUrl="$ART_URL/$PACKAGE_PATH"
    echo "Downloading package from: $installerPackageUrl"
    curl -u $ART_USER:$ART_PASSWORD $installerPackageUrl -o installerpackage
    echo $?

    if [[ "$?" != "0" ]]; then
      echo "Failed to fetch installer package"
      exit 5
    fi

    if [[ "$PACKAGE_TYPE" == "tar.gz" ]]; then
      tar -xvzf installerpackage
      if [[ "$?" != "0" ]]; then
        echo "Failed to extract package content"
        exit 6
      fi
      installerScriptFolder=$(cd ./pipelines-$CONTROL_PLANE_VERSION && pwd)
    elif [[ "$PACKAGE_TYPE" == "deb" ]]; then
      dpkg -i installerpackage
      if [[ "$?" != "0" ]]; then
        echo "Failed to install deb package"
        exit 6
      fi
      installerScriptFolder=/opt/jfrog/pipelines/installer
    elif [[ "$PACKAGE_TYPE" == "rpm" ]]; then
      rpm -Uvh installerpackage
      if [[ "$?" != "0" ]]; then
        echo "Failed to install rpm package"
        exit 6
      fi
      installerScriptFolder=/opt/jfrog/pipelines/installer
    fi
  fi

  echo "Installer script is at: $installerScriptFolder"
  echo ""
}

installArtifactory() {
  echo "Booting Artifactory Container ##########################"
  removeContainerIfPresent $artContainerName
  echo "Using image $ART_DOCKER_IMAGE"
  docker pull $ART_DOCKER_IMAGE
  docker run --name $artContainerName -d -p $artRouterPort:$artRouterPort -p $artPort:$artPort $ART_DOCKER_IMAGE
  echo "Artifactory container is running!"
  echo ""
}

waitForArtifactory() {
  echo "Waiting for Artifactory to be Ready ####################"
  echo "Healthcheck url: $artHealthcheckUrl"
  status=$(getHttpStatus $artHealthcheckUrl)
  maxAttempts=30
  attempt=0
  while [[ "$status" -ne "200" && $attempt -lt $maxAttempts ]]; do
    attempt=$((attempt+1))
    sleep 10
    status=$(getHttpStatus $artHealthcheckUrl)
    echo "Status: $status. Attempt: $attempt/$maxAttempts"
  done
  if [[ "$status" == "200" ]]; then
    echo "Artifactory is ready!"
  else
    echo "Timeout waiting for Artifactory to be ready"
    exit 2
  fi
  echo ""
}

installLicense() {
  echo "Installing License #####################################"
  echo "License installation endpoint url: $postLicenseUrl"
  licenseContent=$(cat $LICENSE_PATH)
  licensePostPayload="{ \"licenseKey\": \"$licenseContent\" }"
  command="curl -s -u admin:password -X POST -H \"Content-Type:application/json\" $postLicenseUrl -d '$licensePostPayload'"
  response=$(eval $command)
  status=$(echo $response | jq .status)
  if [[ "$status" != "200" ]]; then
      echo "Failed to install license"
      echo "Artifactory response: $response"
      exit 3
  else
      echo "License installed!"
      echo ""
  fi
}

getArtifactoryKeys() {
  echo "Fetching Artifactory Keys from Container ###############"
  joinKey=$(docker exec $artContainerName cat var/etc/security/join.key)
  masterKey=$(docker exec $artContainerName cat var/etc/security/master.key)
  echo ""
}

runPipelinesInstaller() {
  echo "Running pipelines installer ############################"

  echo $BUILD_PLANE_VERSION > bin/BUILD_PLANE_VERSION.txt
  echo $CONTROL_PLANE_VERSION > bin/CONTROL_PLANE_VERSION.txt

  cd $installerScriptFolder

  installCommand="./pipelines install                      \
    --base-url http://$HOST_IP:$artRouterPort              \
    --base-url-ui http://$HOST_IP:$artRouterPort           \
    --installer-ip $HOST_IP                                \
    --api-url http://$HOST_IP:$artRouterPort/pipelines/api \
    --image-registry-url docker.jfrog.io                   \
    --devmode                                             \
    --artifactory-joinkey ${joinKey}                      \
    --artifactory-masterkey ${masterKey}"

  if [ ! -z $DB_CONNECTIONSTRING ]; then
    installCommand="${installCommand} --db-connection-string $DB_CONNECTIONSTRING"
  fi
  eval "$installCommand"
  installerExitCode=$?
  if [[ "$installerExitCode" != "0" ]]; then
    echo "Installer failed. Exit code: $installerExitCode"
    exit $installerExitCode
  fi

  echo "Pipelines installed!"
  echo ""
}

runApiHealthcheck() {
  echo "Checking if API is healthy #############################"
  echo "Healthcheck url: $apiHealthcheckUrl"
  status=$(getHttpStatus $apiHealthcheckUrl)
  if [[ "$status" -ne "200" ]]; then
    echo "Pipelines API is not healthy. Got status: $status"
    exit 4
  fi

  echo "Pipelines API is healthy!"
  echo "Base UI url: http://$HOST_IP:$artRouterPort"
  echo ""
}

main() {
  HOST_IP=$1
  LICENSE_PATH=$2
  CONTROL_PLANE_VERSION=""
  BUILD_PLANE_VERSION=""
  PACKAGE_PATH=""
  PACKAGE_TYPE=""
  ART_DOCKER_IMAGE=""
  ART_URL=""
  ART_USER=""
  ART_PASSWORD=""
  DEVMODE=false

  if [[ -z $HOST_IP || -z $LICENSE_PATH ]]; then
    printHelp
  fi

  parseArgs $@
  setDefaults
  validateArgs

  scriptFolder="$( cd "$( dirname "${BASH_SOURCE[0]}" )" &> /dev/null && pwd )"
  artHealthcheckUrl="http://$HOST_IP:$artRouterPort/router/api/v1/system/health"
  postLicenseUrl="http://$HOST_IP:$artPort/artifactory/api/system/licenses"
  apiHealthcheckUrl="http://$HOST_IP:$artRouterPort/pipelines/api"
  joinKey=""
  masterKey=""
  installerScriptFolder=""

  checkdependencies
  fetchInstaller
  installArtifactory
  waitForArtifactory
  installLicense
  getArtifactoryKeys
  runPipelinesInstaller
  runApiHealthcheck
}

main $@

